package archstudio.comp.cake;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class AddDialog extends JFrame implements ActionListener{
	static int WIDTH = 200;
	static int HEIGHT = 320;
	
	JLabel nameLabel;
	JTextField nameField;
	JLabel typeLabel;
	AddDialogRadioButtons radioButtons;
	JButton okButton;
	JButton cancelButton;
	CheckNodeTreeExample tree;
	
	public AddDialog(CheckNodeTreeExample newTree){
		super("Add a Layer");
		tree = newTree;
		
		//The stuff that goes in the dialog
		radioButtons = new AddDialogRadioButtons();
		
		//Text Panel Stuff
		nameLabel = new JLabel("Layer Name:");
		nameField = new JTextField("New Layer", 15);
		nameField.setColumns(15);
		nameField.setPreferredSize(new Dimension(80, 30));
		JPanel blankPanel = new JPanel();
		typeLabel = new JLabel("Layer Type:");
		
		JPanel textPanel = new JPanel();		
		textPanel.setLayout(new GridLayout(4,1));
		textPanel.add(nameLabel);
		textPanel.add(nameField);
		textPanel.add(blankPanel);
		textPanel.add(typeLabel);
		
		//Button Panel Stuff
		okButton = new JButton ("Ok");
		okButton.addActionListener(this);
		cancelButton = new JButton ("Cancel");
		cancelButton.addActionListener(this);
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout());
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);
		
		//Prefered Sizes
		textPanel.setPreferredSize(new Dimension(180, 120));
		radioButtons.setPreferredSize(new Dimension(180, 160));
		buttonPanel.setPreferredSize(new Dimension(180, 40));
		
		//Adding it all in
		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new BorderLayout());
		centerPanel.add(textPanel, BorderLayout.NORTH);
		centerPanel.add(radioButtons, BorderLayout.CENTER);
		centerPanel.add(buttonPanel, BorderLayout.SOUTH);
		centerPanel.setPreferredSize(new Dimension(180, 320));
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(centerPanel, BorderLayout.CENTER);
		
		blankPanel = new JPanel();
		blankPanel.setPreferredSize(new Dimension(10, 320));
		getContentPane().add(blankPanel, BorderLayout.EAST);
		getContentPane().add(blankPanel, BorderLayout.WEST);
		
		
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation(screen.width/2 - WIDTH/2, screen.height/2 - HEIGHT/2);
		
		setSize(WIDTH,HEIGHT);
		setVisible(true);
	}
	
	public void actionPerformed(ActionEvent ae){
		if(ae.getSource() == cancelButton){
			dispose();
		}
		else if(ae.getSource() == okButton){
			String newName = nameField.getText();
			int newType = radioButtons.getType();
			System.out.println("Adding layer called "+newName+" of type "+newType);
			tree.modelAddLayer(newName, newType);
			dispose();
		}
	}
}

class AddDialogRadioButtons extends JPanel implements ActionListener{
	
	int COMPONENT_LAYER = ICakeModelWrapper.COMPONENT_LAYER;
	int CONNECTOR_LAYER = ICakeModelWrapper.CONNECTOR_LAYER;
	int LINK_LAYER = ICakeModelWrapper.LINK_LAYER;
	int ANNOTATION_LAYER = ICakeModelWrapper.ANNOTATION_LAYER;
	static String componentString = "Component Layer";
	static String connectorString = "Connector Layer";
	static String linkString = "Link Layer";
	static String annotationString = "Annotation Layer";
	
	int type = COMPONENT_LAYER;
	
	public AddDialogRadioButtons(){				
		JRadioButton componentButton = new JRadioButton(componentString);
		componentButton.setMnemonic(KeyEvent.VK_C);
		componentButton.setActionCommand(componentString);
		componentButton.setSelected(true);
		
		JRadioButton connectorButton = new JRadioButton(connectorString);
		connectorButton.setMnemonic(KeyEvent.VK_O);
		connectorButton.setActionCommand(connectorString);
		
		JRadioButton linkButton = new JRadioButton(linkString);
		linkButton.setMnemonic(KeyEvent.VK_L);
		linkButton.setActionCommand(linkString);
		
		JRadioButton annotationButton = new JRadioButton(annotationString);
		annotationButton.setMnemonic(KeyEvent.VK_L);
		annotationButton.setActionCommand(annotationString);
		
		//Group the radio buttons.
		ButtonGroup group = new ButtonGroup();
		group.add(componentButton);
		group.add(connectorButton);
		group.add(linkButton);
		group.add(annotationButton);
		
		componentButton.addActionListener(this);
		connectorButton.addActionListener(this);
		linkButton.addActionListener(this);
		annotationButton.addActionListener(this);
			
		setLayout(new GridLayout(0, 1));
		add(componentButton);
		add(connectorButton);
		add(linkButton);
		add(annotationButton);
	}

	public int getType(){
		return type;
	}
	
	public void actionPerformed(ActionEvent ae){
		String result = ae.getActionCommand();
		if((result.compareTo(componentString)) == 0)
			type = COMPONENT_LAYER;
		if((result.compareTo(connectorString)) == 0)
			type = CONNECTOR_LAYER;
		if((result.compareTo(linkString)) == 0)
			type = LINK_LAYER;
		if((result.compareTo(annotationString)) == 0)
			type = ANNOTATION_LAYER;
	}
}

		