/*
 * Created on May 29, 2004
 *
 */
package archstudio.comp.cake;

import java.awt.*;
import java.awt.event.MouseEvent;

import archstudio.comp.archipelago.ThingIDMap;
import c2.util.UIDGenerator;
import edu.uci.ics.bna.*;
import edu.uci.ics.xarchutils.*;


/**
 * @author Chris
 *
 */
public class AddEntityOnClickLogic extends CakeThingLogicAdapter
{	
	private XArchFlatInterface xarch;
	private ObjRef modelXArch;
	private CheckNodeTreeExample checkNodeTree;
	private ThingIDMap thingIDMap;
	private InterfaceMap intfMap;
	private VersionColorMap versionColorMap;	
	
	// Used to cache the top-level arch references
	private ObjRef archStructure;
	private ObjRef archAnnotation;
	
	private ObjRef typesContext;
	private ObjRef instanceContext;
	private ObjRef annotationsContext;
	private ObjRef layersContext;
	
	// temporary storage of link until we are done adding it
	protected CakeLinkThing linkThing;
	protected ObjRef        linkRef;
	//protected ObjRef 		linkContainingElement;
	public static NoThing 	linkParent;
	public static NoThing	brickParent;
	public static NoThing	annotationParent;
	
	protected ModeTrackingLogic modeTrackingLogic;
	//protected BNAComponent bnaComponent;
	
	public AddEntityOnClickLogic(XArchFlatInterface xarch, ObjRef modelXArch, CheckNodeTreeExample checkNodeTree,
		BNAComponent bnaComp, ThingIDMap thingIDMap, InterfaceMap iMap, VersionColorMap versionColorMap,
	    ModeTrackingLogic modeTrackingLogic)
	{
		this.xarch = xarch;
		this.modelXArch = modelXArch;
		this.checkNodeTree = checkNodeTree;
		this.thingIDMap = thingIDMap;
		this.intfMap = iMap;	
		this.archStructure = null;	
		this.versionColorMap = versionColorMap;
		this.modeTrackingLogic = modeTrackingLogic;
		this.bnaComponent = bnaComp;
		
		//do this only once
		linkParent = new NoThing("Link Parent");
		brickParent = new NoThing("Brick Parent");
		annotationParent = new NoThing("Annotation Parent");
		BNAModel m = bnaComponent.getModel();
		m.addThing(linkParent);
		m.addThing(brickParent);
		m.addThing(annotationParent);
	
		 m.stackAbove(brickParent, linkParent);
		 // annotation should be the top most element
		 m.stackAbove(annotationParent, brickParent);
		
		linkThing = null;
		linkRef = null;
		
		typesContext = xarch.createContext(modelXArch, "Types");
		instanceContext = xarch.createContext(modelXArch, "Instance");
		annotationsContext = xarch.createContext(modelXArch, "Annotations");
		layersContext = xarch.createContext(modelXArch, "Layers");
	}
	
	public void mouseClicked(Thing t, MouseEvent evt, int worldX, int worldY)
	{
		// we can't add if we aren't in Add mode
		if(modeTrackingLogic.getMode() != Palette.ADD_MODE)
		{
			return;
		}
		
		BNAComponent c = bnaComponent;
		if(evt.getButton() == MouseEvent.BUTTON1)
		{				
			// Add an entity to the currently selected layer
			int layerType = checkNodeTree.getSelectedLayerType();
			String layerID = checkNodeTree.getSelectedLayerID();
			
			if(!checkNodeTree.getIsEditable(layerID))
			{
				// the layer isn't editable, so don't add anything
				return;
			}
			ObjRef layer = xarch.getByID(modelXArch, layerID);
		
			if(layer == null || checkNodeTree.getSelectedIsVersion())
			{
				// A layer is not currently selected so don't try to add anything
				return;
			}
			
			//	get the name of the layer
			ObjRef nameRef = (ObjRef)xarch.get(layer, "Name");
			String layerName = (String)xarch.get(nameRef, "Value");

			ObjRef versionRef = xarch.getParent(layer);
			String version = (String)xarch.get(versionRef, "Id");
			
			// Figure out what type of layer we are on and hence what kind of entity to add
			ObjRef containingElement = null;
			switch(layerType)
			{
				case ICakeModelWrapper.COMPONENT_LAYER:
					if(t != null)
					{
						return;
					}						
						
					// Create component in structure
					containingElement = addComponentToLayer(layer);
					
					// Add component to BNA model
					CakeComponentThing compt = new CakeComponentThing(version, layerName);											
					compt.setColor(versionColorMap.getColor(version));
					compt.setBoundingBox(worldX, worldY, 100, 100);
					c.getModel().addThing(compt, brickParent);
					
					// Add to ThingIDMap						
					thingIDMap.mapRefToID(containingElement, compt.getID());
					
					break;
				case ICakeModelWrapper.CONNECTOR_LAYER:
					if(t != null)
					{
						return;
					}
					
					containingElement = addConnectorToLayer(layer);
					
					// Add connector to BNA model
					CakeConnectorThing connt = new CakeConnectorThing(version, layerName);
					connt.setColor(versionColorMap.getColor(version));
					connt.setBoundingBox(worldX, worldY, 150, 50);
					c.getModel().addThing(connt, brickParent);
					
					thingIDMap.mapRefToID(containingElement, connt.getID());
					break;
				case ICakeModelWrapper.LINK_LAYER:
					
					if(linkThing == null)
					{
						if(t != null && !(t instanceof CakeInterfaceThing))
						{
							// we clicked on another element, just ignore!
							return;
						}
						// starting brand newlink
						containingElement = addLink(layer);
						linkRef = CakeUtils.resolveXMLLink(xarch, modelXArch,
							containingElement);
							
						// create the link thing
						linkThing = new CakeLinkThing(version, layerName);
	
						thingIDMap.mapRefToID(containingElement, linkThing.getID());
						c.getModel().addThing(linkThing, linkParent);
						
						//linkThing.setSelected(true);
						//linkThing.setColor(Color.GREEN);
						addEndPoint(t, worldX, worldY, true);
						
					}
					else
					{
						if(t != null && !(t instanceof CakeInterfaceThing))
						{
							if(t != linkThing)
							{
								// we clicked on another element, just ignore!
								return;
							}
						}
						  
						// finishing a previous link
						addEndPoint(t, worldX, worldY, false);
						//linkThing.setSelected(false);
						linkThing.setColor(versionColorMap.getColor(version));
						
						// get ready for the next set of adds
						linkThing = null;
						linkRef = null;
					}
					
					break;
				case ICakeModelWrapper.ANNOTATION_LAYER:
					// If something is clicked on but is not something that can be annotated, don't do anything.
					if(t != null && !isThingAnnotatable(t))
					{
						return;
					}
					
					ObjRef annotationRef = addAnnotationToXArch();
					containingElement = addAnnotationToLayer(annotationRef, layer);
	
					// Add annotation to BNAModel
					CakeAnnotationThing annot = new CakeAnnotationThing(version, layerName);
					annot.setColor(versionColorMap.getColor(version));
					annot.setX(worldX);
					annot.setY(worldY);
					c.getModel().addThing(annot, annotationParent);
					thingIDMap.mapRefToID(containingElement, annot.getID());

					// If the thing clicked on is annotatable set it to be a sticky-annotation
					if(isThingAnnotatable(t))
					{	
						LayerableBox layerThing;
						try
						{						
							layerThing = (LayerableBox)t;
						}
						catch(ClassCastException cce)
						{
							System.err.println("Warning; ID/type mismatch.");
							cce.printStackTrace();
							return;
						}
						
						// Position the annotation to the side of the thing it's annotating
						annot.setX(layerThing.getX2() + 20);										
						
						String indicatedThingID = t.getID(); 
						annot.setIndicatorThingId(indicatedThingID);
						
						ObjRef connectedElementRef = thingIDMap.getXArchRef(indicatedThingID);
												
						// Anchor on annotation is to the containingElement of the annotated element.
						ObjRef anchor = CakeUtils.createXMLLink(xarch, instanceContext, (String)xarch.get(connectedElementRef, "Id"));												
						xarch.set(annotationRef, "Anchor", anchor);
					}																				
					
					break;
			}										
		}
	}
	
	public void mouseMoved(Thing t, MouseEvent evt, int worldX, int worldY)
	{
		if(linkThing != null)
		{
			//System.out.println("Moving endpoint!");
			// make sure we are in the process of adding a new link!
			int layerType = checkNodeTree.getSelectedLayerType();
			if(layerType != ICakeModelWrapper.LINK_LAYER)
			{
				// its no longer the link layer anymore, oops
				// we don't want to complete this link I guess...
				// bnaComponent.getModel().removeThing(linkThing);
				
				ObjRef containingElement = thingIDMap.getXArchRef(linkThing.getID());
				
				cancelAddLink(linkRef, containingElement);
				// hm.. no way to remove something from the map...
				linkThing = null;
				linkRef = null;
			}
			else
			{
				// else move the point to where the mouse is
				linkThing.setPointAt(new Point(worldX, worldY), 
					linkThing.getNumPoints() - 1);
			}
		}
	}
	
	public void bnaModelChanged(BNAModelEvent evt)
	{
		Thing t = evt.getTargetThing();
		if((t != null) && (t instanceof Palette))
		{
			Palette palette = (Palette)t;
			ThingEvent thingEvent = evt.getThingEvent();
			if(thingEvent != null)
			{
				// find out the name of property that was changed
				String propertyName = thingEvent.getPropertyName();
				if(propertyName != null)
				{
					if(propertyName.equals(Palette.MODE_PROPERTY_NAME))
					{
						int mode = palette.getMode();
						if(linkThing != null && mode != Palette.ADD_MODE)
						{
							//its no longer in the add mode, so cancel it
							// we don't want to complete this link I guess...
							bnaComponent.getModel().removeThing(linkThing);
	
							ObjRef containingElement = thingIDMap.getXArchRef(linkThing.getID());
	
							cancelAddLink(linkRef, containingElement);
							// hm.. no way to remove something from the map...
							linkThing = null;
							linkRef = null;
						}
					}
				}
			}
		}
	}
	
	// This function cancels the addition of a link, and deletes the partial
	// link created in the xADL document.
	private void cancelAddLink(ObjRef linkRef, ObjRef containingElement)
	{
		ObjRef layer = xarch.getParent(containingElement);
		xarch.remove(layer, "ContainingElement", containingElement);
		
		ObjRef archStruct = xarch.getParent(linkRef);
		xarch.remove(archStruct, "Link", linkRef);
	}
	
	private void addEndPoint(Thing t, int worldX, int worldY, boolean firstEndPoint)
	{
		ObjRef endPointRef = null;
		if(t == null || !(t instanceof CakeInterfaceThing))
		{
			Point p = new Point(worldX, worldY);
			// didn't select a interface thing to link to
			if(firstEndPoint)
			{
				linkThing.addPoint(p);
				// add a second dummy point
				linkThing.addPoint(new Point(worldX + 1, worldY + 1));
			}	
			else
			{
				linkThing.setPointAt(p, linkThing.getNumPoints() - 1);
			}
		}
		else
		{
			CakeInterfaceThing intfThing = (CakeInterfaceThing)t;
			
			String intfID = t.getID();
			//System.out.println("Interface ID: " + intfID);
			
			// the link should be put on some interface
			Rectangle stickyBox = intfThing.getStickyBox();
			Point p = new Point(stickyBox.x, stickyBox.y);
			
			
			if(firstEndPoint)
			{
				//System.out.println("Setting the first endpoint");
				linkThing.addPoint(p);
				//add a second dummy point
				linkThing.addPoint(new Point(worldX + 1, worldY + 1));
				linkThing.setFirstEndpointStuckToID(intfID);
			}
			else
			{
				//System.out.println("Setting the second endpoint");
				linkThing.setPointAt(p, linkThing.getNumPoints() - 1);
				linkThing.setSecondEndpointStuckToID(intfID);
			}			
			endPointRef = CakeUtils.getInterfaceRef(intfThing, thingIDMap, intfMap);
		}
		//addEndPointRef(linkRef, endPointRef);
	}
	
	/**
	 * @param linkRef
	 * @param endPointRef
	 */
	private void addEndPointRef(ObjRef linkRef, ObjRef endPointRef)
	{
		ObjRef point = xarch.create(instanceContext, "Point");
		String targetID = null;
		if(endPointRef != null)
		{
			targetID = (String)xarch.get(endPointRef, "Id");
		}
		// set the end point 
		xarch.set(point, "AnchorOnInterface", 
			CakeUtils.createXMLLink(xarch, instanceContext, targetID));
		// now add the endpoint to the link
		xarch.add(linkRef, "Point", point);		
	}

	// Adds a component to the archStructure, the layer and return the containing element.
	private ObjRef addComponentToLayer(ObjRef layer)
	{
		assert layer != null: "Layer is null";
		
		// Add a new component to the xArch model's ArchStructure
		ObjRef archStructure = getArchStructure();
		ObjRef component = xarch.create(typesContext, "Component");
		String id = UIDGenerator.generateUID();
		xarch.set(component, "Id", id);
		ObjRef description = xarch.create(instanceContext, "Description");
		xarch.set(description, "Value", "");
		xarch.set(component, "Description", description);
		xarch.add(archStructure, "Component", component);
		
		// Create containingElement XMLLink to this component
		ObjRef xmlLink = CakeUtils.createLayeredXMLLink(xarch, layersContext, id);
		xarch.add(layer, "ContainingElement", xmlLink);
		xmlLink = xarch.getByID(modelXArch, (String)xarch.get(xmlLink, "Id"));
		
		return xmlLink;
	}
	
	// Adds a connector to the archStructure, the layer and return the containing element.
	private ObjRef addConnectorToLayer(ObjRef layer)
	{
		assert layer != null: "Layer is null";
		
		// Add new connector to the xArch model's ArchStructure
		ObjRef archStructure = getArchStructure();
		ObjRef connector = xarch.create(typesContext, "Connector");
		String id = UIDGenerator.generateUID();
		xarch.set(connector, "Id", id);
		ObjRef description = xarch.create(instanceContext, "Description");
		xarch.set(description, "Value", "");
		xarch.set(connector, "Description", description);
		xarch.add(archStructure, "Connector", connector);
		
		// Create containingElement LayeredXMLLink to this connector
		ObjRef xmlLink = CakeUtils.createLayeredXMLLink(xarch, layersContext, id);
		xarch.add(layer, "ContainingElement", xmlLink);
		xmlLink = xarch.getByID(modelXArch, (String)xarch.get(xmlLink, "Id"));
		
		return xmlLink;
	}
	
	private ObjRef addLink(ObjRef layer)
	{
		assert layer != null: "Layer is null";
		// Add new link to the xArch model's ArchStructure
		ObjRef archStructure = getArchStructure();
		ObjRef link = xarch.create(typesContext, "Link");
		String id = UIDGenerator.generateUID();
		xarch.set(link, "Id", id);
		ObjRef description = xarch.create(instanceContext, "Description");
		xarch.set(description, "Value", "");
		xarch.set(link, "Description", description);
		xarch.add(archStructure, "Link", link);
		
		// Create containingElement LayeredXMLLink to this connector
		ObjRef xmlLink = CakeUtils.createLayeredXMLLink(xarch, layersContext, id);
		xarch.add(layer, "ContainingElement", xmlLink);
		xmlLink = xarch.getByID(modelXArch, (String)xarch.get(xmlLink, "Id"));
		
		return xmlLink;
	}
	
	private ObjRef addAnnotationToLayer(ObjRef annotation, ObjRef layer)
	{
		assert layer != null: "Layer is null";
		assert annotation != null: "Annotation is null";		
		
		// Create containingElement LayeredXMLLink to this annotation
		String id = (String)xarch.get(annotation, "Id");
		ObjRef xmlLink = CakeUtils.createLayeredXMLLink(xarch, layersContext, id);
		xarch.add(layer, "ContainingElement", xmlLink);
		xmlLink = xarch.getByID(modelXArch, (String)xarch.get(xmlLink, "Id"));
		
		return xmlLink;
	}
	
	private ObjRef addAnnotationToXArch()
	{
		ObjRef archAnnotation = getArchAnnotation();
		ObjRef annotation = xarch.create(annotationsContext, "Annotation");
		ObjRef comment = xarch.create(annotationsContext, "Comment");
		xarch.set(comment, "Value", "");
		xarch.set(annotation, "Comment", comment);
		String id = UIDGenerator.generateUID();
		xarch.set(annotation, "Id", id);
		xarch.add(archAnnotation, "Annotation", annotation);
		
		return annotation;
	}
	
	private boolean isThingAnnotatable(Thing t)
	{
		return t != null && t instanceof CakeComponentThing || t instanceof CakeConnectorThing;
	}
	
	private ObjRef getArchStructure()
	{
		if(archStructure != null)
			return archStructure;
		
		archStructure = xarch.getElement(typesContext, "ArchStructure", modelXArch);
		if(archStructure == null)
		{
			// If non-existant then make it and add to modelXArch since we need one			
			archStructure = xarch.createElement(typesContext, "ArchStructure");
			xarch.add(modelXArch, "Object", archStructure);
		}
		
		return archStructure;
	}
	
	private ObjRef getArchAnnotation()
	{
		if(archAnnotation != null)
			return archAnnotation;
		
		archAnnotation = xarch.getElement(annotationsContext, "ArchAnnotation", modelXArch);
		if(archAnnotation == null)
		{
			// If doesn't exist, create it, add it to modelXArch
			archAnnotation = xarch.createElement(annotationsContext, "ArchAnnotation");
			xarch.add(modelXArch, "Object", archAnnotation);
		}
		
		return archAnnotation;
	}
}
