/*
 * Created on Jun 1, 2004
 *
 */
package archstudio.comp.cake;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;

import archstudio.comp.archipelago.ThingIDMap;
import c2.util.UIDGenerator;
import edu.uci.ics.bna.BNAComponent;
import edu.uci.ics.bna.Thing;
import edu.uci.ics.bna.contextmenu.AbstractSelectionBasedContextMenuPlugin;
import edu.uci.ics.bna.swingthing.ResizableLocalBoxBoundedThing;
import edu.uci.ics.bna.swingthing.SwingPanelThing;
import edu.uci.ics.widgets.windowheader.WindowHeaderPanel;
import edu.uci.ics.widgets.windowheader.WindowHeaderPanelListener;
import edu.uci.ics.xadlutils.XadlUtils;
import edu.uci.ics.xarchutils.ObjRef;
import edu.uci.ics.xarchutils.XArchFlatInterface;

/**
 * @author Chris
 *
 */
public class CakeAddEditInterfaceContextMenuPlugin
	extends AbstractSelectionBasedContextMenuPlugin
{
	protected ThingIDMap thingIDMap;
	protected InterfaceMap interfaceMap;
	protected XArchFlatInterface xarch;
	private ObjRef modelXArch;

	/**
	 * @param bna
	 */
	public CakeAddEditInterfaceContextMenuPlugin(BNAComponent bna, ThingIDMap thingIDMap, InterfaceMap iMap,
		XArchFlatInterface xarch, ObjRef modelXArch)
	{
		super(bna);
		this.thingIDMap = thingIDMap;
		this.interfaceMap = iMap;
		this.xarch = xarch;
		this.modelXArch = modelXArch;
	}

	/* (non-Javadoc)
	 * @see edu.uci.ics.bna.contextmenu.SelectionBasedContextMenuPlugin#addToContextMenu(javax.swing.JPopupMenu, edu.uci.ics.bna.Thing[], edu.uci.ics.bna.Thing)
	 */
	public JPopupMenu addToContextMenu(
		JPopupMenu currentContextMenu,
		Thing[] selectedThingSet,
		Thing thingUnderCursor)
	{
		Thing t = null;
		if(selectedThingSet.length == 0)
		{
			if(thingUnderCursor != null)
			{
				t = thingUnderCursor;
			}
			else
			{
				return currentContextMenu;
			}
		}
		else if(selectedThingSet.length > 1)
		{
			return currentContextMenu;
		}
		else
		{
			t = selectedThingSet[0];
		}

		if((t instanceof CakeComponentThing) ||
		(t instanceof CakeConnectorThing) ||
		(t instanceof CakeInterfaceThing))
		{
			AddEditInterfaceMenuItemSet aimis = new AddEditInterfaceMenuItemSet(t);
			JMenuItem[] miArray = aimis.getMenuItemSet();
			for(int i = 0; i < miArray.length; i++)
			{
				currentContextMenu.add(miArray[i]);
			}
		}

		return currentContextMenu;
	}
	
	protected void centerInComponent(ResizableLocalBoxBoundedThing lbbt)
	{
		Rectangle lbbtBoundingBox = lbbt.getLocalBoundingBox();
	
		int lbbtWidth = lbbtBoundingBox.width;
		int lbbtHeight = lbbtBoundingBox.height;
	
		Dimension bnaSize = getBNAComponent().getSize();
	
		int bnacx = bnaSize.width / 2;
		int bnacy = bnaSize.height / 2;
	
		int ulx = bnacx - (lbbtWidth / 2);
		int uly = bnacy - (lbbtHeight / 2);
	
		lbbtBoundingBox.setLocation(ulx, uly);
		lbbt.setLocalBoundingBox(lbbtBoundingBox);
	}
	
	private class AddEditInterfaceMenuItemSet implements ActionListener
	{
		protected Thing thingToEdit;
		protected JMenuItem miAddEditInterface;
		

		/**
		 * @param t
		 */
		public AddEditInterfaceMenuItemSet(Thing t)
		{
			this.thingToEdit = t;
			if(thingToEdit instanceof CakeInterfaceThing)
			{
				miAddEditInterface = new JMenuItem("Edit Interface...");
			}
			else
			{
				miAddEditInterface = new JMenuItem("Add Interface...");
			}
			
			miAddEditInterface.addActionListener(this);
		}
		
		public JMenuItem[] getMenuItemSet()
		{
			return new JMenuItem[]{miAddEditInterface};
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent evt)
		{
			SwingPanelThing addInterfacePanel = new SwingPanelThing();
			AddEditInterfacePanel edp = new AddEditInterfacePanel(addInterfacePanel, thingToEdit);

			getBNAComponent().getModel().addThing(addInterfacePanel);
			addInterfacePanel.setLocalBoundingBox(new Rectangle(0, 0, 300, 90));
			centerInComponent(addInterfacePanel);
		}
				
		private class AddEditInterfacePanel extends JPanel implements WindowHeaderPanelListener, ActionListener
		{
			protected Thing swingPanelThing;
			protected Thing thingToEdit;

			protected JComboBox cbDirection;
			protected JTextField tfDescription;
			protected JButton bOK;

			/**
			 * @param descEditPanel
			 * @param thingToEdit
			 */
			public AddEditInterfacePanel(SwingPanelThing swingPanelThing, Thing thingToEdit)
			{
				super();
				this.swingPanelThing = swingPanelThing;
				this.thingToEdit = thingToEdit;

				this.setLayout(new BorderLayout());

				WindowHeaderPanel whp = null;
				if(thingToEdit instanceof CakeInterfaceThing)
				{
					whp = new WindowHeaderPanel("Edit Interface");
				}
				else
				{
					whp = new WindowHeaderPanel("Add Interface");	
				}
				
				whp.addWindowHeaderPanelListener(this);

				this.add("North", whp);

				DefaultComboBoxModel cbm = new DefaultComboBoxModel();
				cbm.addElement("none");
				cbm.addElement("in");
				cbm.addElement("out");
				cbm.addElement("inout");

				cbDirection = new JComboBox(cbm);
				cbDirection.setEditable(true);
				cbDirection.addActionListener(this);

				bOK = new JButton("OK");
				bOK.addActionListener(this);
				
				tfDescription = new JTextField(15);
				tfDescription.addActionListener(this);
				
				
				JPanel centerPanel = new JPanel();
				centerPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
				centerPanel.add(tfDescription);				

				this.add("Center", centerPanel);								
				
				JPanel southPanel = new JPanel();
				southPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
				southPanel.add(cbDirection);
				southPanel.add(bOK);
				
				this.add("South", southPanel);  
				
				swingPanelThing.setPanel(this);
				swingPanelThing.setIndicatorThingId(thingToEdit.getID());
				if(thingToEdit instanceof CakeInterfaceThing)
				{
					updateInterfaceFields();
				}			
				
				// make the text field gain focus
				tfDescription.addAncestorListener(
					new javax.swing.event.AncestorListener(){
						public void ancestorAdded(javax.swing.event.AncestorEvent event){
							tfDescription.requestFocus();
						}
						public void ancestorRemoved(javax.swing.event.AncestorEvent event){}
						public void ancestorMoved(javax.swing.event.AncestorEvent event){}
					}
				);
				 
				
			}
			
			protected void updateInterfaceFields()
			{
				CakeInterfaceThing interfaceThing;
				try
				{
					interfaceThing = (CakeInterfaceThing)thingToEdit;
				}
				catch(ClassCastException cce)
				{
					System.err.println("Could not cast to CakeInterfaceThing");
					cce.printStackTrace();
					return;
				}
				
				ObjRef interfaceRef = CakeUtils.getInterfaceRef(interfaceThing, thingIDMap, interfaceMap);
																											
				String direction = XadlUtils.getDirection(xarch, interfaceRef);				
				cbDirection.getEditor().setItem(direction);
				String description = XadlUtils.getDescription(xarch, interfaceRef);				
				tfDescription.setText(description);												
			}

			/* (non-Javadoc)
			 * @see edu.uci.ics.widgets.windowheader.WindowHeaderPanelListener#closeButtonPressed(edu.uci.ics.widgets.windowheader.WindowHeaderPanel)
			 */
			public void closeButtonPressed(WindowHeaderPanel src)
			{
				remove();
			}

			/* (non-Javadoc)
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			public void actionPerformed(ActionEvent evt)
			{
				if(evt.getSource() == bOK)
				{
					String newDescription = tfDescription.getText();
					String newDirection = (String)cbDirection.getEditor().getItem();
					remove();
					
					if(thingToEdit instanceof CakeInterfaceThing)
					{
						updateInterface(newDescription, newDirection);
					}
					else
					{												
						addNewInterface(newDescription, newDirection);	
					}					
				}							
			}
			
			private void updateInterface(String description, String direction)
			{
				assert thingToEdit instanceof CakeInterfaceThing: "thingToEdit is not of type CakeInterfaceThing";
				
				CakeInterfaceThing interfaceThing = (CakeInterfaceThing)thingToEdit;
				ObjRef interfaceRef = CakeUtils.getInterfaceRef(interfaceThing, thingIDMap, interfaceMap);
				XadlUtils.setDescription(xarch, interfaceRef, description);
				XadlUtils.setDirection(xarch, interfaceRef, direction);		
			}
			
			/**			 
			 * @param description
			 * @param direction
			 */
			private void addNewInterface(String description, String direction)
			{
				// Get element to add to
				if(thingToEdit instanceof CakeComponentThing || thingToEdit instanceof CakeConnectorThing)
				{	
					ObjRef elementRef = thingIDMap.getXArchRef(thingToEdit.getID());
					ObjRef elementToEditRef = CakeUtils.resolveXMLLink(xarch, modelXArch, elementRef);			
					if(elementToEditRef != null)
					{
						// Create a new interface and add to element
						ObjRef typesContext = xarch.createContext(modelXArch, "Types");
						ObjRef instanceContext = xarch.createContext(modelXArch, "Instance");
						ObjRef interfaceRef = xarch.create(typesContext, "Interface");
						ObjRef descriptionRef = xarch.create(instanceContext, "Description");
						xarch.set(descriptionRef, "Value", description);
						xarch.set(interfaceRef, "Description", descriptionRef);
						xarch.set(interfaceRef, "Id", UIDGenerator.generateUID());
						ObjRef directionRef = xarch.create(instanceContext, "Direction");
						xarch.set(directionRef, "Value", direction);
						xarch.set(interfaceRef, "Direction", directionRef);

						xarch.add(elementToEditRef, "Interface", interfaceRef);							
					}
				}										
			}			
			
			public void remove()
			{
				//((SwingPanelThing)swingPanelThing).setIndicatorThingId(null);
				//System.out.println("Removing... " + ((SwingPanelThing)swingPanelThing).getIndicatorThingId());
				getBNAComponent().getModel().removeThing(swingPanelThing);
			}
		}
		
	}

}
