/*
 * Created on Jun 7, 2004
 *
 */
package archstudio.comp.cake;

import archstudio.comp.archipelago.ThingIDMap;
import edu.uci.ics.bna.BNAComponent;
import edu.uci.ics.bna.BNAModel;
import edu.uci.ics.bna.Thing;
import edu.uci.ics.xarchutils.ObjRef;
import edu.uci.ics.xarchutils.XArchFlatEvent;
import edu.uci.ics.xarchutils.XArchFlatInterface;
import edu.uci.ics.xarchutils.XArchPath;

/**
 * @author Chris
 *
 */
public class CakeAnnotationMappingLogic extends CakeAbstractMappingLogic
{
	protected ThingIDMap thingIDMap;
	protected BNAModel bnaModel;
	protected XArchFlatInterface xarch;
	private ObjRef modelXArch;

	/**
	 * @param bnaComponent
	 * @param xarch
	 */
	public CakeAnnotationMappingLogic(
		BNAComponent bnaComponent,
		XArchFlatInterface xarch,
		ThingIDMap thingIDMap,
		ObjRef modelXArch)
	{
		super(bnaComponent, xarch);
		this.thingIDMap = thingIDMap;
		this.bnaModel = bnaComponent.getModel();
		this.xarch = xarch;
		this.modelXArch = modelXArch;
	}

	/**
	 * This function handles the updates to xArch annotations.
	 */
	public void handleXArchFlatEvent(XArchFlatEvent evt)
	{
		/*
		try
		{
		*/
		//System.out.println("Handling events! Here");
		//System.out.println(evt);
		ObjRef src = evt.getSource();
		ObjRef docRef = xarch.getXArch(src);
		
		XArchPath path = evt.getTargetPath();
		if(path == null || evt.getEventType() == XArchFlatEvent.REMOVE_EVENT)
		{
			// Remove event the target path is just the element
			// so we want the source path instead.
			
			path = evt.getSourcePath();
		}
		if(path == null)
		{
			//System.out.println("Null again");
			return;
		}

		if(evt.getEventType() == XArchFlatEvent.REMOVE_EVENT &&
		   handleRemoves(evt))
		{
			ObjRef target = (ObjRef)evt.getTarget();
			// we are removing an annotation, so gotta go through
			// and update all the containingElements and remove them
			deleteElement(target, docRef);
			// done handling this event
			return;
		}
				
		String pathString = path.toTagsOnlyString();
		//System.out.println("pathString = " + pathString);
		if(pathString.startsWith("xArch/archAnnotation/annotation"))
		{
			//System.out.println("Annotation");
			//It's an annotation event
			if(src != null)
			{
				ObjRef[] ancestors = xarch.getAllAncestors(src);
				// the last one is xArch, then archAnnotation, then annotation
				int annotationIndex = ancestors.length - 3;
				//The annotation will be ancestor #
				ObjRef annotationRef = ancestors[annotationIndex];
				//System.out.println(xarch.getType(annotationRef));
				updateElement(annotationRef, docRef);
			}
		}
		/*
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		*/
	}
	
	private boolean handleRemoves(XArchFlatEvent evt)
	{
		String targetName = evt.getTargetName();
		return targetName.equals("annotation");
	}
	
	private void deleteElement(ObjRef elementRef, ObjRef docRef)
	{
		ObjRef[] references = xarch.getReferences(docRef, 
			(String) xarch.get(elementRef, "Id"));
		//System.out.println(references.length);
		for(int i = 0; i < references.length; i++)
		{
			// find the references that are of type LayeredXMLLink
			// since these are the ones that are used in containingElement
			// for layers.
			if(xarch.isInstanceOf(references[i], 
				"edu.uci.isr.xarch.layers.ILayeredXMLLink"))
			{
				ObjRef parent = xarch.getParent(references[i]);
				xarch.remove(parent, "ContainingElement", references[i]);
			}
		}		
	}
	
	private void updateElement(ObjRef elementRef, ObjRef docRef)
	{
		LayerableFloatingLabel layerableLabel = null;		
		ObjRef[] references = xarch.getReferences(docRef, (String)xarch.get(elementRef, "Id"));
		String comment = CakeUtils.getAnnotationComment(xarch, elementRef);
		ObjRef annotatedElement = CakeUtils.getAnchoredElement(xarch, elementRef, docRef);
		
		String annotLabel = "";
		if(comment != null)
		{
			annotLabel = comment;
		}				
		
		for(int i = 0; i < references.length; ++i)
		{							
			updateAnnotation(references[i], annotLabel, annotatedElement);
		}
	}

	public void updateAnnotation(ObjRef containingElement, String annotLabel, ObjRef annotatedElement)
	{
		if(xarch.isInstanceOf(containingElement, "edu.uci.isr.xarch.layers.ILayeredXMLLink"))
		{
			LayerableFloatingLabel layerableLabel = null;
			String existingThingID = thingIDMap.getThingID(containingElement);
			if(existingThingID != null)
			{
				try
				{
					layerableLabel = (LayerableFloatingLabel)bnaModel.getThing(existingThingID);
				}
				catch(ClassCastException cce)
				{
					System.out.println("Warning: ID/Type mismatch");
					cce.printStackTrace();
					return;
				}
				layerableLabel.setLabel(annotLabel);
				
				if(annotatedElement != null)
				{
					// Update the indicator on the annotation thing 
					String anchoredThingID = thingIDMap.getThingID(annotatedElement);
					Thing anchoredThing = bnaModel.getThing(anchoredThingID); 
					layerableLabel.setIndicatorThingId(anchoredThing.getID());					
				}
			}
		}		
	}
	
	public void updateAnnotation(ObjRef containingElementRef)
	{				
		LayerableFloatingLabel layerableLabel = null;
		ObjRef elementRef = CakeUtils.resolveXMLLink(xarch, modelXArch, containingElementRef);
		if(elementRef != null)
		{
		
			String comment = CakeUtils.getAnnotationComment(xarch, elementRef);
			ObjRef annotatedElement = CakeUtils.getAnchoredElement(xarch, elementRef, modelXArch);
			
			String annotLabel = "";
			if(comment != null)
			{
				annotLabel = comment;
			}		
			
			updateAnnotation(containingElementRef, annotLabel, annotatedElement);
		}		
	}
}
