/*
 * Created on Jun 2, 2004
 *
 */
package archstudio.comp.cake;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.font.FontRenderContext;
import java.awt.font.LineMetrics;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.StringTokenizer;

import edu.uci.ics.bna.BNAComponent;
import edu.uci.ics.bna.CoordinateMapper;
import edu.uci.ics.bna.Thing;
	
/**
 * @author Chris
 *
 */
public class CakeAnnotationThingPeer extends LayerableFloatingLabelPeer
{
	private CakeAnnotationThing t;
	private String annotLabel;
	private String annotHeader;
	private String wordWrappedLabel;
	private String[] lines;
	private Rectangle2D[] bounds;
	private LineMetrics[] lineMetrics;
	private int width;
	private int height;

	/**
	 * @param c
	 * @param t
	 */
	public CakeAnnotationThingPeer(BNAComponent c, Thing t)
	{
		super(c, t);
		if(!(t instanceof CakeAnnotationThing))
		{
			throw new IllegalArgumentException("CakeAnnotationThingPeer can only be a peer for" +
				"a CakeAnnotationThing");
		}
		
		this.t = (CakeAnnotationThing)t;
		annotLabel = null;
		wordWrappedLabel = null;
	}
	
	/* (non-Javadoc)
	 * @see edu.uci.ics.bna.ThingPeer#draw(java.awt.Graphics2D, edu.uci.ics.bna.CoordinateMapper)
	 */
	public void draw(Graphics2D g, CoordinateMapper cm)
	{									
		if(t.isVisible())
		{	
			String label = t.getLabel();
			if(label == null)
				return;
				
			int lx = cm.worldXtoLocalX(t.getX());
			int ly = cm.worldYtoLocalY(t.getY());
			
			// If the label property has changed in anyway reinitialize the necessary label line variables	
			if(!label.equals(annotLabel))
			{				
				constructMultiLineAnnotation(g, cm);
			}							
			
			int localX1 = lx;
			int localY1 = ly;
			int localX2 = localX1 + width + 2;
			int localY2 = localY1 + height;

			// Draw a glass-moded box for the annotation
			g.setPaint(t.getColor());
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
			g.fillRect(localX1, localY1, localX2 - localX1, localY2 - localY1);
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.00f));

			if(t.isSelected())
			{
				// Draw a marqueed rectangle around the box
				g.setColor(t.getColor().darker().darker());
				edu.uci.ics.bna.MarqueeUtils.drawMarqueeRectangle(g, localX1, localY1, localX2 - localX1, localY2 - localY1, t.getOffset());
			}
			else
			{
				// Draw a darkened border
				g.setColor(t.getColor().darker().darker());
				g.draw3DRect(localX1, localY1, localX2 - localX1, localY2 - localY1, true);
			}
			
 			// Draw the annotation lines in black			
			g.setColor(Color.BLACK);
			
			// Start drawing one height distance down from the top of the box; so that there is a line of text's descent
			// distance between top of the box and the ascent of the text.
			int drawY = ly + (int)bounds[0].getHeight();	 
			for(int i = 0; i < lines.length; ++i)
			{
				g.drawString(lines[i], lx + 2, drawY);
				if(i < (bounds.length - 1))
				{			
					drawY += lineMetrics[i + 1].getHeight();
				}
			}

			Rectangle boundingBox = new Rectangle();
			boundingBox.x = cm.localXtoWorldX(localX1);
			boundingBox.y = cm.localYtoWorldY(localY1);
			int wx2 = cm.localXtoWorldX(localX2);
			int wy2 = cm.localYtoWorldY(localY2);
			boundingBox.width = (wx2 - boundingBox.x);
			boundingBox.height = (wy2 - boundingBox.y);
			t.setWidth(boundingBox.width);
			t.setHeight(boundingBox.height);			

			// Draw indicator shape
			drawIndicatorShape(g, cm);
		}
	}
	
	/**
	 * Helper method used to parse the annotation label into a sequence of lines and figure out the
	 * width and height occupied by the lines. This method is intended to be called whenever it is 
	 * detected that the label property has changed. 
	 *  
	 * @param g
	 * @param cm
	 */
	private void constructMultiLineAnnotation(Graphics2D g, CoordinateMapper cm)
	{			
		Font f = edu.uci.ics.bna.Utils.DEFAULT_FONT;
		Font derivedFont = f.deriveFont(AffineTransform.getScaleInstance(cm.getScale(), cm.getScale()));
		g.setFont(derivedFont);

		FontRenderContext frc = g.getFontRenderContext();			

		annotHeader = t.getHeader();
		annotLabel = t.getLabel();
		
		wordWrappedLabel = createWordWrappedString(annotLabel, t.getMaximumWidth(), g, derivedFont);						
		
		String[] headerLines = annotHeader.split("\n", -1);
		String[] labelLines = wordWrappedLabel.split("\n", -1);
		lines = new String[headerLines.length + labelLines.length];
		ArrayList labelList = new ArrayList();
		labelList.addAll(Arrays.asList(headerLines));
		labelList.addAll(Arrays.asList(labelLines));
		lines = (String[])labelList.toArray(lines);
		int numLines = lines.length;

		bounds = new Rectangle2D[numLines];
		lineMetrics = new LineMetrics[numLines];
		for(int i = 0; i < numLines; i++)
		{
			bounds[i] = derivedFont.getStringBounds(lines[i], frc);
			lineMetrics[i] = derivedFont.getLineMetrics(lines[i], frc);
		}

		int sumOfHeights = 0;
		int maxWidth = -1;
		for(int i = 0; i < numLines; i++)
		{				
			sumOfHeights += (int)lineMetrics[i].getHeight();					
			int currWidth = (int)bounds[i].getWidth();
			if(currWidth > maxWidth)
			{
				maxWidth = currWidth;
			}
		}
		sumOfHeights += (int)lineMetrics[0].getDescent();
		
		width = maxWidth;
		height = sumOfHeights;			
	}
	
	/**
	 * Creates and returns a String that has newline characters ('\n') instrumented throughout so as to
	 * make the String wordwrap appropriately with respect to a given width.
	 *  
	 * @param s
	 * @param maxWidth
	 * @param g Graphics2D object
	 * @param f Font to use
	 * @return
	 */
	private String createWordWrappedString(String s, int maxWidth, Graphics2D g, Font f)
	{
		// Setup the Graphics object to use the specified font
		g.setFont(f);
		FontRenderContext frc = g.getFontRenderContext();	
		
		StringTokenizer st = new StringTokenizer(s, " \n", true);
		
		Rectangle2D stringBounds = null;
		String wrappedLine = "";
		String oldLine = "";
		String currentLine = "";
		while(st.hasMoreTokens())
		{			
			String token = st.nextToken();		
			
			if(token.equals("\n"))
			{
				wrappedLine += oldLine + "\n";
				oldLine = "";
				currentLine = "";
			}
			else
			{
				currentLine += token;
				
				stringBounds = f.getStringBounds(currentLine, frc);
				int stringWidth = (int)stringBounds.getWidth();
				if(stringWidth > maxWidth)
				{
					// If we encounter a word that, by itself, is greater in width than max width
					if(currentLine.equals(token))
					{
						System.out.println("Word length is longer than max line length and that's sad :(");
						// TODO: Do some clever trick to deal with this
					}
					else
					{
						wrappedLine += oldLine + "\n";
						currentLine = token;
						oldLine = "";
					}
				}
				else
				{
					oldLine = currentLine;
				}
			}		
		}
		
		// If the previously stored line is not empty that means we still have a line to append
		if(!currentLine.equals(""))
		{
			wrappedLine += currentLine;
		}		
		
		return wrappedLine;
	}

	public boolean isInThing(Graphics2D g, CoordinateMapper cm, int worldX, int worldY)
	{
		String label = t.getLabel();
		if(label == null)
			return false;
		
		// If the label property has changed in anyway reinitialize the necessary label line variables	
		if(!label.equals(annotLabel))
		{
			constructMultiLineAnnotation(g, cm);
		}		
				
		int lx = cm.worldXtoLocalX(t.getX());
		int ly = cm.worldYtoLocalY(t.getY());
		
		int localX1 = lx;
		int localY1 = ly;
		int localX2 = localX1 + width + 2;
		int localY2 = localY1 + height;

		//Okay, we have the bounding box, in local coordinates, of the drawn string.
		//Now we just need to find out if the coordinate in question is in that box.
	
		int checkLocalX = cm.worldXtoLocalX(worldX);
		int checkLocalY = cm.worldYtoLocalY(worldY);
	
		return
			(checkLocalX >= localX1) &&
			(checkLocalX <= localX2) &&
			(checkLocalY >= localY1) &&
			(checkLocalY <= localY2);
	}
	
	protected Shape getIndicatorShape(CoordinateMapper cm)
	{		
		Point p = t.getIndicatorPoint();
		if(p == null) return null;
		
		int ix = (int)p.getX();
		int iy = (int)p.getY();
		
		int x1 = t.getX();
		int x2 = t.getX2();
		
		if(x1 > x2){
			int temp = x1;
			x1 = x2;
			x2 = temp;
		}
				
		// Get Y values
		int y1 = t.getY();
		int y2 = t.getY2();
		
		if(y1 > y2){
			int temp = y1;
			y1 = y2;
			y2 = temp;
		}
		
		int cx = ((x2 - x1) / 2) + x1;
		int cy = ((y2 - y1) / 2) + y1;
		
		Point p1 = new Point();
		Point p2 = new Point();
		
		//Now figure out which corner to move to the indicator point.
		if(ix < x1 && iy < y1)
		{
			p1.x = cx - 5;
			p1.y = y1;
			p2.x = cx + 5;
			p2.y = y1;
		}
		else if(ix >= x1 && ix <= x2 && iy < y1)
		{
			p1.x = cx - 5;
			p1.y = y1;
			p2.x = cx + 5;
			p2.y = y1;
		}
		else if(ix > x2 && iy < y1)
		{
			p1.x = cx - 5;
			p1.y = y1;
			p2.x = cx + 5;
			p2.y = y1;	
		}
		else if(ix < x1 && iy >= y1 && iy <= y2)
		{
			p1.x = x1;
			p1.y = cy - 5;
			p2.x = x1;
			p2.y = cy + 5;
		}
		else if(ix > x2 && iy >= y1 && iy <= y2)
		{
			p1.x = x2;
			p1.y = cy - 5;
			p2.x = x2;
			p2.y = cy + 5;
		}
		else if(ix < x1 && iy > y2)
		{
			p1.x = cx - 5;
			p1.y = y2;
			p2.x = cx + 5;
			p2.y = y2;
		}
		else if(ix >= x1 && ix <= x2 && iy > y2)
		{
			p1.x = cx - 5;
			p1.y = y2;
			p2.x = cx + 5;
			p2.y = y2;
		}
		else if(ix > x2 && iy > y2)
		{
			p1.x = cx - 5;
			p1.y = y2;
			p2.x = cx + 5;
			p2.y = y2;	
		}

		ix = cm.worldXtoLocalX(ix);
		p1.x = cm.worldXtoLocalX(p1.x);
		p2.x = cm.worldXtoLocalX(p2.x);				

		iy = cm.worldYtoLocalY(iy);
		p1.y = cm.worldYtoLocalY(p1.y);		
		p2.y = cm.worldYtoLocalY(p2.y);
		
		//Now make a shape.
		Polygon poly = new Polygon();
		poly.addPoint(ix, iy);
		poly.addPoint(p1.x, p1.y);
		poly.addPoint(p2.x, p2.y);	
		
		return poly;
	}
}
