/*
 * Created on Jun 2, 2004
 *
 * BrickMappingLogic.java
 */
package archstudio.comp.cake;

import edu.uci.ics.bna.*;
import edu.uci.ics.xarchutils.*;

import archstudio.comp.archipelago.*;


//import java.awt.*;
import java.awt.Color;
import java.util.*;
/**
 * @author Ping
 *
 * This class provides a mapping of xADL components/connectors
 * to the Cake world Things.  
 */
public class CakeBrickMappingLogic extends CakeAbstractMappingLogic
{
	protected ThingIDMap thingIDMap;
	protected InterfaceMap intfMap;

	protected BNAModel bnaModel;
		
	//protected ObjRef docRef;
	//protected NoThing brickParent;
	//static int offset = 0;
	
	public CakeBrickMappingLogic(BNAComponent bnaComponent, XArchFlatInterface xarch, 
		ThingIDMap thingIDMap, InterfaceMap iMap)
	{
		super(bnaComponent, xarch);
		this.bnaModel = bnaComponent.getModel();
		this.thingIDMap = thingIDMap;	
		this.intfMap = iMap;
		//this.docRef = docRef;
		/*
		brickParent = new NoThing("$$BrickParent");
		bnaBNAModel.addThing(brickParent);
		*/
	}
	
	/**
	 * This function handles the updates to xArch components and 
	 * connectors.
	 */
	public void handleXArchFlatEvent(XArchFlatEvent evt)
	{
		/*
		try
		{
		*/
		//System.out.println("Handling events! Here");
		//System.out.println(evt);
		ObjRef src = evt.getSource();
		ObjRef docRef = xarch.getXArch(src);
		
		XArchPath path = evt.getTargetPath();
		if(path == null || evt.getEventType() == XArchFlatEvent.REMOVE_EVENT)
		{
			// Remove event the target path is just the element
			// so we want the source path instead.
			
			path = evt.getSourcePath();
		}
		if(path == null)
		{
			//System.out.println("Null again");
			return;
		}

		if(evt.getEventType() == XArchFlatEvent.REMOVE_EVENT &&
		   handleRemoves(evt))
		{
			ObjRef target = (ObjRef)evt.getTarget();
			// we are removing a component, so gotta go through
			// and update all the containingElements and remove them
			deleteElement(target, docRef);
			// done handling this event
			return;
		}
		
		
		String pathString = path.toTagsOnlyString();
		//System.out.println("pathString = " + pathString);
		if(pathString.startsWith("xArch/archStructure/component"))
		{
			//System.out.println("Component");
			//It's a component event
			if(src != null)
			{
				ObjRef[] ancestors = xarch.getAllAncestors(src);
				// the last one is xArch, then archstructure, then component
				int componentIndex = ancestors.length - 3;
				//The component will be ancestor #
				ObjRef componentRef = ancestors[componentIndex];
				//System.out.println(xarch.getType(componentRef));
				updateElement(componentRef, docRef);
			}
		}
		else if(pathString.startsWith("xArch/archStructure/connector"))
		{
			//It's a connector event						
			if(src != null)
			{
				ObjRef[] ancestors = xarch.getAllAncestors(src);
				// the last one is xArch, then archstructure, then connector
				int connectorIndex = ancestors.length - 3;
				ObjRef connectorRef = ancestors[connectorIndex];
				//System.out.println(xarch.getType(connectorRef));
				updateElement(connectorRef, docRef);
			}
		}
		/*
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		*/
	}
	
	private boolean handleRemoves(XArchFlatEvent evt)
	{
		String targetName = evt.getTargetName();
		return targetName.equals("component") ||
			   targetName.equals("connector");
	}
	
	/**
	 * This function deals with the deletion of a component
	 * or connector and updates all the containingElements of the layers
	 * as appropriate.
	 * 
	 * @param element This is the element that was removed 
	 * @param type This specifies the class name of the element (connector or component)
	 */
	private void deleteElement(ObjRef element, ObjRef docRef)
	{
		ObjRef[] references = xarch.getReferences(docRef, 
			(String) xarch.get(element, "Id"));
		//System.out.println(references.length);
		for(int i = 0; i < references.length; i++)
		{
			ObjRef parent = xarch.getParent(references[i]);
			// find the references that are of type LayeredXMLLink
			// since these are the ones that are used in containingElement
			// for layers.
			if(xarch.isInstanceOf(references[i], 
				"edu.uci.isr.xarch.layers.ILayeredXMLLink"))
			{
				xarch.remove(parent, "ContainingElement", references[i]);
			}
			else if(xarch.isInstanceOf(parent,
				"edu.uci.isr.xarch.annotations.IAnnotation"))
			{
				xarch.clear(references[i], "Href");
			}
		}
	}
	
	protected void updateElement(ObjRef elementRef, ObjRef docRef)
	{
		
		ObjRef[] references = xarch.getReferences(docRef, 
			(String) xarch.get(elementRef, "Id"));
		ObjRef[] interfaceRefs = xarch.getAll(elementRef, "Interface");
		boolean[] visited = new boolean[interfaceRefs.length];
				
		String desc = edu.uci.ics.xadlutils.XadlUtils.getDescription(
			xarch, elementRef);
		// determine what the label should be.
		String label = "";
		if(desc != null)
			label = desc;
	
		//System.out.println("Found " + references.length + " references");
		for(int i = 0; i < references.length; i++)
		{
			updateElementThing(references[i], interfaceRefs, visited, label);
		}
	}
	public void updateElementThing(ObjRef containingElement, ObjRef docRef)
	{
		ObjRef element = CakeUtils.resolveXMLLink(xarch, docRef, containingElement);
		if(element != null)
		{
			ObjRef[] interfaceRefs = xarch.getAll(element, "Interface");
			boolean[] visited = new boolean[interfaceRefs.length];
			String desc = edu.uci.ics.xadlutils.XadlUtils.getDescription(
				xarch, element);
			// determine what the label should be.
			String label = "";
			if(desc != null)
				label = desc;
			updateElementThing(containingElement, interfaceRefs, visited, label);
		}
		
	}
	
	public void updateElementThing(ObjRef containingElement, ObjRef[] interfaceRefs,
		boolean[] visited, String label)
	{
		LayerableBox boxThing;
		String existingThingID = thingIDMap.getThingID(containingElement);
		if(existingThingID != null)
		{
			//System.out.println("Found Thing");
			for(int j = 0; j < visited.length; j++)
			{
				visited[j] = false;
			}
				
			try
			{
				boxThing = (LayerableBox)bnaModel.getThing(existingThingID);
			}
			catch(ClassCastException cce)
			{
				System.err.println("Warning; ID/type mismatch.");
				cce.printStackTrace();
				return;
			}
			boxThing.setLabel(label);
				
			//System.out.println(references[i]);
			Collection interfacePairs = intfMap.getInterfacePairs(containingElement);
			if(interfacePairs != null)
			{
				//System.out.println("Element already has a set of interfaces...");
				Iterator it = interfacePairs.iterator();
				while(it.hasNext())
				{
					//System.out.println("hello there!");
					InterfaceMap.InterfacePair pair = 
						(InterfaceMap.InterfacePair)it.next();
					boolean found = false;
					for(int j = 0; j < interfaceRefs.length; j++)
					{
						//System.out.println(j);
						// ok, found it, must update now
						if(pair.getInterfaceRef().equals(interfaceRefs[j]))
						{
							//System.out.println("Found a matching ObjRefs");
							visited[j] = true;
							found = true;
							CakeInterfaceThing interfaceThing;
							try
							{
								interfaceThing = (CakeInterfaceThing)bnaModel.getThing(
									pair.getInterfaceThingID());
							}
							catch(ClassCastException cce)
							{
								System.err.println("Warning; ID/type mismatch.");
								cce.printStackTrace();
								return;
							}
							updateInterfaceThing(interfaceThing,
								interfaceRefs[j]);
							break;
						}
					}
					if(!found)
					{					
						String id = pair.getInterfaceThingID();
							
						//System.out.println("Deleting " + id);
						// the interface was not found, so its not there anymore
						// we have to delete this interface now...
						bnaModel.removeThing(id);
						//intfMap.removeInterfacePair(references[i], 
						//	id);
						it.remove();
					}
				}
			}
			for(int j = 0; j < interfaceRefs.length; j++)
			{
				//System.out.println("Making sure everything gets added");
				if(!visited[j])
				{
					//System.out.println("Adding at index" + j);
					// an obj ref has not been visited, it must be new
					// interface!
					CakeInterfaceThing interfaceThing = new CakeInterfaceThing();
					updateInterfaceThing(interfaceThing, interfaceRefs[j]);
					interfaceThing.setTargetThingID(existingThingID);
					interfaceThing.moveRelative(boxThing.getWidth(), 0);
						
					bnaModel.addThing(interfaceThing, boxThing);
						
					//System.out.println("Adding new interface " + interfaceThing.getID());
						
					intfMap.mapRefToInterfacePair(containingElement,
						interfaceRefs[j], interfaceThing.getID());
					interfaceThing.setColor(Color.WHITE);
						
				}
			}
		}
	}
	
	/**
	 * This function makes sure that the InterfaceThing matches
	 * the content of hte xADL interface
	 * @param string
	 * @param ref
	 */
	private void updateInterfaceThing(CakeInterfaceThing interfaceThing, 
		ObjRef interfaceRef)
	{
		//System.out.println("Update!");
		String iDescription = edu.uci.ics.xadlutils.XadlUtils.getDescription(
			xarch, interfaceRef);
		if(iDescription == null)
			iDescription = "";
		interfaceThing.setToolTip(iDescription);
	
		String iDirection = edu.uci.ics.xadlutils.XadlUtils.getDirection(
			xarch, interfaceRef);
		if(iDirection == null){
			interfaceThing.setFlow(CakeInterfaceThing.FLOW_NONE);
		}
		else if(iDirection.equals("inout")){
			interfaceThing.setFlow(CakeInterfaceThing.FLOW_INOUT);
		}
		else if(iDirection.equals("in")){
			interfaceThing.setFlow(CakeInterfaceThing.FLOW_IN);
		}
		else if(iDirection.equals("out")){
			interfaceThing.setFlow(CakeInterfaceThing.FLOW_OUT);
		}
		else{
			interfaceThing.setFlow(CakeInterfaceThing.FLOW_NONE);
		}
	}
}
