/*
 * Created on May 26, 2004
 *
 * CakeC2Component.java
 */
package archstudio.comp.cake;

import archstudio.invoke.*;
import c2.fw.*;
import c2.legacy.*;
import c2.pcwrap.*;
import edu.uci.ics.xarchutils.*;

import java.awt.event.*;
import java.util.Vector;
import javax.swing.*;

/**
 * @author Ping
 *
 * This is the C2 wrapper for CAKE the architectural design editor
 */
public class CakeC2Component extends AbstractC2DelegateBrick 
	implements c2.fw.Component, InvokableBrick
{
	public static final String PRODUCT_NAME = "Cake";
	public static final String PRODUCT_VERSION = "1.0";

	protected Vector openWindows;
	
	XArchFlatInterface xarch;
	/**
	 * @param id
	 */
	public CakeC2Component(Identifier id)
	{
		super(id);
		//System.out.println("Entering");
		xarch = (XArchFlatInterface)EBIWrapperUtils.addExternalService(
			this, topIface, XArchFlatInterface.class);
		

		openWindows = new Vector();
		this.addLifecycleProcessor(new CakeLifecycleProcessor());
		this.addMessageProcessor(new StateChangeMessageProcessor());
		
		//System.out.println("Here");
		
		InvokeUtils.deployInvokableService(this, bottomIface, PRODUCT_NAME, 
			"A layered architecture editor");
	}
	
	// This is just an internal class handling window closed events
	class WindowClosingAdapter extends WindowAdapter
	{
		public void windowClosed(WindowEvent evt)
		{
			openWindows.removeElement(evt.getWindow());
		}
	}
	
	// Go through and dispose of all the open windows when shutting down
	class CakeLifecycleProcessor extends LifecycleAdapter
	{
		public void end()
		{
			for(int i = 0; i < openWindows.size(); i++)
			{
				((JFrame)openWindows.elementAt(i)).setVisible(false);
				((JFrame)openWindows.elementAt(i)).dispose();
			}
		}
	}
	
	public void invoke(InvokeMessage im)
	{
		if(im.getServiceName().equals(PRODUCT_NAME)){
			String url = im.getArchitectureURL();

			if(url == null)
			{
				String[] openUrls = xarch.getOpenXArchURLs();
				if(openUrls != null && openUrls.length > 0)
				{
					
					Object selectedValue = null;
					while(selectedValue == null)
					{
						// asks the user to select an open architecture
						selectedValue = JOptionPane.showInputDialog(null,
			  				"Please select an architecture to open:", "Open Architecture",
			   				JOptionPane.INFORMATION_MESSAGE, null,
			   				openUrls, openUrls[0]);
					}
					newWindow((String)selectedValue);
				}
				else
				{
					JOptionPane.showMessageDialog(null, 
						"You must first create/open an architecture",
						"Error", JOptionPane.ERROR_MESSAGE);
				}
			}
			else
			{
				newWindow(url);
			}
		}
	}
	
	class StateChangeMessageProcessor implements MessageProcessor
	{
		public void handle(Message m)
		{
			if(m instanceof NamedPropertyMessage)
			{
				NamedPropertyMessage npm = (NamedPropertyMessage)m;
				try
				{
					if(npm.getBooleanParameter("stateChangeMessage"))
					{
						//System.out.println("Got state change message: " + m);
						//System.out.println("ArchEditComponent component got state change message: " + m);
						Object evtObject = npm.getParameter("paramValue0");
						
						if(evtObject instanceof XArchFlatEvent)
						{
							XArchFlatEvent evt = (XArchFlatEvent)evtObject;
							handleStateChangeEvent(evt);
						}
						else if(evtObject instanceof XArchFileEvent)
						{
							XArchFileEvent evt = (XArchFileEvent)evtObject;
							handleFileEvent(evt);
						}
					}
				}
				catch(Exception e)
				{
					//if(!(e instanceof IllegalArgumentException)){
					//	e.printStackTrace();
					//}
				}
			}
		}
	}
	
	public void handleFileEvent(XArchFileEvent evt)
	{
		if(evt.getEventType() == XArchFileEvent.XARCH_CLOSED_EVENT)
		{
			CakeFrame f = getWindow(evt.getURL());
			if(f != null)
			{
				f.doClose();
			}
			/*
			synchronized(openWindows)
			{
				int size = openWindows.size();
				
				for(int i = (size - 1); i >= 0; i--)
				{
					CakeFrame f = (CakeFrame)openWindows.elementAt(i);
					if(f.getDocumentSourceType() == CakeFrame.SRC_URL)
					{
						String url = f.getDocumentSource();
						if(url.equals(evt.getURL()))
						{
							f.doClose();
						}
					}
				}
			}
			*/		
		}
	}
	
	public void handleStateChangeEvent(XArchFlatEvent evt)
	{
		CakeFrame hasFocus = null;
		ObjRef sourceRef = evt.getSource();
		ObjRef sourceDocRef = xarch.getXArch(sourceRef);
		String url = xarch.getXArchURL(sourceDocRef);
		synchronized(openWindows)
		{
			int size = openWindows.size();
			for(int i = (size - 1); i >= 0; i--)
			{
				CakeFrame f = (CakeFrame)openWindows.elementAt(i);
				if(f.hasFocus())
				{
					hasFocus = f;
					break;
				}
				else if(f.getFocusOwner() != null)
				{
					hasFocus = f;
					break;
				}
			}
		}
		CakeFrame f = getWindow(url);
		if(f != null)
		{
			if(!f.isShowing())
			{
				openWindows.remove(f);
			}
			else
			{
				f.handleXArchFlatEvent(evt);
			} 
		}
		/*
		for(int i = (size - 1); i >= 0; i--)
		{
			CakeFrame f = (CakeFrame)openWindows.elementAt(i);
			if(!f.isShowing())
			{
				openWindows.removeElementAt(i);
			}
			else if(f.getDocumentSource)
			{
				f.handleXArchFlatEvent(evt);
			}
		}
		*/
		if(hasFocus != null)
		{
			hasFocus.requestFocus();
		}
	}

	public CakeFrame newWindow(String url)
	{
		CakeFrame f = new CakeFrame(xarch, url);
		f.addWindowListener(new WindowClosingAdapter());
		openWindows.addElement(f);
		return f;
	}
	
	private CakeFrame getWindow(String url)
	{
		synchronized(openWindows)
		{
			int size = openWindows.size();
			for(int i = (size - 1); i >= 0; i--)
			{
				CakeFrame f = (CakeFrame)openWindows.elementAt(i);
				String frameURL = f.getDocumentSource();
				if(frameURL != null)
				{
					if(frameURL.equals(url))
					{
						return f;
					}
				}
			}
		}
		return null;
	}
	
	/*
	public void showWindow(String url, String id)
	{
		synchronized(openWindows)
		{
			int size = openWindows.size();
			for(int i = (size - 1); i >= 0; i--)
			{
				CakeFrame f = (CakeFrame)openWindows.elementAt(i);
				String frameURL = f.getDocumentSource();
				if(url.equals(frameURL))
				{
					f.showID(id);
					return;
				}
			}
			//We didn't find a window with that URL
			CakeFrame newFrame = newWindow(url);
			newFrame.showID(id);
		}
	}
	*/
}

