/*
 * Created on Jun 6, 2004
 *
 */
package archstudio.comp.cake;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import archstudio.comp.archipelago.ThingIDMap;

import edu.uci.ics.bna.BNAComponent;
import edu.uci.ics.bna.Thing;
import edu.uci.ics.bna.contextmenu.AbstractSelectionBasedContextMenuPlugin;
import edu.uci.ics.bna.swingthing.ResizableLocalBoxBoundedThing;
import edu.uci.ics.bna.swingthing.SwingPanelThing;
import edu.uci.ics.widgets.windowheader.WindowHeaderPanel;
import edu.uci.ics.widgets.windowheader.WindowHeaderPanelListener;
import edu.uci.ics.xarchutils.ObjRef;
import edu.uci.ics.xarchutils.XArchFlatInterface;

/**
 * @author Chris
 *
 */
public class CakeEditAnnotationContextMenuPlugin
	extends AbstractSelectionBasedContextMenuPlugin
{
	protected XArchFlatInterface xarch;
	protected ThingIDMap thingIDMap;
	private ObjRef modelXArch;

	/**
	 * @param bna
	 */
	public CakeEditAnnotationContextMenuPlugin(BNAComponent c, ThingIDMap thingIDMap, XArchFlatInterface xarch, 
		ObjRef modelXArch)
	{
		super(c);
		this.xarch = xarch;
		this.modelXArch = modelXArch;
		this.thingIDMap = thingIDMap;
	}

	/* (non-Javadoc)
	 * @see edu.uci.ics.bna.contextmenu.SelectionBasedContextMenuPlugin#addToContextMenu(javax.swing.JPopupMenu, edu.uci.ics.bna.Thing[], edu.uci.ics.bna.Thing)
	 */
	public JPopupMenu addToContextMenu(
		JPopupMenu currentContextMenu,
		Thing[] selectedThingSet,
		Thing thingUnderCursor)
	{
		Thing t = null;
		if(selectedThingSet.length == 0){
			if(thingUnderCursor != null){
				t = thingUnderCursor;
			}
			else{
				return currentContextMenu;
			}
		}
		else if(selectedThingSet.length > 1){
			return currentContextMenu;
		}
		else{
			t = selectedThingSet[0];
		}
		
		if(t instanceof CakeAnnotationThing)		
		{
			EditAnnotationMenuItemSet edmis = new EditAnnotationMenuItemSet(t);
			JMenuItem[] miArray = edmis.getMenuItemSet();
			for(int i = 0; i < miArray.length; i++){
				currentContextMenu.add(miArray[i]);
			}
		}
		
		return currentContextMenu;
	}
	
	protected void centerInComponent(ResizableLocalBoxBoundedThing lbbt)
	{
		Rectangle lbbtBoundingBox = lbbt.getLocalBoundingBox();
	
		int lbbtWidth = lbbtBoundingBox.width;
		int lbbtHeight = lbbtBoundingBox.height;
	
		Dimension bnaSize = getBNAComponent().getSize();
	
		int bnacx = bnaSize.width / 2;
		int bnacy = bnaSize.height / 2;
	
		int ulx = bnacx - (lbbtWidth / 2);
		int uly = bnacy - (lbbtHeight / 2);
	
		lbbtBoundingBox.setLocation(ulx, uly);
		lbbt.setLocalBoundingBox(lbbtBoundingBox);
	}
	
	private class EditAnnotationMenuItemSet implements ActionListener 
	{
		protected Thing thingToEdit;
		protected JMenuItem miEditAnnotation;

		/**
		 * @param t
		 */
		public EditAnnotationMenuItemSet(Thing t)
		{
			this.thingToEdit = t;
			miEditAnnotation = new JMenuItem("Edit Annotation...");
			miEditAnnotation.addActionListener(this);
		}

		/**
		 * @return
		 */
		public JMenuItem[] getMenuItemSet()
		{
			return new JMenuItem[]{miEditAnnotation};
		}
		
		public void actionPerformed(ActionEvent evt)
		{
			SwingPanelThing annotEditPanel = new SwingPanelThing();
			EditAnnotationPanel eap = new EditAnnotationPanel(annotEditPanel, thingToEdit);
	
			getBNAComponent().getModel().addThing(annotEditPanel);
			annotEditPanel.setLocalBoundingBox(new Rectangle(0, 0, 300, 140));
			centerInComponent(annotEditPanel);
		}
		
		private class EditAnnotationPanel extends JPanel implements WindowHeaderPanelListener, ActionListener
		{	
			protected Thing swingPanelThing;
			protected CakeAnnotationThing annotThingtoEdit;
		
			protected JTextArea taAnnotation;
			protected JButton bOK;

			/**
			 * @param descEditPanel
			 * @param thingToEdit
			 */
			public EditAnnotationPanel(SwingPanelThing annotEditPanel, Thing thingToEdit)
			{
				super();
				this.swingPanelThing = annotEditPanel;
				this.annotThingtoEdit = (CakeAnnotationThing) thingToEdit;

				this.setLayout(new BorderLayout());

				WindowHeaderPanel whp = new WindowHeaderPanel("Edit Annotation");
				whp.addWindowHeaderPanelListener(this);

				this.add("North", whp);

				taAnnotation = new JTextArea(5, 20);	
				taAnnotation.setLineWrap(true);			
				JScrollPane scrollPane = new JScrollPane(taAnnotation);								

				bOK = new JButton("OK");
				bOK.addActionListener(this);

				JPanel centerPanel = new JPanel();
				centerPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
				centerPanel.add(scrollPane);
				centerPanel.add(bOK);

				this.add("Center", centerPanel);
				annotEditPanel.setPanel(this);
				annotEditPanel.setIndicatorThingId(thingToEdit.getID());
				updateTextArea();
				
				//make the text field gain focus
				taAnnotation.addAncestorListener(
					 new javax.swing.event.AncestorListener(){
						 public void ancestorAdded(javax.swing.event.AncestorEvent event){
							taAnnotation.requestFocus();
						 }
						 public void ancestorRemoved(javax.swing.event.AncestorEvent event){}
						 public void ancestorMoved(javax.swing.event.AncestorEvent event){}
					 }
				 );
			}

			/**
			 * 
			 */
			protected void updateTextArea()
			{										
				taAnnotation.setText(annotThingtoEdit.getLabel());
			}

			/* (non-Javadoc)
			 * @see edu.uci.ics.widgets.windowheader.WindowHeaderPanelListener#closeButtonPressed(edu.uci.ics.widgets.windowheader.WindowHeaderPanel)
			 */
			public void closeButtonPressed(WindowHeaderPanel src)
			{
				remove();
			}

			/* (non-Javadoc)
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			public void actionPerformed(ActionEvent evt)
			{
				if(evt.getSource() == bOK)
				{
					String newValue = taAnnotation.getText();
					annotThingtoEdit.setLabel(newValue);					
					
					CakeUtils.setAnnotationComment(xarch, modelXArch, thingIDMap, annotThingtoEdit, newValue);					
															
					remove();
				}
			}
			
			public void remove()
			{
				getBNAComponent().getModel().removeThing(swingPanelThing);
			}
		}		
	}
}
