/*
 * Created on Jun 10, 2004
 *
 */
package archstudio.comp.cake;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import archstudio.comp.archipelago.ThingIDMap;
import edu.uci.ics.bna.BNAComponent;
import edu.uci.ics.bna.CoordinateMapper;
import edu.uci.ics.bna.Thing;
import edu.uci.ics.bna.swingthing.ResizableLocalBoxBoundedThing;
import edu.uci.ics.bna.swingthing.SwingPanelThing;
import edu.uci.ics.widgets.windowheader.WindowHeaderPanel;
import edu.uci.ics.widgets.windowheader.WindowHeaderPanelListener;
import edu.uci.ics.xarchutils.ObjRef;
import edu.uci.ics.xarchutils.XArchFlatInterface;

/**
 * @author Chris
 *
 */
public class CakeEditAnnotationDoubleClickLogic extends CakeThingLogicAdapter
{
	protected ThingIDMap thingIDMap;
	protected XArchFlatInterface xarch;
	protected ObjRef modelXArch;
	
	public CakeEditAnnotationDoubleClickLogic(ThingIDMap thingIDMap, XArchFlatInterface xarch, ObjRef modelXArch)
	{
		this.thingIDMap = thingIDMap;
		this.xarch = xarch;
		this.modelXArch = modelXArch;
	}
	
	
	/* (non-Javadoc)
	 * @see edu.uci.ics.bna.ThingLogic#mouseClicked(edu.uci.ics.bna.Thing, java.awt.event.MouseEvent, int, int)
	 */
	public void mouseClicked(Thing t, MouseEvent evt, int worldX, int worldY)
	{		
		if(!(t instanceof CakeAnnotationThing))
		{
			return;
		}
		
		if(evt.getButton() == MouseEvent.BUTTON1)
		{
			int clickCount = evt.getClickCount();
			if(clickCount == 2)
			{
				SwingPanelThing annotEditPanel = new SwingPanelThing();
				EditAnnotationPanel eap = new EditAnnotationPanel(annotEditPanel, t);

				getBNAComponent().getModel().addThing(annotEditPanel);
				annotEditPanel.setLocalBoundingBox(new Rectangle(0, 0, 300, 140));
				centerAboveAnnotation(annotEditPanel, (CakeAnnotationThing) t);
			}
		}
	}
	
	protected void centerAboveAnnotation(ResizableLocalBoxBoundedThing lbbt, CakeAnnotationThing t)
	{
		BNAComponent c = getBNAComponent();
		CoordinateMapper cm = c.getCoordinateMapper();
		Rectangle lbbtBoundingBox = lbbt.getLocalBoundingBox();

		int lbbtWidth = lbbtBoundingBox.width;
		int lbbtHeight = lbbtBoundingBox.height;
		
		// Center position of thing
		int tx = t.getWidth() / 2;
		int ty = t.getHeight() / 2;
		
		// Delta between center positions of box and thing
		int dx = tx - (lbbtWidth / 2);
		int dy = ty - (lbbtHeight / 2);
		
		// New x and y coordinates of box so as to be centered around thing
		// Convert values to local coordinates since SwingPanelThings are drawn in local coords
		// and things store their values in world coords.
		int bx = cm.worldXtoLocalX(t.getX() + dx);
		int by = cm.worldYtoLocalY(t.getY() + dy);

		lbbtBoundingBox.setLocation(bx, by);
		lbbt.setLocalBoundingBox(lbbtBoundingBox); 
	}

	private class EditAnnotationPanel extends JPanel implements WindowHeaderPanelListener, ActionListener
	{	
		protected SwingPanelThing swingPanelThing;
		protected CakeAnnotationThing annotThingtoEdit;

		protected JTextArea taAnnotation;
		protected JButton bOK;

		/**
		 * @param descEditPanel
		 * @param thingToEdit
		 */
		public EditAnnotationPanel(SwingPanelThing annotEditPanel, Thing thingToEdit)
		{
			super();
			this.swingPanelThing = annotEditPanel;
			this.annotThingtoEdit = (CakeAnnotationThing) thingToEdit;

			this.setLayout(new BorderLayout());

			WindowHeaderPanel whp = new WindowHeaderPanel("Edit Annotation");
			whp.addWindowHeaderPanelListener(this);

			this.add("North", whp);

			taAnnotation = new JTextArea(5, 20);	
			taAnnotation.setLineWrap(true);			
			JScrollPane scrollPane = new JScrollPane(taAnnotation);								

			bOK = new JButton("OK");
			bOK.addActionListener(this);

			JPanel centerPanel = new JPanel();
			centerPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
			centerPanel.add(scrollPane);
			centerPanel.add(bOK);

			this.add("Center", centerPanel);
			annotEditPanel.setPanel(this);
			annotEditPanel.setIndicatorThingId(thingToEdit.getID());
			updateTextArea();
			
			//make the text field gain focus
			taAnnotation.addAncestorListener(
				new javax.swing.event.AncestorListener(){
					public void ancestorAdded(javax.swing.event.AncestorEvent event){
						taAnnotation.requestFocus();
					}
					public void ancestorRemoved(javax.swing.event.AncestorEvent event){}
					public void ancestorMoved(javax.swing.event.AncestorEvent event){}
				}
			);
		}

		/**
		 * 
		 */
		protected void updateTextArea()
		{										
			taAnnotation.setText(annotThingtoEdit.getLabel());
		}

		/* (non-Javadoc)
		 * @see edu.uci.ics.widgets.windowheader.WindowHeaderPanelListener#closeButtonPressed(edu.uci.ics.widgets.windowheader.WindowHeaderPanel)
		 */
		public void closeButtonPressed(WindowHeaderPanel src)
		{
			remove();
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent evt)
		{
			if(evt.getSource() == bOK)
			{
				String newValue = taAnnotation.getText();
				annotThingtoEdit.setLabel(newValue);					
			
				CakeUtils.setAnnotationComment(xarch, modelXArch, thingIDMap, annotThingtoEdit, newValue);					
													
				remove();
			}
		}
	
		public void remove()
		{
			getBNAComponent().getModel().removeThing(swingPanelThing);
		}
	}		
}
