/*
 * Created on May 31, 2004
 *
 * CakeEditDescriptionContextMenuPlugin.java
 */
package archstudio.comp.cake;

import archstudio.comp.archipelago.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import edu.uci.ics.bna.*;
import edu.uci.ics.bna.contextmenu.*;
import edu.uci.ics.bna.swingthing.*;
import edu.uci.ics.widgets.windowheader.*;

import edu.uci.ics.xarchutils.*;

public class CakeEditDescriptionContextMenuPlugin extends AbstractSelectionBasedContextMenuPlugin{
	
	protected XArchFlatInterface xarch;
	protected ThingIDMap thingIDMap;
	private ObjRef modelXArch;
	
	public CakeEditDescriptionContextMenuPlugin(BNAComponent c, 
		ThingIDMap thingIDMap, InterfaceMap intfMap, XArchFlatInterface xarch, ObjRef modelXArch)
	{
		super(c);
		this.thingIDMap = thingIDMap;
		this.xarch = xarch;
		this.modelXArch = modelXArch;
	}

	public JPopupMenu addToContextMenu(JPopupMenu currentContextMenu, Thing[] selectedThingSet, Thing thingUnderCursor){
		Thing t = null;
		if(selectedThingSet.length == 0){
			if(thingUnderCursor != null){
				t = thingUnderCursor;
			}
			else{
				return currentContextMenu;
			}
		}
		else if(selectedThingSet.length > 1){
			return currentContextMenu;
		}
		else{
			t = selectedThingSet[0];
		}
		
		if(
		(t instanceof CakeComponentThing) ||
		(t instanceof CakeConnectorThing) ||
		(t instanceof CakeLinkThing)
		//(t instanceof CakeInterfaceThing)
		){
			EditDescriptionMenuItemSet edmis = new EditDescriptionMenuItemSet(t);
			JMenuItem[] miArray = edmis.getMenuItemSet();
			for(int i = 0; i < miArray.length; i++){
				currentContextMenu.add(miArray[i]);
			}
		}
		
		return currentContextMenu;
	}
	
	protected void centerInComponent(ResizableLocalBoxBoundedThing lbbt){
		Rectangle lbbtBoundingBox = lbbt.getLocalBoundingBox();
		
		int lbbtWidth = lbbtBoundingBox.width;
		int lbbtHeight = lbbtBoundingBox.height;
		
		Dimension bnaSize = getBNAComponent().getSize();
		
		int bnacx = bnaSize.width / 2;
		int bnacy = bnaSize.height / 2;
		
		int ulx = bnacx - (lbbtWidth / 2);
		int uly = bnacy - (lbbtHeight / 2);
		
		lbbtBoundingBox.setLocation(ulx, uly);
		lbbt.setLocalBoundingBox(lbbtBoundingBox);
	}
	
	class EditDescriptionMenuItemSet implements ActionListener{
		protected Thing thingToEdit;
		protected JMenuItem miEditDescription;
		
		public EditDescriptionMenuItemSet(Thing thingToEdit){
			this.thingToEdit = thingToEdit;
			miEditDescription = new JMenuItem("Edit Description...");
			miEditDescription.addActionListener(this);
		}
		
		public JMenuItem[] getMenuItemSet(){
			return new JMenuItem[]{miEditDescription};
		}
		
		public void actionPerformed(ActionEvent evt){
			SwingPanelThing descEditPanel = new SwingPanelThing();
			EditDescriptionPanel edp = new EditDescriptionPanel(descEditPanel, thingToEdit);
			
			getBNAComponent().getModel().addThing(descEditPanel);
			descEditPanel.setLocalBoundingBox(new Rectangle(0, 0, 300, 90));
			centerInComponent(descEditPanel);
		}

		class EditDescriptionPanel extends JPanel implements WindowHeaderPanelListener, ActionListener{
			protected Thing swingPanelThing;
			protected Thing thingToEdit;
		
			protected JTextField tfDescription;
			protected JButton bOK;
		
			public EditDescriptionPanel(SwingPanelThing swingPanelThing, Thing thingToEdit){
				super();
				this.swingPanelThing = swingPanelThing;
				this.thingToEdit = thingToEdit;
		
				this.setLayout(new BorderLayout());
			
				WindowHeaderPanel whp = new WindowHeaderPanel("Edit Description");
				whp.addWindowHeaderPanelListener(this);
			
				this.add("North", whp);
			
				tfDescription = new JTextField(15);
				tfDescription.addActionListener(this);
			
				bOK = new JButton("OK");
				bOK.addActionListener(this);
			
				JPanel centerPanel = new JPanel();
				centerPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
				centerPanel.add(tfDescription);
				centerPanel.add(bOK);
			
				this.add("Center", centerPanel);
				swingPanelThing.setPanel(this);
				swingPanelThing.setIndicatorThingId(thingToEdit.getID());
				updateTextField();
				
				//make the text field gain focus
				tfDescription.addAncestorListener(
					new javax.swing.event.AncestorListener(){
						 public void ancestorAdded(javax.swing.event.AncestorEvent event){
							tfDescription.requestFocus();
						 }
						 public void ancestorRemoved(javax.swing.event.AncestorEvent event){}
						 public void ancestorMoved(javax.swing.event.AncestorEvent event){}
					}
				);
			}
		
			protected void updateTextField(){
				String thingToEditId = thingToEdit.getID();
				ObjRef thingToEditRef = thingIDMap.getXArchRef(thingToEditId);
				if(thingToEditRef != null){
					String description = edu.uci.ics.xadlutils.XadlUtils.getDescription(xarch, thingToEditRef);
					tfDescription.setText(description);
				}
			}
			
			protected void setNewValue(String newValue){
				String thingToEditId = thingToEdit.getID();
				ObjRef thingToEditRef = thingIDMap.getXArchRef(thingToEditId);
				
				if(thingToEdit instanceof CakeComponentThing || 
				   thingToEdit instanceof CakeConnectorThing ||
				   thingToEdit instanceof CakeLinkThing)
				{	
					ObjRef elementToEditRef = CakeUtils.resolveXMLLink(xarch, modelXArch, thingToEditRef);			
					if(elementToEditRef != null)
					{
						edu.uci.ics.xadlutils.XadlUtils.setDescription(xarch, elementToEditRef, newValue);
					}
				}				
			}

			public void closeButtonPressed(WindowHeaderPanel src){
				remove();
			}
			
			public void actionPerformed(ActionEvent evt){
				if(evt.getSource() == bOK){
					String newDescription = tfDescription.getText();
					remove();
					setNewValue(newDescription);
				}
				if(evt.getSource() == tfDescription){
					bOK.doClick();
				}
			}
			
			public void remove(){
				getBNAComponent().getModel().removeThing(swingPanelThing);
			}
		}
	}
}
