package archstudio.comp.cake;

import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.Border;

import archstudio.comp.archipelago.ThingIDMap;
import c2.util.UIDGenerator;
import edu.uci.ics.bna.*;
//import edu.uci.ics.bna.contextmenu.SelectionBasedContextMenuLogic;
import edu.uci.ics.bna.swingthing.*;
import edu.uci.ics.xarchutils.*;



public class CakeFrame extends JFrame implements XArchFlatListener
{
	public static final int SRC_NOT_OPEN = 100;
	public static final int SRC_FILE = 200;
	public static final int SRC_URL = 300;
	
	private int documentSourceType;
	private String documentSource;
	private ObjRef documentRef;
	private XArchFlatInterface xarch;
	
	// this is a list of the xarch flat event listeners
	private Vector flatListeners;
	private CheckNodeTreeExample versionTree;
	private ThingIDMap thingIDMap;
	private InterfaceMap interfaceMap;
	private VersionColorMap versionColorMap;
	
	private CakeBNAComponent bnaComponent;
	private BNAModel bnaModel;
	private Palette palette;
	
	/**
	 * @param xarch
	 * @param url
	 */
	public CakeFrame(XArchFlatInterface xarch, String url)
	{
		super();
		//System.out.println("Hello!");
		this.xarch = xarch;
		setDocumentSource(url);
		
		documentRef = xarch.getOpenXArch(url);
		if(documentRef == null)
		{
			JOptionPane.showMessageDialog(this,"Must have an open architecture", "Error",
				JOptionPane.ERROR_MESSAGE);
		}
		init();
		
	}
	
	private void init()
	{
		thingIDMap = new ThingIDMap();
		interfaceMap = new InterfaceMap();
		versionColorMap = new VersionColorMap();
		
		// Create relevent BNA stuff
		bnaModel = new DefaultBNAModel();
		bnaComponent = new CakeBNAComponent(UIDGenerator.generateUID("Cake"), bnaModel);
		bnaComponent.setBackground(Color.WHITE);
		
		// create the tree logic stuff
		TreeLogic treeLogic = new TreeLogic(xarch, thingIDMap, interfaceMap, 
			documentRef);
		
		flatListeners = new Vector(10);
		CakeModelWrapper modelWrapper = new CakeModelWrapper(xarch, documentRef, bnaComponent, versionColorMap, 
			thingIDMap, interfaceMap);
			
		// create the various components
		versionTree = new CheckNodeTreeExample(modelWrapper, treeLogic, versionColorMap);
					
		// Add necessary logics
		ModeTrackingLogic modeTrackingLogic = new ModeTrackingLogic();
		bnaComponent.addThingLogic(modeTrackingLogic);
		SelectionTrackingLogic stl = new SelectionTrackingLogic();
		bnaComponent.addThingLogic(stl);
		BoundingBoxTrackingLogic bbtl = new BoundingBoxTrackingLogic();
		bnaComponent.addThingLogic(bbtl);
		StickyBoxTrackingLogic sbtl = new StickyBoxTrackingLogic();
		
		bnaComponent.addThingLogic(sbtl);
		bnaComponent.addThingLogic(treeLogic);
		
		bnaComponent.addThingLogic(new AddEntityOnClickLogic(xarch, documentRef, versionTree, 
			bnaComponent, thingIDMap, interfaceMap, versionColorMap, modeTrackingLogic));
		bnaComponent.addThingLogic(new DeleteEntityLogic(xarch, thingIDMap, interfaceMap,
			modeTrackingLogic));
		bnaComponent.addThingLogic(new LayeredDragMovableSelectionLogic(modeTrackingLogic));
		bnaComponent.addThingLogic(new LayeredNoThingLogic(modeTrackingLogic));
		bnaComponent.addThingLogic(new LayeredOneClickSelectionLogic(modeTrackingLogic));
		bnaComponent.addThingLogic(new BoxReshapeLogic(stl, bbtl));
		bnaComponent.addThingLogic(new LayeredSplineReshapeLogic(stl, modeTrackingLogic));
		bnaComponent.addThingLogic(new StickySplineLogic(sbtl));
		bnaComponent.addThingLogic(new CustomCursorLogic());
		bnaComponent.addThingLogic(new LayeredToolTipLogic());
		bnaComponent.addThingLogic(new LayeredMoveEndpointLogic(bbtl, modeTrackingLogic));
		bnaComponent.addThingLogic(new CakeMoveIndicatorPointLogic(bbtl));		
		bnaComponent.addThingLogic(new CakeEditAnnotationDoubleClickLogic(thingIDMap, xarch, documentRef));
		
		// swing integration stuff
		bnaComponent.addThingLogic(new LocalDragMovableLogic());
		bnaComponent.addThingLogic(new OptionalScrollLocalBoxBoundedThingLogic());
		
		LocalBoundingBoxTrackingLogic lbbtl = new LocalBoundingBoxTrackingLogic();
		bnaComponent.addThingLogic(lbbtl);
		bnaComponent.addThingLogic(new LocalBoxReshapeLogic(stl, lbbtl));
		
		//Context Menu Logics:
		LayeredSelectionBasedContextMenuLogic sbcml = 
			new LayeredSelectionBasedContextMenuLogic(stl);
		sbcml.addPlugin(new CakeEditDescriptionContextMenuPlugin(bnaComponent, 
			thingIDMap, interfaceMap, xarch, documentRef));
		sbcml.addPlugin(new CakeAddEditInterfaceContextMenuPlugin(bnaComponent, thingIDMap, interfaceMap, 
			xarch, documentRef));
		sbcml.addPlugin(new CakeEditAnnotationContextMenuPlugin(bnaComponent, 
			thingIDMap, xarch, documentRef));
//		sbcml.addPlugin(new EditDirectionContextMenuPlugin(structureBNAComponent, thingIDMap, xarch));
//		sbcml.addPlugin(new OrderContextMenuPlugin(structureBNAComponent));
//		sbcml.addPlugin(new AlignContextMenuPlugin(structureBNAComponent));
//		sbcml.addPlugin(new DistributeContextMenuPlugin(structureBNAComponent));
		bnaComponent.addThingLogic(sbcml);
		
		
		// create the mapping logic that listens to xArch events and
		// changes the BNA model
		CakeBrickMappingLogic brickMappingLogic = new CakeBrickMappingLogic(
			bnaComponent, xarch, thingIDMap, interfaceMap);
		VersionTreeMappingLogic versionTreeMappingLogic = new VersionTreeMappingLogic(
			bnaComponent, xarch, versionTree);		
		CakeLinkMappingLogic linkMappingLogic = new CakeLinkMappingLogic(bnaComponent,
			xarch, thingIDMap, interfaceMap);		
		CakeAnnotationMappingLogic annotationMappingLogic = new CakeAnnotationMappingLogic(bnaComponent, 
			xarch, thingIDMap, documentRef);
		LayersMappingLogic layersMappingLogic = new LayersMappingLogic(bnaComponent, 
			xarch, thingIDMap, interfaceMap, brickMappingLogic, linkMappingLogic,
			annotationMappingLogic);
		
		// add our XArchFlat listenders
		bnaModel.addBNAModelListener(versionTreeMappingLogic);
		flatListeners.add(versionTreeMappingLogic);
		bnaModel.addBNAModelListener(brickMappingLogic);
		flatListeners.add(brickMappingLogic);
		bnaModel.addBNAModelListener(linkMappingLogic);
		flatListeners.add(linkMappingLogic);
		bnaModel.addBNAModelListener(annotationMappingLogic);
		flatListeners.add(annotationMappingLogic);
		bnaModel.addBNAModelListener(layersMappingLogic);
		flatListeners.add(layersMappingLogic);
			
		// now populate the tree model with whats already in the document
		versionTreeMappingLogic.populateTreeModel(documentRef);
		
		//Add the Palette thing
		palette = new Palette();
		bnaModel.addThing(palette);
		
		// set up UI stuff
		Toolkit tk = getToolkit();
		Dimension screenSize = tk.getScreenSize();
		double xPos = (screenSize.getWidth() * 0.1);
		double yPos = (screenSize.getHeight() * 0.1);
		int xSize = (int) (screenSize.getWidth() * 0.80);
		int ySize = (int) (screenSize.getHeight() * 0.80);
		
		Border line = BorderFactory.createRaisedBevelBorder();
		versionTree.setBorder(line);
		this.getContentPane().add(versionTree, BorderLayout.EAST);
		
		line = BorderFactory.createRaisedBevelBorder();
		ScrollableBNAComponent scrollableBNAComponent = new 
			ScrollableBNAComponent(bnaComponent, true, true, true);
			
		scrollableBNAComponent.setBorder(line);
		this.getContentPane().add(scrollableBNAComponent, BorderLayout.CENTER);
		
		
		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(
			new WindowAdapter(){
				public void windowClosing(WindowEvent evt){
					closeWindow();
				}
			}
			);
		
		bnaComponent.setFocusable(true);
		requestFocus();
		bnaComponent.requestFocus();
		
		setWindowTitle(documentSource);
		setVisible(true);
		setSize(xSize, ySize);
		setLocation( ( int )xPos, ( int )yPos );
		setVisible(true);
		paint( getGraphics( ) );
		//System.out.println("exitting");	
	}
	
	//* * * * * * * * * * * * * * * * * * * *
	//Main
	//* * * * * * * * * * * * * * * * * * * *
	/*
	public static void main(String args[]) {
	  CakeFrame frame = new CakeFrame();
	  frame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {System.exit(0);}
	  });
	  frame.setSize(800, 500);
	  frame.setVisible(true);
	}
	*/
	private void closeWindow()
	{
		doClose();
		this.setVisible(false);
		this.dispose();
	}
	
	public void setDocumentSource(String url)
	{
		this.documentSource = url;
		if(url == null)
		{
			this.documentSourceType = SRC_NOT_OPEN;
		}
		else if(url.startsWith("file:"))
		{
			this.documentSourceType = SRC_FILE;
		}
		else
		{
			this.documentSourceType = SRC_URL;
		}
	}
			
	public int getDocumentSourceType()
	{
		return documentSourceType;
	}
	
	public String getDocumentSource()
	{
		return documentSource;
	}

	/**
	 * 
	 */
	public void doClose()
	{
		// Do clean up stuff
		
		setWindowTitle(null);
		documentSourceType = SRC_NOT_OPEN;
		documentSource = null;
	}
	
	// This function sets the title on the window bar
	// its the product name, version, and then the file
	private void setWindowTitle(String openFile)
	{
		StringBuffer title = new StringBuffer();
		title.append(CakeC2Component.PRODUCT_NAME);
		title.append(' ');
		title.append(CakeC2Component.PRODUCT_VERSION);
		if(openFile == null)
		{
			title.append(" - [None]");
		}
		else
		{
			title.append(" - [").append(openFile).append("]");
		}
		setTitle(title.toString());
	}

	/* 
	 * 
	 */
	public void handleXArchFlatEvent(XArchFlatEvent evt)
	{
		int length = flatListeners.size();
		for(int i = 0; i < length; i++)
		{
			XArchFlatListener list = (XArchFlatListener)flatListeners.elementAt(i);
			
			list.handleXArchFlatEvent(evt);
		}
		
	}
}
