/*
 * Created on Jun 4, 2004
 *
 * CakeLinkMappingLogic.java
 */
package archstudio.comp.cake;

import java.awt.Point;
import java.awt.Rectangle;

import edu.uci.ics.bna.*;
import edu.uci.ics.xadlutils.XadlUtils;
import edu.uci.ics.xarchutils.*;
import archstudio.comp.archipelago.ThingIDMap;


/**
 * @author Ping
 *
 * This class provides the mapping between xADL links and the splines in the BNA
 * world.  It listens for updates from the xADL model and reconnects the links
 * if possible.  And also will modify the xADL model if the links were reconnected
 * in the BNA world
 */
public class CakeLinkMappingLogic extends CakeAbstractMappingLogic
{

	protected ThingIDMap thingIDMap;
	protected InterfaceMap intfMap;
	//protected BNAComponent structureBNAComponent;

	protected BNAModel bnaModel;

	static int offset = 0;

	public CakeLinkMappingLogic(BNAComponent bnaComponent, XArchFlatInterface xarch, 
		ThingIDMap thingIDMap, InterfaceMap iMap)
	{
		super(bnaComponent, xarch);
		//this.typesBNAComponent = bnaComponents[0];
	
		//this.typesBNAModel = bnaComponents[0].getModel();
		this.bnaModel = bnaComponent.getModel();
		this.thingIDMap = thingIDMap;
		this.intfMap = iMap;
	
		//linkParent = new NoThing("$$LinkParent");
		//structureBNAModel.addThing(linkParent);
	}

	public void handleXArchFlatEvent(XArchFlatEvent evt)
	{
		/*
		try
		{
		*/
		//System.out.println("Handling events! Here");
		//System.out.println(evt);
		ObjRef src = evt.getSource();
		ObjRef docRef = xarch.getXArch(src);
		
		XArchPath path = evt.getTargetPath();
		if(path == null || evt.getEventType() == XArchFlatEvent.REMOVE_EVENT)
		{
			// Remove event the target path is just the element
		  	// so we want the source path instead.
			path = evt.getSourcePath();
		}
		if(path == null)
		{
			//System.out.println("Null again");
			return;
		}
		
		if(evt.getEventType() == XArchFlatEvent.REMOVE_EVENT &&
		   evt.getTargetName().equals("link"))
		{
			ObjRef target = (ObjRef)evt.getTarget();
			// we are removing a link, so gotta go through
		    // and update all the containingElements and remove them
			removeLink((ObjRef)evt.getTarget(), docRef);
		  	// done handling this event
			return;
		}
		String pathString = path.toTagsOnlyString();
		//System.out.println("pathString = " + pathString);
		if(pathString.startsWith("xArch/archStructure/link"))
		{
			//System.out.println("Link");
		  	//It's a component event
		  	if(src != null)
		  	{
				ObjRef[] ancestors = xarch.getAllAncestors(src);
				// 3 is where the actual link is...
				ObjRef linkRef = ancestors[ancestors.length - 3];
				//System.out.println("Updating Link");
				updateLink(linkRef, docRef);
			}
		}
		/*
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		*/
	}

	public void handleXArchFileEvent(XArchFileEvent evt)
	{
		// empty...
	}
	
	/**
	 * This function handles the removal of links by removing
	 * all the containing elements.  We let another logic
	 * handle the actual deletion of BNA things when we capture
	 * the removal of ContainingELements
	 * 
	 * @param linkRef The ObjRef of the link that was removed
	 * @param docRef The ObjRef of the IXArch element for the document
	 */
	public void removeLink(ObjRef linkRef, ObjRef docRef)
	{
		ObjRef[] references = xarch.getReferences(docRef, 
			(String) xarch.get(linkRef, "Id"));
		//System.out.println(references.length);
		for(int i = 0; i < references.length; i++)
		{
			ObjRef parent = xarch.getParent(references[i]);
			// find the references that are of type LayeredXMLLink
			// since these are the ones that are used in containingElement
			// for layers.
			if(xarch.isInstanceOf(references[i], 
				"edu.uci.isr.xarch.layers.ILayeredXMLLink"))
			{
				xarch.remove(parent, "ContainingElement", references[i]);
			}
			/*  links can't be annotated anyways...
			else if(xarch.isInstanceOf(parent,
				"edu.uci.isr.xarch.annotations.IAnnotation"))
			{
				xarch.clear(references[i], "Href");
			}
			*/
		}
	}

	protected void updateLink(ObjRef linkRef, ObjRef docRef)
	{
		//System.out.println("updateLink called.");
		
		
		ObjRef[] references = xarch.getReferences(docRef, 
			(String) xarch.get(linkRef, "Id"));
		//System.out.println(references.length);
		
		String label = "";
		String ltDesc = edu.uci.ics.xadlutils.XadlUtils.getDescription(
			xarch, linkRef);
		if(ltDesc != null) 
			label = ltDesc;
			
		//System.out.println("Label should be... " + label);
		
		CakeUtils.LinkInfo li = CakeUtils.getLinkInfo(xarch, linkRef, true);
		for(int i = 0; i < references.length; i++)
		{
			updateLinkThing(references[i], li, label, docRef);
		}
	}

	public void updateLinkThing(ObjRef containingElement, ObjRef docRef)
	{
		ObjRef linkRef = CakeUtils.resolveXMLLink(xarch, docRef, containingElement);
		//System.out.println("Here " + linkRef);
		if(linkRef != null)
		{
			String label = "";
			String ltDesc = edu.uci.ics.xadlutils.XadlUtils.getDescription(
				xarch, linkRef);
			if(ltDesc != null) 
				label = ltDesc;
		
			//System.out.println("Label should be... " + label);
			CakeUtils.LinkInfo li = CakeUtils.getLinkInfo(xarch, linkRef, true);
			updateLinkThing(containingElement, li, label, docRef);
		}
	}
	
	public void updateLinkThing(ObjRef containingElement, CakeUtils.LinkInfo li, 
		String label, ObjRef docRef)
	{

		CakeLinkThing linkThing;
		String existingThingID = thingIDMap.getThingID(containingElement);
		if(existingThingID != null)
		{
			try
			{
				linkThing = (CakeLinkThing)bnaModel.getThing(existingThingID);
			}
			catch(ClassCastException cce)
			{
				System.err.println("Warning; ID/type mismatch.");
				cce.printStackTrace();
				return;
			}
			// update the label, make sure its correct
			//System.out.println("Updating its label");
			linkThing.setLabel(label);
	
			if(li.getPoint1Ref() != null )
			{
				//System.out.println("Updating 1st point");
				updatePoint(linkThing, li, docRef, true);
			}
			if(li.getPoint2Ref() != null)
			{
				//System.out.println("Updating 2nd point");
				updatePoint(linkThing, li, docRef, false);
			}
		}
	}
	// This function updates the spline thing's endpoints as appropriate
	private void updatePoint(
		CakeLinkThing linkThing,
		CakeUtils.LinkInfo li,
		ObjRef docRef,
		boolean isFirstPoint)
	{
		ObjRef interfaceTarget;
		String id = null;
		if(isFirstPoint)
		{
			interfaceTarget = li.getPoint1Target();
			//obtain the Thing that the link is stuck to
			id = linkThing.getFirstEndpointStuckToID();
		}
		else
		{
			interfaceTarget = li.getPoint2Target();
			//obtain the Thing that the link is stuck to	
			id = linkThing.getSecondEndpointStuckToID();
		}
		ObjRef endPointRef = null;
		
		if(id != null)
		{
		
			CakeInterfaceThing endPoint = 
				(CakeInterfaceThing)bnaModel.getThing(id);
			if(endPoint != null)
			{
				endPointRef = CakeUtils.getInterfaceRef(endPoint,
					thingIDMap, intfMap);
			}
		}
		if(endPointRef == null && interfaceTarget == null)
		{
			//System.out.println("both null, returning");
			return;
		}
		else if(interfaceTarget == null)
		{
			// the target doesn't exist, just reset the link
			//System.out.println("Target is null");
			if(isFirstPoint)
			{
				linkThing.setPointAt(new Point(bnaComponent.getCoordinateMapper().localXtoWorldX(0), 
					bnaComponent.getCoordinateMapper().localYtoWorldY(0)), 0);
				linkThing.setFirstEndpointStuckToID(null);
			}
			else
			{
				linkThing.setPointAt(new Point(bnaComponent.getCoordinateMapper().localXtoWorldX(0) + 10, 
					bnaComponent.getCoordinateMapper().localYtoWorldY(0) + 10),
					linkThing.getNumPoints() - 1);
				linkThing.setSecondEndpointStuckToID(null);
			}
			
		}
		else if(endPointRef == null || !endPointRef.equals(interfaceTarget))
		{

			// the endpoint has changed.. oh no!!!
			// make some effort at reconnecting it... guessing game
			String targetID = getNewTargetThing(interfaceTarget, docRef, linkThing);
			if(targetID != null)
			{
				//System.out.println("Found a new target");
				// found a likely target... gonna hook it up and see!
				Thing interfaceThing = bnaModel.getThing(targetID);
				if(interfaceThing instanceof CakeInterfaceThing)
				{
					Rectangle stickyBox = 
						((CakeInterfaceThing)interfaceThing).getStickyBox();
					
					if(isFirstPoint)
					{
						linkThing.setPointAt(new Point(stickyBox.x, stickyBox.y), 0);
						linkThing.setFirstEndpointStuckToID(targetID);
					}
					else
					{
						linkThing.setPointAt(new Point(stickyBox.x, stickyBox.y), 
							linkThing.getNumPoints() - 1);
						linkThing.setSecondEndpointStuckToID(targetID);
					}
					
				}
			}
			else
			{
				//System.out.println("Just setting it to empty space");
				if(isFirstPoint)
				{
					linkThing.setPointAt(new Point(bnaComponent.getCoordinateMapper().localXtoWorldX(0), 
						bnaComponent.getCoordinateMapper().localYtoWorldY(0)), 0);
					linkThing.setFirstEndpointStuckToID(null);
				}
				else
				{
					linkThing.setPointAt(new Point(bnaComponent.getCoordinateMapper().localXtoWorldX(0) + 10, 
						bnaComponent.getCoordinateMapper().localYtoWorldY(0) + 10),
						linkThing.getNumPoints() - 1);
					linkThing.setSecondEndpointStuckToID(null);
				}
				
			}				
		}
	}

	// Thins function returns the ID of the interface Thing that 
	// we the link should be hooked up with.
	private String getNewTargetThing(ObjRef interfaceTarget, ObjRef docRef,
		CakeLinkThing linkThing)
	{
		ObjRef parentRef = xarch.getParent(interfaceTarget);
		ObjRef[] references = xarch.getReferences(docRef, 
			(String)xarch.get(parentRef, "Id"));
		String targetID = null;
		if(references.length == 1)
		{
			// only 1 thing, no need to guess, it has to be it.
			targetID = intfMap.getInterfaceThingID(references[0], interfaceTarget);
		}
		for(int i = 0; i < references.length; i++)
		{
			// find the references that are of type LayeredXMLLink
			// since these are the ones that are used in containingElement
			// for layers.
			if(xarch.isInstanceOf(references[i], 
				"edu.uci.isr.xarch.layers.ILayeredXMLLink"))
			{
				ObjRef layer = xarch.getParent(references[i]);
				ObjRef layerSet = xarch.getParent(layer);
				
				String version = (String)xarch.get(layerSet, "Id");
				if(version != null && version.equals(linkThing.getVersion()))
				{
					// guesses that the link is going to be connected to someone
					// within the same version (layerSet)
					targetID = intfMap.getInterfaceThingID(references[i], 
						interfaceTarget);
					break;
				}
			}
		}
		return targetID;
	}

	public void bnaModelChanged(BNAModelEvent evt)
	{
		
		Thing t = evt.getTargetThing();
		if((t != null) && (t instanceof CakeLinkThing))
		{
			CakeLinkThing lt = (CakeLinkThing)t;
			ThingEvent thingEvent = evt.getThingEvent();
			if(thingEvent != null)
			{
				// find out the name of property that was changed
				String propertyName = thingEvent.getPropertyName();
				if(propertyName != null)
				{
					if(propertyName.equals(CakeLinkThing.FIRST_ENDPOINT_STUCK_TO_ID_PROPERTY_NAME))
					{
						updateEndPoint(thingEvent, lt, true);
					}
					else if(propertyName.equals(CakeLinkThing.SECOND_ENDPOINT_STUCK_TO_ID_PROPERTY_NAME))
					{
						updateEndPoint(thingEvent, lt, false);
					}
				}
			}
		}
	}
		
	protected void updateEndPoint(ThingEvent thingEvent, CakeLinkThing lt, 
		boolean isFirstEndPoint)
	{
		String oldValue = (String)thingEvent.getOldPropertyValue();
		String newValue = (String)thingEvent.getNewPropertyValue();
		// make sure the values actually changed
		if(oldValue == newValue)
		{
			return;
		}
		else
		{
			if((oldValue != null) && (newValue != null))
			{
				if(oldValue.equals(newValue))
				{
					return;
				}
			}
		}

		ObjRef anchorRef = null;
		String href = null;
		ObjRef pointRef = null;
		
		ObjRef containingElement = thingIDMap.getXArchRef(lt.getID());
		ObjRef docRef = xarch.getXArch(containingElement);
		ObjRef linkRef = CakeUtils.resolveXMLLink(xarch, docRef, 
			containingElement);
		if(linkRef != null)
		{
			CakeUtils.LinkInfo li = CakeUtils.getLinkInfo(xarch, linkRef, false);

			if(isFirstEndPoint)
			{
				pointRef = li.getPoint1Ref();
				anchorRef = li.getAnchor1Ref();
				href = li.getAnchor1Href();
			}
			else
			{
				pointRef = li.getPoint2Ref();
				anchorRef = li.getAnchor2Ref();
				href = li.getAnchor2Href();
			}
			//Two cases: either it was stuck to something or it was unstuck
			//from something
			String stuckToID = (String)thingEvent.getNewPropertyValue();
			if(stuckToID == null)
			{
				if((href == null) || (href.equals("")))
				{
					//prevent infinite loop
					//System.out.println("stopping the madness1");
					return;
				}
				//Unhook the link
				if(anchorRef != null)
				{
					//xarch.clear(anchorRef, "type");
					xarch.clear(anchorRef, "href");
				}
			}
			else
			{
				//Hook up the link
				Thing intfThing = bnaModel.getThing(stuckToID);
				ObjRef stuckToRef = null;
				// find the interface ref of the new stuck to thing
				if(intfThing != null && intfThing instanceof CakeInterfaceThing)
				{
					stuckToRef = CakeUtils.getInterfaceRef((CakeInterfaceThing)intfThing,
						thingIDMap, intfMap);
				}

				if(stuckToRef != null)
				{
					String stuckToRefID = XadlUtils.getID(xarch, stuckToRef);
					if(stuckToRefID != null)
					{
						if((href != null) && (href.endsWith("#" + stuckToRefID)))
						{
							//prevent infinite loop
							//System.out.println("stopping the madness2");
							return;
						}
	
						ObjRef typesContextRef = null;
						if((pointRef == null) || (anchorRef == null))
						{
							ObjRef xArchRef = xarch.getXArch(linkRef);										
							typesContextRef = xarch.createContext(xArchRef, "types");
						}
						if(pointRef == null)
						{
							//Add the point + anchor
							pointRef = xarch.create(typesContextRef, "point");
							xarch.add(linkRef, "point", pointRef);
						}
						if(anchorRef == null)
						{
							//Add the anchor
							anchorRef = xarch.create(typesContextRef, "XMLLink");
							xarch.set(pointRef, "anchorOnInterface", anchorRef);
						}
						xarch.set(anchorRef, "type", "simple");
						xarch.set(anchorRef, "href", "#" + stuckToRefID);
					}
				}
			}
		}
	}
}
