/*
 * Created on May 23, 2004
 *
 */
package archstudio.comp.cake;

import java.awt.Rectangle;
import java.util.Collection;
import java.util.Iterator;

import archstudio.comp.archipelago.ThingIDMap;
import c2.util.UIDGenerator;
import edu.uci.ics.bna.BNAComponent;
import edu.uci.ics.bna.BNAModel;
import edu.uci.ics.xarchutils.ObjRef;
import edu.uci.ics.xarchutils.XArchFlatInterface;
import edu.uci.isr.xarch.IXArchElement;

/**
 * @author Chris
 *
 */
public class CakeModelWrapper implements ICakeModelWrapper
{
	
	protected XArchFlatInterface xarch;
	protected ObjRef modelXArch;
	protected ObjRef layersContext;
	protected ObjRef versionsContext;
	protected ObjRef instanceContext;
	protected ObjRef archLayers;
	protected ObjRef archLayerVersions;
	protected ObjRef layerSetVersionGraph;
	protected BNAComponent bnaComponent;
	protected BNAModel bnaModel;
	protected VersionColorMap versionColorMap;
	protected ThingIDMap thingIDMap;
	protected InterfaceMap interfaceMap;
	
	// Keeps track of the most recently added version number
	private int recentVersion;
	
	  
	
	public CakeModelWrapper(XArchFlatInterface xarch, ObjRef modelXArch, BNAComponent bnaComponent, 
		VersionColorMap versionColorMap, ThingIDMap thingIDMap, InterfaceMap interfaceMap)
	{
		//System.out.println("Entering Wrapper");
		
		this.xarch = xarch;
		this.modelXArch = modelXArch;
		
		this.bnaComponent = bnaComponent;
		this.versionColorMap = versionColorMap;
		this.thingIDMap = thingIDMap;
		this.interfaceMap = interfaceMap;
		this.bnaModel = bnaComponent.getModel();
		
		// Create contexts
		layersContext = xarch.createContext(modelXArch, "Layers");
		versionsContext = xarch.createContext(modelXArch, "Versions");
		instanceContext = xarch.createContext(modelXArch, "Instance");
		
		// Get the ArchLayers child		
		archLayers = xarch.getElement(layersContext, "ArchLayers", modelXArch);
		if(archLayers == null)
		{
			archLayers = xarch.createElement(layersContext, "ArchLayers");
			xarch.add(modelXArch, "Object", archLayers);
		}
		
		// Get the ArchLayerVersions child		
		archLayerVersions = xarch.getElement(versionsContext, "ArchVersions", modelXArch);
		if(archLayerVersions == null)
		{
			archLayerVersions = xarch.create(layersContext, "ArchLayerVersions");
			xarch.add(modelXArch, "Object", archLayerVersions);
		}
		if(!xarch.isInstanceOf(archLayerVersions, "edu.uci.isr.xarch.layers.IArchLayerVersions"))
		{
			xarch.promoteTo(layersContext, "ArchLayerVersions", archLayerVersions);
		}
		
		// Get the LayerSetVersionGraph
		ObjRef[] graphs = xarch.getAll(archLayerVersions, "LayerSetVersionGraph");
		if(graphs == null || graphs.length == 0)
		{
			layerSetVersionGraph = xarch.create(layersContext, "LayerSetVersionGraph");
			xarch.add(archLayerVersions, "LayerSetVersionGraph", layerSetVersionGraph);
		}
		else
		{
			// We consider that there is only one LayerSetVersionGraph in the ArchLayerVersions element
			layerSetVersionGraph = graphs[0];
		}
		
		// Set number of versions handed out so far
		recentVersion = getMostRecentVersionNumber(layerSetVersionGraph);
	}
	
	private int getMostRecentVersionNumber(ObjRef versionGraph)
	{
		if(versionGraph == null)
			return 0;
		
		ObjRef lastNode = getLastNodeInGraph(versionGraph);
		if(lastNode == null)
			return 0;
		
		ObjRef versionID = (ObjRef)xarch.get(lastNode, "VersionID");
		String idValue = (String)xarch.get(versionID, "Value");
		
		// Parse out the value
		int value = Integer.parseInt(idValue);
		
		return value;
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ICakeModelWrapper#addLayer(java.lang.String, int, java.lang.String)
	 */
	public void addLayer(String name, int layerType, String versionID)
	{
		// Find version with versionID
		ObjRef layerSet = getLayerSetWithID(versionID);
		if(layerSet != null)
		{
			ObjRef newLayer = null;
			
			// Create a unique Id for the layer ID
			String newId = UIDGenerator.generateUID();			
			
			// Create the name of the layer
			ObjRef layerName = xarch.create(instanceContext, "Description");
			xarch.set(layerName, "Value", name);
			
			// Add appropriate layer
			switch(layerType)
			{
				case COMPONENT_LAYER:
					newLayer = xarch.create(layersContext, "ComponentLayer");
					xarch.set(newLayer, "Id", newId);
					xarch.set(newLayer, "Name", layerName);
					xarch.add(layerSet, "ComponentLayer", newLayer);					
					break;
				case CONNECTOR_LAYER:
					newLayer = xarch.create(layersContext, "ConnectorLayer");
					xarch.set(newLayer, "Id", newId);
					xarch.set(newLayer, "Name", layerName);
					xarch.add(layerSet, "ConnectorLayer", newLayer);					
					break;
				case ANNOTATION_LAYER:
					newLayer = xarch.create(layersContext, "AnnotationLayer");
					xarch.set(newLayer, "Id", newId);
					xarch.set(newLayer, "Name", layerName);
					xarch.add(layerSet, "AnnotationLayer", newLayer);
					break;
				case LINK_LAYER:
					newLayer = xarch.create(layersContext, "LinkLayer");
					xarch.set(newLayer, "Id", newId);
					xarch.set(newLayer, "Name", layerName);
					xarch.add(layerSet, "LinkLayer", newLayer);
					break;
				default:
					// !!! layer type should be of only one of the above types
					// Specify exception or assertion to check this
					break;
			}
		}	
	}
	
	// Finds the LayerSet object that has the provided versionID
	private ObjRef getLayerSetWithID(String versionID)
	{
		return xarch.get(archLayers, "LayerSet", versionID);		
	}
	
	private ObjRef getVersionGraphNode(ObjRef layerSet)
	{
		ObjRef xmlLink = (ObjRef)xarch.get(layerSet, "VersionGraphNode");
		String href = (String)xarch.get(xmlLink, "Href");
		ObjRef versionNode = xarch.resolveHref(modelXArch, href);
		return versionNode;
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ICakeModelWrapper#copyVersion(java.lang.String)
	 */
	public void copyVersion(String versionID)
	{		
		// Duplicate LayerSet with given versionID
		ObjRef layerSetToCopy = xarch.get(archLayers, "LayerSet", versionID);
		ObjRef dupedLayerSet = xarch.cloneElement(layerSetToCopy, IXArchElement.DEPTH_INFINITY);
		
		// Set new version value
		String versionValue = generateNextVersionValue();
		xarch.set(dupedLayerSet, "Id", versionValue);		
			
		// Create new Node
		ObjRef node = xarch.create(versionsContext, "Node");	
		ObjRef newvID = xarch.create(versionsContext, "VersionID");		
		xarch.set(newvID, "Value", versionValue);
		xarch.set(node, "VersionID", newvID);
		xarch.set(node, "Id", UIDGenerator.generateUID());
		
		// This node is not the first to be added, and there is a potential parent node
		ObjRef lastNode = getLastNodeInGraph(layerSetVersionGraph);
		String parentID = (String)xarch.get(lastNode, "Id");		
		ObjRef xmlLink = createXMLLink(parentID);
		xarch.add(node, "Parent", xmlLink);			
		
		// Add to version graph		
		xarch.add(layerSetVersionGraph, "Node", node);
	
		// Create XMLLink to act as a VersionGraphNode to add to the new LayerSet
		ObjRef versionGraphNode = createXMLLink((String)xarch.get(node, "Id"));
		xarch.set(dupedLayerSet, "VersionGraphNode", versionGraphNode);
		
		// Add to ArchLayers
		xarch.add(archLayers, "LayerSet", dupedLayerSet);
				
		duplicateVersionModelThings(layerSetToCopy, dupedLayerSet);		
	}
	
	private void duplicateVersionModelThings(ObjRef layerSet, ObjRef duplicatedLayerSet)
	{
		ObjRef[] origLayers = null;
		ObjRef[] dupedLayers = null;
				
		origLayers = xarch.getAll(layerSet, "ComponentLayer");
		dupedLayers = xarch.getAll(duplicatedLayerSet, "ComponentLayer");
		duplicateComponentLayerThings(origLayers, dupedLayers);
		
		origLayers = xarch.getAll(layerSet, "ConnectorLayer");
		dupedLayers = xarch.getAll(duplicatedLayerSet, "ConnectorLayer");
		duplicateConnectorLayerThings(origLayers, dupedLayers);
		
		origLayers = xarch.getAll(layerSet, "LinkLayer");
		dupedLayers = xarch.getAll(duplicatedLayerSet, "LinkLayer");
		duplicateLinkLayerThings(origLayers, dupedLayers);
		
		origLayers = xarch.getAll(layerSet, "AnnotationLayer");
		dupedLayers = xarch.getAll(duplicatedLayerSet, "AnnotationLayer");	
		duplicateAnnotationLayerThings(origLayers, dupedLayers);		
	}
	
	/**
	 * @param origLayers
	 * @param dupedLayers
	 */
	private void duplicateLinkLayerThings(ObjRef[] origLayers, ObjRef[] dupedLayers)
	{
		assert origLayers.length == dupedLayers.length : "There should be the same number of duplicated layers as original layers";
		
		for(int i = 0; i < origLayers.length; ++i)
		{
			duplicateLinkThings(origLayers[i], dupedLayers[i]);
		}
	}

	
	/**
	 * @param origLayer
	 * @param dupedLayer
	 */
	private void duplicateLinkThings(ObjRef origLayer, ObjRef dupedLayer)
	{
		ObjRef[] origContainingElements = xarch.getAll(origLayer, "ContainingElement");
		ObjRef[] dupedContainingElements = xarch.getAll(dupedLayer, "ContainingElement");
		
		assert origContainingElements.length == dupedContainingElements.length : 
			"Layers should have same number of containingElements";

		for(int i = 0; i < origContainingElements.length; ++i)
		{
			duplicateLinkThing(origContainingElements[i], dupedContainingElements[i]);
		}
	}

	
	/**
	 * @param origContainingElement
	 * @param dupedContainingElement
	 */
	private void duplicateLinkThing(ObjRef origContainingElement, ObjRef dupedContainingElement)
	{
		String thingID = thingIDMap.getThingID(origContainingElement);		
		CakeLinkThing origThing = (CakeLinkThing)bnaModel.getThing(thingID);				
		String origVersionID = CakeUtils.getContainingElementVersionID(xarch, origContainingElement);
		origThing.setSelected(false);
		
		// Copy over the href from the original containing element.
		String origHref = (String)xarch.get(origContainingElement, "Href"); 
		xarch.set(dupedContainingElement, "Href", origHref);
				
		// Get new thing's layer and version info		
		String versionID = CakeUtils.getContainingElementVersionID(xarch, dupedContainingElement);
		String layerName = CakeUtils.getContainingElementLayerName(xarch, dupedContainingElement);
		
		// Create link and set necessary properties
		CakeLinkThing duplicatedThing = (CakeLinkThing)origThing.duplicate();
		duplicatedThing.setID(UIDGenerator.generateUID());
		duplicatedThing.setVersion(versionID);
		duplicatedThing.setLayer(layerName);				 
		duplicatedThing.setColor(versionColorMap.getColor(versionID));		
		duplicatedThing.setEditable(true);
		
		// Add to model
		bnaModel.addThing(duplicatedThing);
		
		// Hoop up to thingIDMap
		thingIDMap.mapRefToID(dupedContainingElement, duplicatedThing.getID());
		
		// Check for end points on the original link and create new end points on the duped link.
		ObjRef origLinkRef = CakeUtils.resolveXMLLink(xarch, modelXArch, origContainingElement);
		ObjRef[] origPoints = xarch.getAll(origLinkRef, "Point");
		if(origPoints != null)
		{
			String firstEndPointID = origThing.getFirstEndpointStuckToID();
			String secondEndPointID = origThing.getSecondEndpointStuckToID();
			ObjRef firstContainingElement = interfaceMap.getElementWithInterfaceThingID(firstEndPointID);
			ObjRef secondContainingElement = interfaceMap.getElementWithInterfaceThingID(secondEndPointID);
			CakeInterfaceThing firstInterfaceThing = (CakeInterfaceThing)bnaModel.getThing(firstEndPointID);
			CakeInterfaceThing secondInterfaceThing = (CakeInterfaceThing)bnaModel.getThing(secondEndPointID);
			
			// If the original link containing element and the connected containing elements are in the same
			// version then we can bother duplicating the end points. 
			if(firstContainingElement != null && secondContainingElement != null && 
				origVersionID.equals(CakeUtils.getContainingElementVersionID(xarch, firstContainingElement)) &&
				origVersionID.equals(CakeUtils.getContainingElementVersionID(xarch, secondContainingElement)))
			{
				ObjRef firstElementRef = CakeUtils.resolveXMLLink(xarch, modelXArch, firstContainingElement);
				ObjRef secondElementRef = CakeUtils.resolveXMLLink(xarch, modelXArch, secondContainingElement);
				
				ObjRef firstDupedContainingElement = getDupedContainingElementInVersion(firstElementRef, versionID);
				ObjRef secondDupedContainingElement = getDupedContainingElementInVersion(secondElementRef, versionID); 

				CakeInterfaceThing firstDupedEndPoint = null;
				CakeInterfaceThing secondDupedEndPoint = null;
				
				Rectangle r = firstInterfaceThing.getBoundingBox();
				Collection interfaceThingIDs = interfaceMap.getAllInterfaceThingIDs(firstDupedContainingElement);
				for(Iterator thingIDIterator = interfaceThingIDs.iterator(); thingIDIterator.hasNext(); )
				{
					String id = (String)thingIDIterator.next();
					CakeInterfaceThing testInterface = (CakeInterfaceThing)bnaModel.getThing(id);
					if(testInterface.getBoundingBox().equals(r))
					{
						firstDupedEndPoint = testInterface;
					}
				}
				
				r = secondInterfaceThing.getBoundingBox();
				interfaceThingIDs = interfaceMap.getAllInterfaceThingIDs(secondDupedContainingElement);
				for(Iterator thingIDIterator = interfaceThingIDs.iterator(); thingIDIterator.hasNext(); )
				{
					String id = (String)thingIDIterator.next();
					CakeInterfaceThing testInterface = (CakeInterfaceThing)bnaModel.getThing(id);
					if(testInterface.getBoundingBox().equals(r))
					{
						secondDupedEndPoint = testInterface;
					}
				}
				
				duplicatedThing.setFirstEndpointStuckToID(firstDupedEndPoint.getID());
				duplicatedThing.setSecondEndpointStuckToID(secondDupedEndPoint.getID());				
			}
			else
			{
				// If the link is in a different version to either (or both) of the endpoints
				duplicatedThing.setFirstEndpointStuckToID(origThing.getFirstEndpointStuckToID());
				duplicatedThing.setSecondEndpointStuckToID(origThing.getSecondEndpointStuckToID());	
			}
			
		}
	}

	/**
	 * @param firstContainingElement
	 * @param versionID
	 * @return
	 */
	private ObjRef getDupedContainingElementInVersion(ObjRef element, String versionID)
	{
		ObjRef[] references = xarch.getReferences(modelXArch, (String)xarch.get(element, "Id"));
		for(int i = 0; i < references.length; ++i)
		{
			if(xarch.isInstanceOf(references[i], "edu.uci.isr.xarch.layers.ILayeredXMLLink"))
			{
				String thingID = thingIDMap.getThingID(references[i]);
				if(thingID == null)
					return null;
					
				ILayerableThing thing = (ILayerableThing)bnaModel.getThing(thingID);
				if(thing.getVersion().equals(versionID))
				{
					return references[i];
				}
			}
		}
		return null;
	}

	/**
	 * @param origLayers
	 * @param dupedLayers
	 */
	private void duplicateConnectorLayerThings(ObjRef[] origLayers, ObjRef[] dupedLayers)
	{
		assert origLayers.length == dupedLayers.length : "There should be the same number of duplicated layers as original layers";
	
		for(int i = 0; i < origLayers.length; ++i)
		{
			duplicateConnectorThings(origLayers[i], dupedLayers[i]);
		}	
	}

	/**
	 * @param origLayer
	 * @param dupedLayer
	 */
	private void duplicateConnectorThings(ObjRef origLayer, ObjRef dupedLayer)
	{
		ObjRef[] origContainingElements = xarch.getAll(origLayer, "ContainingElement");
		ObjRef[] dupedContainingElements = xarch.getAll(dupedLayer, "ContainingElement");

		assert origContainingElements.length == dupedContainingElements.length : 
			"Layers should have same number of containingElements";

		for(int i = 0; i < origContainingElements.length; ++i)
		{
			duplicateConnectorThing(origContainingElements[i], dupedContainingElements[i]);
		}
	}

	
	/**
	 * @param origContainingElement
	 * @param dupedContainingElement
	 */
	private void duplicateConnectorThing(ObjRef origContainingElement, ObjRef dupedContainingElement)
	{		
		String thingID = thingIDMap.getThingID(origContainingElement);		
		CakeConnectorThing origThing = (CakeConnectorThing)bnaModel.getThing(thingID);		
		origThing.setSelected(false);

		// Copy over the href from the original containing element.
		String origHref = (String)xarch.get(origContainingElement, "Href"); 
		xarch.set(dupedContainingElement, "Href", origHref);
		
		// Get new thing's layer and version info		
		String versionID = CakeUtils.getContainingElementVersionID(xarch, dupedContainingElement);
		String layerName = CakeUtils.getContainingElementLayerName(xarch, dupedContainingElement);

		// Create connector and set necessary properties
		CakeConnectorThing duplicatedThing = (CakeConnectorThing)origThing.duplicate();
		duplicatedThing.setID(UIDGenerator.generateUID());
		duplicatedThing.setVersion(versionID);
		duplicatedThing.setLayer(layerName);				 
		duplicatedThing.setColor(versionColorMap.getColor(versionID));	
		duplicatedThing.setEditable(true);	
	
		// Add to model
		bnaModel.addThing(duplicatedThing);

		// Create interfaces on connector thing
		duplicateInterfaceThings(
			origContainingElement,
			dupedContainingElement,
			versionID,
			layerName,
			duplicatedThing);
			
		// Hook up to thingIDMap
		thingIDMap.mapRefToID(dupedContainingElement, duplicatedThing.getID());
	}

	/**
	 * @param origLayers
	 * @param dupedLayers
	 */
	private void duplicateComponentLayerThings(ObjRef[] origLayers, ObjRef[] dupedLayers)
	{
		assert origLayers.length == dupedLayers.length : "There should be the same number of duplicated layers as original layers";
		
		for(int i = 0; i < origLayers.length; ++i)
		{
			duplicateComponentThings(origLayers[i], dupedLayers[i]);
		}
	}

	/**
	 * @param origLayer
	 * @param dupedLayer
	 */
	private void duplicateComponentThings(ObjRef origLayer, ObjRef dupedLayer)
	{
		ObjRef[] origContainingElements = xarch.getAll(origLayer, "ContainingElement");
		ObjRef[] dupedContainingElements = xarch.getAll(dupedLayer, "ContainingElement");
		
		assert origContainingElements.length == dupedContainingElements.length : 
			"Layers should have same number of containingElements";
		
		for(int i = 0; i < origContainingElements.length; ++i)
		{
			duplicateComponentThing(origContainingElements[i], dupedContainingElements[i]);
		}
	}
	
	private void duplicateComponentThing(ObjRef origContainingElement, ObjRef dupedContainingElement)
	{		
		String thingID = thingIDMap.getThingID(origContainingElement);		
		CakeComponentThing origThing = (CakeComponentThing)bnaModel.getThing(thingID);				
		origThing.setSelected(false);
		
		// Copy over the href from the original containing element.
		String origHref = (String)xarch.get(origContainingElement, "Href"); 
		xarch.set(dupedContainingElement, "Href", origHref);
				
		// Get new thing's layer and version info		
		String versionID = CakeUtils.getContainingElementVersionID(xarch, dupedContainingElement);
		String layerName = CakeUtils.getContainingElementLayerName(xarch, dupedContainingElement);
		
		// Create component and set necessary properties
		CakeComponentThing duplicatedThing = (CakeComponentThing)origThing.duplicate();
		duplicatedThing.setID(UIDGenerator.generateUID());
		duplicatedThing.setVersion(versionID);
		duplicatedThing.setLayer(layerName);				 
		duplicatedThing.setColor(versionColorMap.getColor(versionID));		
		duplicatedThing.setEditable(true);
			
		// Add to model
		bnaModel.addThing(duplicatedThing);
		
		// Create interfaces on component thing
		duplicateInterfaceThings(
			origContainingElement,
			dupedContainingElement,
			versionID,
			layerName,
			duplicatedThing);
					
		// Hook up to thingIDMap
		thingIDMap.mapRefToID(dupedContainingElement, duplicatedThing.getID());
	}

	private void duplicateInterfaceThings(
		ObjRef origContainingElement,
		ObjRef dupedContainingElement,
		String versionID,
		String layerName,
		ILayerableThing duplicatedThing)
	{
		Collection interfacePairs = interfaceMap.getInterfacePairs(origContainingElement);
		if(interfacePairs != null)
		{
			for(Iterator intIterator = interfacePairs.iterator(); intIterator.hasNext(); )
			{
				InterfaceMap.InterfacePair interfacePair = (InterfaceMap.InterfacePair)intIterator.next();
				CakeInterfaceThing interfaceThing = (CakeInterfaceThing)bnaModel.getThing(interfacePair.getInterfaceThingID());
				CakeInterfaceThing dupedInterfaceThing = (CakeInterfaceThing)interfaceThing.duplicate();
				
				dupedInterfaceThing.setLayer(layerName);
				dupedInterfaceThing.setVersion(versionID);
				dupedInterfaceThing.setTargetThingID(duplicatedThing.getID());
				dupedInterfaceThing.setID(UIDGenerator.generateUID());
				dupedInterfaceThing.setEditable(true);											
				bnaModel.addThing(dupedInterfaceThing);
				
				dupedInterfaceThing.setBoundingBox(interfaceThing.getBoundingBox());
				
				interfaceMap.mapRefToInterfacePair(dupedContainingElement, interfacePair.getInterfaceRef(), 
					dupedInterfaceThing.getID());				
			}
		}		
	}

	/**
	 * @param origLayers
	 * @param dupedLayers
	 */
	private void duplicateAnnotationLayerThings(ObjRef[] origLayers, ObjRef[] dupedLayers)
	{
		assert origLayers.length == dupedLayers.length : "There should be the same number of duplicated layers as original layers";
		
		for(int i = 0; i < origLayers.length; ++i)
		{
			duplicateAnnotationThings(origLayers[i], dupedLayers[i]);
		}
	}

	
	/**
	 * @param origLayer
	 * @param dupedLayer
	 */
	private void duplicateAnnotationThings(ObjRef origLayer, ObjRef dupedLayer)
	{
		ObjRef[] origContainingElements = xarch.getAll(origLayer, "ContainingElement");
		ObjRef[] dupedContainingElements = xarch.getAll(dupedLayer, "ContainingElement");
		
		assert origContainingElements.length == dupedContainingElements.length : 
			"Layers should have same number of containingElements";
		
		for(int i = 0; i < origContainingElements.length; ++i)
		{
			duplicateAnnotationThing(origContainingElements[i], dupedContainingElements[i]);
		}
	}

	/**
	 * @param origContainingElement
	 * @param dupedContainingElement
	 */
	private void duplicateAnnotationThing(ObjRef origContainingElement, ObjRef dupedContainingElement)
	{
		String thingID = thingIDMap.getThingID(origContainingElement);		
		CakeAnnotationThing origThing = (CakeAnnotationThing)bnaModel.getThing(thingID);				
		origThing.setSelected(false);
		
		// Copy over the href from the original containing element.
		String origHref = (String)xarch.get(origContainingElement, "Href"); 
		xarch.set(dupedContainingElement, "Href", origHref);
				
		// Get new thing's layer and version info		
		String versionID = CakeUtils.getContainingElementVersionID(xarch, dupedContainingElement);
		String layerName = CakeUtils.getContainingElementLayerName(xarch, dupedContainingElement);
		
		// Create annotation and set necessary properties
		CakeAnnotationThing duplicatedThing = (CakeAnnotationThing)origThing.duplicate();
		duplicatedThing.setID(UIDGenerator.generateUID());
		duplicatedThing.setVersion(versionID);
		duplicatedThing.setLayer(layerName);				 
		duplicatedThing.setColor(versionColorMap.getColor(versionID));		
		duplicatedThing.setEditable(true);
			
		// Add to model
		bnaModel.addThing(duplicatedThing);
		
        // Hook up to thingIDMap
		thingIDMap.mapRefToID(dupedContainingElement, duplicatedThing.getID());
	}

	private ObjRef createXMLLink(String targetID)
	{		
		ObjRef xmlLink = xarch.create(instanceContext, "XMLLink");		
		xarch.set(xmlLink, "Href", "#" + targetID);
		xarch.set(xmlLink, "Type", "simple");
		return xmlLink;
	}
	
	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ICakeModelWrapper#addVersion()
	 */
	public void addVersion()
	{	
		// Create a new LayerSet
		ObjRef newLayerSet = xarch.create(layersContext, "LayerSet");		
		String versionValue = generateNextVersionValue();
		xarch.set(newLayerSet, "Id", versionValue);
				
		// Create a new version node
		ObjRef node = xarch.create(versionsContext, "Node");
		ObjRef newvID = xarch.create(versionsContext, "VersionID");		
		xarch.set(newvID, "Value", versionValue);
		xarch.set(node, "VersionID", newvID);
		xarch.set(node, "Id", UIDGenerator.generateUID());
		
		// Add node to LayerSetVersionGraph	(we assume there is only one LayerSetVersionGraph)							
		ObjRef lastNode = getLastNodeInGraph(layerSetVersionGraph);
		if(lastNode != null)
		{
			// In this case the node is not the first to be added, and there is a potential parent node
			String parentID = (String)xarch.get(lastNode, "Id");		
			ObjRef xmlLink = createXMLLink(parentID);
			xarch.add(node, "Parent", xmlLink);			
		}				
		xarch.add(layerSetVersionGraph, "Node", node);
		
        // Create XMLLink to act as a VersionGraphNode to add to the new LayerSet
	    ObjRef versionGraphNode = createXMLLink((String)xarch.get(node, "Id"));
	    xarch.set(newLayerSet, "VersionGraphNode", versionGraphNode);
	    
	    // Add to ArchLayers
		xarch.add(archLayers, "LayerSet", newLayerSet);		
	}

	/**
	 * @param layerSetVersionGraph
	 * @return
	 */
	private ObjRef getLastNodeInGraph(ObjRef layerSetVersionGraph)
	{
		ObjRef[] nodes = xarch.getAll(layerSetVersionGraph, "Node");
		boolean isLastNode = false;
		for(int i = 0; i < nodes.length; ++i)
		{		
			if(isLastNodeInGraph(nodes[i]))
			{
				return nodes[i];
			}			
		}
		
		return null;
	}
	
	private boolean isLastNodeInGraph(ObjRef node)
	{
		String id = (String)xarch.get(node, "Id");
		ObjRef[] references = xarch.getReferences(modelXArch, id);
		
		// If any of the references to this Node is another Node then this is not the last one
		// Node in the graph (since the graph is linear with references to parent nodes)
		for(int i = 0; i < references.length; ++i)
		{
			// References are simply XMLLinks and the XMLLinks' parents are the nodes
			ObjRef parent = xarch.getParent(references[i]);			
			if(xarch.isInstanceOf(parent, "edu.uci.isr.xarch.versions.INode"))
			{
				return false;
			}
		}
	
		return true;
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ICakeModelWrapper#removeLayer(java.lang.String)
	 */
	public void removeLayer(String layerID)
	{		
		//System.out.println("LayerID is: " + layerID);
		
		ObjRef layerToRemove = xarch.getByID(modelXArch, layerID);
		if(layerToRemove == null)
		{
			System.out.println("Layer ID provided is not valid.  Do something dude.. excpetion or something");
		}
		
		ObjRef parent = xarch.getParent(layerToRemove);
		if(xarch.isInstanceOf(layerToRemove, "edu.uci.isr.xarch.layers.IComponentLayer"))
		{
			xarch.remove(parent, "ComponentLayer", layerToRemove);
		}
		else if(xarch.isInstanceOf(layerToRemove, "edu.uci.isr.xarch.layers.IConnectorLayer"))
		{
			xarch.remove(parent, "ConnectorLayer", layerToRemove);
		}
		else if(xarch.isInstanceOf(layerToRemove, "edu.uci.isr.xarch.layers.ILinkLayer"))
		{
			xarch.remove(parent, "LinkLayer", layerToRemove);
		}
		else if(xarch.isInstanceOf(layerToRemove, "edu.uci.isr.xarch.layers.IAnnotationLayer"))
		{
			xarch.remove(parent, "AnnotationLayer", layerToRemove);
		}		
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ICakeModelWrapper#removeVersion(java.lang.String)
	 */
	public void removeVersion(String versionID)
	{
		ObjRef layerSetToRemove = getLayerSetWithID(versionID);
		
		if(layerSetToRemove == null)
		{
			System.out.println("The VersionID is invalid");
		}
		
		// Get node associated with this LayerSet		
		ObjRef nodeToRemove = getVersionGraphNode(layerSetToRemove);
		
		// If this node has a child
		ObjRef child = getNodeChild(nodeToRemove);
		if(child != null)
		{
			ObjRef[] parents = xarch.getAll(nodeToRemove, "Parent");
			
			// This is the root
			if(parents == null || parents.length == 0)
			{				
				xarch.clear(child, "Parent");
			}
			
			// This has a child and is not the root
			else
			{							
				xarch.clear(child, "Parent");
				ObjRef parent = xarch.cloneElement(parents[0], IXArchElement.DEPTH_INFINITY); 
				xarch.add(child, "Parent", parent);
				
  				// Clear the href on the node's parent so at to force an update
			    // of the references cash in xArch ADT.
				xarch.clear(parents[0], "Href");
			}								
		}
		
		// Remove node from its version graph
		ObjRef nodeParent = xarch.getParent(nodeToRemove);
		xarch.remove(nodeParent, "Node", nodeToRemove);
		
		// Now remove the LayerSet itself
		xarch.remove(archLayers, "LayerSet", layerSetToRemove); 			
	}	
	
	private ObjRef getNodeChild(ObjRef node)
	{
		String id = (String)xarch.get(node, "Id");
		ObjRef[] refs = xarch.getReferences(modelXArch, id);
		for(int i = 0; i < refs.length; ++i)
		{
			ObjRef parent = xarch.getParent(refs[i]);	// the ref is an XMLLink and the parent is what we're interested in
			if(xarch.isInstanceOf(parent, "edu.uci.isr.xarch.versions.INode"))
			{
				return parent;
			}
		}
		
		return null;
	}
	
	public void setLayerName(String layerID, String name)
	{
		if(name == null)
		{
			name = new String("");
		}
		if(layerID == null || layerID == "")
		{
			System.out.println("CakeModelWrapper.setLayerName(...) has versionID set to: " + layerID); 
		}
		
		ObjRef layer = xarch.getByID(modelXArch, layerID);
		ObjRef layerName = (ObjRef)xarch.get(layer, "Name");
		String nameStr = "";
		
		if(layerName != null)
		{
			nameStr = (String)xarch.get(layerName, "Value");
		}
		else
		{
			layerName = xarch.create(instanceContext, "Description");
			xarch.set(layer, "Name", layerName);
		}
		
		// make sure we don't  set the name unless its necessary
		if(nameStr == null || !nameStr.equals(name))
		{
			xarch.set(layerName, "Value", name);
		}	 
	}
	
	// Generates the next version value to use for version nodes or Ids.
	// This method isolates the task of creating a new version value so that
	// the versioning scheme can be encapsulated and easily changed.
	// Returns a String representation of the version value. 
	private String generateNextVersionValue()
	{
		return String.valueOf(++recentVersion);
	}
}
