package archstudio.comp.cake;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import edu.uci.ics.bna.*;

public class CakeMoveIndicatorPointLogic extends ThingLogicAdapter implements BoundingBoxTrackingListener{

	protected BoundingBoxTrackingLogic boundingBoxTrackingLogic;
	
	//Maps ids of indicated things to the id of the IIndicator pointing at them.
	protected Map indicatedThingMap = new HashMap();
	
	public CakeMoveIndicatorPointLogic(BoundingBoxTrackingLogic boundingBoxTrackingLogic){
		this.boundingBoxTrackingLogic = boundingBoxTrackingLogic;
		boundingBoxTrackingLogic.addBoundingBoxTrackingListener(this);
	}
	
	private void addThingIndicated(IndicatorThing it, String indicatedThingId){
		String[] indicatorThingIdArray = (String[])indicatedThingMap.get(indicatedThingId);
		if(indicatorThingIdArray == null){
			indicatedThingMap.put(indicatedThingId, new String[]{it.getID()});
			return;
		}
		HashSet s = new HashSet(Arrays.asList(indicatorThingIdArray));
		s.add(it.getID());
		indicatorThingIdArray = (String[])s.toArray(indicatorThingIdArray);
		indicatedThingMap.put(indicatedThingId, indicatorThingIdArray);
	}
	
	private void removeThingIndicated(IndicatorThing it, String indicatedThingId){
		String[] indicatorThingIdArray = (String[])indicatedThingMap.get(indicatedThingId);
		if(indicatorThingIdArray == null){
			return;
		}
		HashSet s = new HashSet(Arrays.asList(indicatorThingIdArray));
		s.remove(it.getID());
		if(s.isEmpty()){
			indicatedThingMap.remove(indicatedThingId);
		}
		else{
			indicatorThingIdArray = (String[])s.toArray(indicatorThingIdArray);
			indicatedThingMap.put(indicatedThingId, indicatorThingIdArray);
		}
	}

	final String[] emptyStringArray = new String[0];
		
	private String[] getIndicatorThingIds(String indicatedThingId){
		String[] arr = (String[])indicatedThingMap.get(indicatedThingId);
		return (arr == null) ? emptyStringArray : arr;  
	}
	
	public void bnaModelChanged(BNAModelEvent evt){
		int evtType = evt.getEventType();
		if(evtType == BNAModelEvent.THING_ADDED){
			Thing targetThing = evt.getTargetThing();
			if(targetThing instanceof IndicatorThing){
				IndicatorThing it = (IndicatorThing)targetThing;
				String indicatedThingId = it.getIndicatorThingId();
				if(indicatedThingId != null){
					addThingIndicated(it, indicatedThingId);
				}
				else{
					removeThingIndicated(it, indicatedThingId);
				}
				updateIndicator(it);
			}
		}
		else if(evtType == BNAModelEvent.THING_CHANGED){
			Thing targetThing = evt.getTargetThing();
			if(targetThing instanceof IndicatorThing){
				ThingEvent tevt = evt.getThingEvent();
				String propertyName = tevt.getPropertyName();
				if(propertyName.equals(IndicatorThing.INDICATOR_THING_ID_PROPERTY_NAME)){
					IndicatorThing it = (IndicatorThing)targetThing;
					String indicatedThingId = it.getIndicatorThingId();
					if(indicatedThingId != null){
						String oldIndicatedThingId = (String)evt.getThingEvent().getOldPropertyValue();
						if(oldIndicatedThingId != null){
							removeThingIndicated(it, oldIndicatedThingId);
						}
						addThingIndicated(it, indicatedThingId);
					}
					else{
						String oldIndicatedThingId = (String)evt.getThingEvent().getOldPropertyValue();
						removeThingIndicated(it, oldIndicatedThingId);
					}
					updateIndicator(it);
				}
			}
			if(targetThing instanceof ReshapableSplineThing){
				String[] indicatorThingIds = getIndicatorThingIds(targetThing.getID());
				for(int i = 0; i < indicatorThingIds.length; i++){
					Thing it = evt.getSource().getThing(indicatorThingIds[i]);
					if(it != null){
						ThingEvent tevt = evt.getThingEvent();
						String propertyName = tevt.getPropertyName();
						if(propertyName.startsWith("point")){
							updateIndicator((IndicatorThing)it);
						}
					}
				}
			}
		}
		else if(evtType == BNAModelEvent.THING_REMOVED){
			Thing targetThing = evt.getTargetThing();
			if(targetThing == null) return;
			if(targetThing instanceof IndicatorThing){
				IndicatorThing it = (IndicatorThing)targetThing;
				String indicatedThingId = it.getIndicatorThingId();
				if(indicatedThingId != null){
					removeThingIndicated(it, indicatedThingId);
				}
			}
			if(targetThing instanceof BoxBoundedThing){
				String targetThingId = targetThing.getID();
				String[] indicatorThingIds = getIndicatorThingIds(targetThingId);
				for(int i = 0; i < indicatorThingIds.length; i++){
					BNAComponent c = getBNAComponent();
					if(c != null){
						BNAModel m = c.getModel();
						if(m != null){
							IndicatorThing indicatorThing = (IndicatorThing)m.getThing(indicatorThingIds[i]);
							indicatorThing.setIndicatorThingId(null);
							indicatorThing.setIndicatorPoint(null);
						}
					}
				}
			}
		}
	}	
	
	public void updateIndicator(IndicatorThing t){
		String indicatedThingId = t.getIndicatorThingId();
		if(indicatedThingId != null){
			BNAComponent c = getBNAComponent();
			if(c == null) return;
			BNAModel m = c.getModel();
			if(m == null) return;
			
			Thing indicatedThing = m.getThing(indicatedThingId);
			if(indicatedThing != null){
				if(indicatedThing instanceof ReshapableSplineThing){
					ReshapableSplineThing rst = (ReshapableSplineThing)indicatedThing;
					Point[] points = rst.getAllPoints();
					if(points.length == 0){
						t.setIndicatorPoint(null);
					}
					else if(points.length == 1){
						t.setIndicatorPoint(points[0]);
					}
					else if(points.length == 2){
						Point p1 = points[0];
						Point p2 = points[1];
						
						int dx = p2.x - p1.x;
						if(dx < 0) dx = -dx;
						dx /= 2;
						
						int dy = p2.y - p1.y;
						if(dy < 0) dy = -dy;
						dy /= 2;
						
						int x, y;
						if(p1.x < p2.x){
							x = p1.x + dx;
						}
						else{
							x = p2.x + dx;
						}
						
						if(p1.y < p2.y){
							y = p1.y + dy;
						}
						else{
							y = p2.y + dy;
						}
						
						t.setIndicatorPoint(new Point(x,y));
					}
					else{ //points.length > 2
						t.setIndicatorPoint(points[points.length / 2]);
					}
				}
				else if(indicatedThing instanceof BoxBoundedThing){
					BoxBoundedThing bbt = (BoxBoundedThing)indicatedThing;
					//System.err.println("indicatedThing=" + bbt);
					Rectangle r = bbt.getBoundingBox();
					int mx = r.x + (r.width / 2);
					int my = r.y + (r.height / 2);
					//System.err.println("Setting indicator point to " + r.x + "," + r.y);
					t.setIndicatorPoint(new Point(mx, my));
				}
			}
		}
	}
	
	public void boundingBoxChanged(BoundingBoxChangedEvent evt){
		String targetThingId = evt.getTargetThing().getID();
		if(targetThingId != null){
			String[] indicatorThingIds = getIndicatorThingIds(targetThingId);
			for(int i = 0; i < indicatorThingIds.length; i++){
				BNAComponent c = getBNAComponent();
				if(c != null){
					BNAModel m = c.getModel();
					if(m != null){
						IndicatorThing indicatorThing = (IndicatorThing)m.getThing(indicatorThingIds[i]);
						if(indicatorThing != null){
							updateIndicator(indicatorThing);
						}
					}
				}
			}
		}
	}
}