/*
 * Created on May 29, 2004
 *
 */
package archstudio.comp.cake;

import archstudio.comp.archipelago.ThingIDMap;
import c2.util.UIDGenerator;
import edu.uci.ics.xarchutils.*;



/**
 * A utility class containing useful methods.
 * 
 * @author Chris
 */
public class CakeUtils
{
	/**
	 * Creates and returns an XMLLink <code>ObjRef</code>
	 * @param xarch An <code>XArchFlatInterface</code> object needed to create the XMLLink 
	 * @param instanceContext Context with which the XMLLink can be created
	 * @param targetID A <code>String</code> ID specifying where the created XMLLink should link
	 * @return An <code>ObjRef</code> to a created XMLLink
	 */
	public static ObjRef createXMLLink(XArchFlatInterface xarch, ObjRef instanceContext, String targetID)
	{		
		ObjRef xmlLink = xarch.create(instanceContext, "XMLLink");
		if(targetID != null)		
			xarch.set(xmlLink, "Href", "#" + targetID);
		xarch.set(xmlLink, "Type", "simple");
		return xmlLink;
	}
	
	/**
	 * Creates and returns a LayeredXMLLink <code>ObjRef</code> with a randomly generated id. 
	 * @param xarch An <code>XArchFlatInterface</code> object needed to create the LayeredXMLLink
	 * @param layersContext Context with which the LayeredXMLLink can be created
	 * @param targetID A <code>String</code> ID specifying where the created LayeredXMLLink should link
	 * @return An <code>ObjRef</code> to a created LayeredXMLLink
	 */
	public static ObjRef createLayeredXMLLink(XArchFlatInterface xarch, ObjRef layersContext, String targetID)
	{
		ObjRef xmlLink = xarch.create(layersContext, "LayeredXMLLink");
		if(targetID != null)
			xarch.set(xmlLink, "Href", "#" + targetID);
		xarch.set(xmlLink, "Type", "simple");
		xarch.set(xmlLink, "Id", UIDGenerator.generateUID());		
		return xmlLink;		
	}
	
	/**
	 * Resolves an XMLLink and returns an <code>ObjRef</code> to the element pointed to
	 * @param xarch An <code>XArchFlatInterface</code> object needed to resolve the XMLLink
	 * @param xArchRef <code>ObjRef</code> to the document containing the XMLLink to resolve
	 * @param xmlLink <code>ObjRef</code> to the XMLLink to resolve
	 * @return An <code>ObjRef</code> to the element the XMLLink points to
	 */
	public static ObjRef resolveXMLLink(XArchFlatInterface xarch, ObjRef xArchRef, ObjRef xmlLink)
	{
		ObjRef resolvedObject = null;
		try
		{
			String href = (String)xarch.get(xmlLink, "Href");
			resolvedObject = xarch.resolveHref(xArchRef, href);
		}
		catch(Exception e)
		{
			resolvedObject = null;
		}
		return resolvedObject;
	}
	
	public static String getAnnotationComment(XArchFlatInterface xarch, ObjRef xArchRef,
		ThingIDMap thingIDMap, CakeAnnotationThing t)
	{
		ObjRef containingRef = thingIDMap.getXArchRef(t.getID());			
		ObjRef annotRef = CakeUtils.resolveXMLLink(xarch, xArchRef, containingRef);
		ObjRef commentRef = (ObjRef)xarch.get(annotRef, "Comment");
		return (String)xarch.get(commentRef, "Value");
	}
	
	public static void setAnnotationComment(XArchFlatInterface xarch, ObjRef xArchRef, 
		ThingIDMap thingIDMap, CakeAnnotationThing t, String newValue)
	{
		ObjRef containingRef = thingIDMap.getXArchRef(t.getID());			
		ObjRef annotRef = CakeUtils.resolveXMLLink(xarch, xArchRef, containingRef);
		ObjRef commentRef = (ObjRef)xarch.get(annotRef, "Comment");
		xarch.set(commentRef, "Value", newValue);					
	}
	
	public static String getAnnotationComment(XArchFlatInterface xarch, ObjRef annotRef)
	{
		ObjRef commentRef = (ObjRef)xarch.get(annotRef, "Comment");
		return (String)xarch.get(commentRef, "Value");
	}
	
	public static void setAnnotationComment(XArchFlatInterface xarch, ObjRef annotRef, String newValue)
	{
		ObjRef commentRef = (ObjRef)xarch.get(annotRef, "Comment");
		xarch.set(commentRef, "Value", newValue);
	}
	
	/**
	 * Returns an <code>ObjRef</code> to a containingElement that the provided annotation is anchored to.
	 *  
	 * @param xarch An XArchFlatInterface object.
	 * @param elementRef An <code>ObjRef</code> to an annotation.
	 * @param docRef The document in which the annotation is stored.
	 * @return An <code>ObjRef</code> to the containing Element that the annotation is anchored to
	 */
	public static ObjRef getAnchoredElement(XArchFlatInterface xarch, ObjRef elementRef, ObjRef docRef)
	{
		ObjRef anchorRef = (ObjRef)xarch.get(elementRef, "Anchor");
		if(anchorRef == null)
			return null;
		ObjRef annotatedElement = CakeUtils.resolveXMLLink(xarch, docRef, anchorRef);
		return annotatedElement;
	}
	
	/**
	 * This function finds the corresponding Interface ObjRef for a given
	 * Interface Thing (provides a mapping from the BNA interface to the 
	 * xADL interface)
	 * @param t The interface Thing that we want to find the ObjRef for
	 * @param thingIDMap The map of ThingIDs to xADL ObjRefs
	 * @param intfMap The map of ThingID and interface IDs to ObjRefs
	 * @return The ObjRef of the interface
	 */
	public static ObjRef getInterfaceRef(CakeInterfaceThing t,
		ThingIDMap thingIDMap, InterfaceMap intfMap)
	{
		// gets the ID of the Thing that this interface is stuck on
		String brickID = t.getTargetThingID();
			
		ObjRef endPointRef = intfMap.getInterfaceRef(
			thingIDMap.getXArchRef(brickID),
			t.getID());
			
		return endPointRef;
	}
	
	public static String getContainingElementLayerName(XArchFlatInterface xarch, ObjRef containingElement)
	{
		ObjRef layerRef = xarch.getParent(containingElement);
		ObjRef layerNameRef = (ObjRef) xarch.get(layerRef, "Name");
		String layerName = (String) xarch.get(layerNameRef, "Value");
		return layerName;
	}

	public static String getContainingElementVersionID(XArchFlatInterface xarch, ObjRef containingElement)
	{
		ObjRef layerRef = xarch.getParent(containingElement);
		ObjRef versionRef = xarch.getParent(layerRef);
		String versionID = (String)xarch.get(versionRef, "Id");
		return versionID;
	}
	
	public static LinkInfo getLinkInfo(XArchFlatInterface xarch, ObjRef linkRef, boolean resolve){
		LinkInfo li = new LinkInfo();
		
		li.setLinkRef(linkRef);
		
		ObjRef[] pointRefs = xarch.getAll(linkRef, "Point");
		
		ObjRef xArchRef = null;
		if(resolve){
			xArchRef = xarch.getXArch(linkRef);
		}
		
		if(pointRefs.length > 0){
			li.setPoint1Ref(pointRefs[0]);
			ObjRef anchor1Ref = (ObjRef)xarch.get(pointRefs[0], "AnchorOnInterface");
			if(anchor1Ref != null){
				li.setAnchor1Ref(anchor1Ref);
				String anchor1Type = (String)xarch.get(anchor1Ref, "type");
				li.setAnchor1Type(anchor1Type);
				String anchor1Href = (String)xarch.get(anchor1Ref, "href");
				li.setAnchor1Href(anchor1Href);
				
				if(resolve)
				{
					try
					{
						ObjRef point1Target = xarch.resolveHref(xArchRef, anchor1Href);
						li.setPoint1Target(point1Target);
					}
					catch(Exception e)
					{
						System.out.println(e);
						li.setPoint1Target(null);
					}
					
				}
			}
		}
		if(pointRefs.length > 1){
			li.setPoint2Ref(pointRefs[1]);
			ObjRef anchor2Ref = (ObjRef)xarch.get(pointRefs[1], "AnchorOnInterface");
			if(anchor2Ref != null){
				li.setAnchor2Ref(anchor2Ref);
				String anchor2Type = (String)xarch.get(anchor2Ref, "type");
				li.setAnchor2Type(anchor2Type);
				String anchor2Href = (String)xarch.get(anchor2Ref, "href");
				li.setAnchor2Href(anchor2Href);
				
				if(resolve)
				{
					try
					{
						ObjRef point2Target = xarch.resolveHref(xArchRef, anchor2Href);
						li.setPoint2Target(point2Target);
					}
					catch(Exception e)
					{
						System.out.println(e);
						li.setPoint2Target(null);
					}
					
				}
			}
		}
		return li;
	}

	public static class LinkInfo{
		protected ObjRef linkRef;
		protected ObjRef point1Ref;
		protected ObjRef anchor1Ref;
		protected ObjRef point2Ref;
		protected ObjRef anchor2Ref;
		protected String anchor1Type;
		protected String anchor1Href;
		protected String anchor2Type;
		protected String anchor2Href;
		
		protected ObjRef point1Target;
		protected ObjRef point2Target;

		public String getAnchor1Href() {
			return anchor1Href;
		}

		public ObjRef getAnchor1Ref() {
			return anchor1Ref;
		}

		public String getAnchor1Type() {
			return anchor1Type;
		}

		public String getAnchor2Href() {
			return anchor2Href;
		}

		public ObjRef getAnchor2Ref() {
			return anchor2Ref;
		}

		public String getAnchor2Type() {
			return anchor2Type;
		}

		public ObjRef getLinkRef() {
			return linkRef;
		}

		public ObjRef getPoint1Ref() {
			return point1Ref;
		}

		public ObjRef getPoint1Target() {
			return point1Target;
		}

		public ObjRef getPoint2Ref() {
			return point2Ref;
		}

		public ObjRef getPoint2Target() {
			return point2Target;
		}

		public void setAnchor1Href(String string) {
			anchor1Href = string;
		}

		public void setAnchor1Ref(ObjRef ref) {
			anchor1Ref = ref;
		}

		public void setAnchor1Type(String string) {
			anchor1Type = string;
		}

		public void setAnchor2Href(String string) {
			anchor2Href = string;
		}

		public void setAnchor2Ref(ObjRef ref) {
			anchor2Ref = ref;
		}

		public void setAnchor2Type(String string) {
			anchor2Type = string;
		}

		public void setLinkRef(ObjRef ref) {
			linkRef = ref;
		}

		public void setPoint1Ref(ObjRef ref) {
			point1Ref = ref;
		}

		public void setPoint1Target(ObjRef ref) {
			point1Target = ref;
		}

		public void setPoint2Ref(ObjRef ref) {
			point2Ref = ref;
		}

		public void setPoint2Target(ObjRef ref) {
			point2Target = ref;
		}
	}
}
