/*
 * Created on Jun 7, 2004
 *
 * DeleteEntityLogic.java
 */
package archstudio.comp.cake;



import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Iterator;
import javax.swing.JOptionPane;

import archstudio.comp.archipelago.ThingIDMap;

import edu.uci.ics.bna.*;
import edu.uci.ics.xarchutils.*;

/**
 * @author Ping
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class DeleteEntityLogic extends CakeThingLogicAdapter
{
	final protected String COMPONENT = "Component";
	final protected String CONNECTOR = "Connector";
	final protected String LINK = "Link";
	final protected String ANNOTATION = "Annotation";
	final protected String INTERFACE = "Interface";
	
	protected DotMarquee marqueeSelection = null;
	
	protected int initDownX = -1;
	protected int initDownY = -1;

	protected ModeTrackingLogic modeTrackingLogic = null;
	protected ThingIDMap thingIDMap = null;
	protected InterfaceMap intfMap = null;
	protected XArchFlatInterface xarch = null;
	
	
	protected Vector deletedThings = null;
	// we need a separate vector for annotations because we want to delete them
	// first.  This is because annotations has a reference to Components that 
	// we may want to delete...
	protected Vector deletedAnnotations = null;
	
	public DeleteEntityLogic(XArchFlatInterface xarch, ThingIDMap thingIDMap, 
		InterfaceMap iMap, ModeTrackingLogic mtl)
	{
		this.xarch = xarch;
		this.thingIDMap = thingIDMap;
		this.intfMap = iMap;
		this.modeTrackingLogic = mtl;
	}
	
	public void mouseClicked(Thing t, MouseEvent evt, int worldX, int worldY)
	{
		// process only when we are in delete mode
		if(modeTrackingLogic.getMode() == Palette.REMOVE_MODE)
		{
			if(t == null)
			{
				return;
			}
			if(t instanceof ILayerableThing)
			{
				ILayerableThing layeredThing = (ILayerableThing)t;
				if(!layeredThing.isEditable())
				{
					// if its not editable, don't process it
					return;				
				}
			}
			if(continueDelete())
			{
				delete(t);
			}
		}
	}

	public void mousePressed(Thing t, MouseEvent evt, int worldX, int worldY)
	{
		if(modeTrackingLogic.getMode() == Palette.REMOVE_MODE)
		{
			BNAComponent c = bnaComponent;
			if(evt.getButton() == MouseEvent.BUTTON1)
			{
				if(t == null)
				{
					initDownX = worldX;
					initDownY = worldY;
			
					marqueeSelection = new DotMarquee();
					marqueeSelection.setID("MarqueeSelection");
					Utils.setStackingPriority(marqueeSelection, Utils.STACKING_PRIORITY_ALWAYS_ON_TOP);
					marqueeSelection.setX1(worldX);
					marqueeSelection.setY1(worldY);
					marqueeSelection.setX2(worldX);
					marqueeSelection.setY2(worldY);
					c.getModel().addThing(marqueeSelection);
				}
			}
		}
	}

	public void mouseReleased(Thing t, MouseEvent evt, int worldX, int worldY)
	{
		BNAComponent c = bnaComponent;
		if(evt.getButton() == MouseEvent.BUTTON1)
		{
			if(marqueeSelection != null)
			{
				//System.out.println("Handle selection");
				int wx1 = marqueeSelection.getX1();
				int wx2 = marqueeSelection.getX2();
				int wy1 = marqueeSelection.getY1();
				int wy2 = marqueeSelection.getY2();
			
				if(wx1 > wx2){
					int tmp = wx2;
					wx2 = wx1;
					wx1 = tmp;
				}
				if(wy1 > wy2){
					int tmp = wy2;
					wy2 = wy1;
					wy1 = tmp;
				}
				c.getModel().removeThing(marqueeSelection);
			
				Rectangle selectRect = new Rectangle();
				selectRect.x = wx1;
				selectRect.y = wy1;
				selectRect.width = wx2 - wx1;
				selectRect.height = wy2 - wy1;
			
				if(!Utils.wasControlPressed(evt)){
					SelectionUtils.removeAllSelections(c);
				}
				
				deletedThings = new Vector(20);
				//deletedAnnotations = new Vector(20);
				
				for(Iterator it = c.getModel().getThingIterator(); it.hasNext(); )
				{
					Thing t2 = (Thing)it.next();
					if(t2 instanceof ILayerableThing)
					{
						ILayerableThing layerableThing = (ILayerableThing)t2;
						if(!layerableThing.isEditable())
						{
							// its not editable, just ignore it
							// so continue with the loop (next iteration)
							continue;
						}
					}
					if(t2 instanceof MarqueeSelectableThing)
					{
						Rectangle boundingBox = ((BoxBoundedThing)t2).getBoundingBox();
						if(Utils.isWithin(selectRect, boundingBox))
						{
							((SelectableThing)t2).setSelected(true);
							deletedThings.add(t2);
							/*
							if(t2 instanceof CakeAnnotationThing)
							{
								deletedAnnotations.add(t2);
							}
							else
							{
								deletedThings.add(t2);
							}
							*/					
						}
					}
				}
				if(deletedThings.size() != 0 /*|| deletedAnnotations.size() != 0*/)
				{	
					if(continueDelete())
					{
						//deleteThings(deletedAnnotations.elements());
						deleteThings(deletedThings.elements());
					}
				}
				marqueeSelection = null;
				deletedThings = null;
				//deletedAnnotations = null;
			}
		}
	}

	public void mouseDragged(Thing t, MouseEvent evt, int worldX, int worldY)
	{
		BNAComponent c = bnaComponent;
		if(marqueeSelection != null)
		{
			marqueeSelection.setX2(worldX);
			marqueeSelection.setY2(worldY);
		}
	}
	
	
	// This function prompts the user if they are sure they want to
	// proceed with the delete
	protected boolean continueDelete()
	{
		int choice = JOptionPane.showConfirmDialog(null, "Are you sure you want to delete these element(s)?",
			"Proceed?", JOptionPane.YES_NO_OPTION);
			
		return choice == JOptionPane.YES_OPTION;		
	}
	
	protected void deleteThings(Enumeration enum)
	{
		Thing t;
		while(enum.hasMoreElements())
		{
			t = (Thing)enum.nextElement();
			delete(t);
			/*
			if(t instanceof CakeInterfaceThing)
			{
				System.out.println("Checking interfaces");
				
				// its an interface, lets check to see if the
				// corresponding Component/Connector is also removed
				CakeInterfaceThing intfThing = (CakeInterfaceThing)t;
				if(!deleteTable.containsKey(intfThing.getTargetThingID()))
				{
					System.out.println("Deleting the interface!");
					delete(intfThing);					
				}
			}
			else
			{
				delete(t);
			}
			*/
		}
	}
	/**
	 * This function figures out the type of thing it is and deletes it.
	 * @param t The thing to delete
	 */
	protected void delete(Thing t)
	{
		if(t instanceof ILayerableThing)
		{
			// right now we really can only delete layerable things
			ILayerableThing layerableThing = (ILayerableThing)t;
			if(layerableThing.hasOwnLayerType())
			{
				// it has its own layer type, so we can directly
				// extract from the thingID map
				ObjRef containingElement = thingIDMap.getXArchRef(layerableThing.getID());
				if(containingElement != null)
				{
				
					ObjRef docRef = xarch.getXArch(containingElement);
					// now delete the containingElement
					ObjRef parent = xarch.getParent(containingElement);
					xarch.remove(parent, "ContainingElement", containingElement);
				}
			}
			else
			{
				if(layerableThing instanceof CakeInterfaceThing)
				{
					CakeInterfaceThing intfThing = (CakeInterfaceThing)layerableThing;
					// find the corresponding interface ObjRef
					ObjRef intfRef = CakeUtils.getInterfaceRef(intfThing, thingIDMap,
						intfMap);
					if(intfRef != null)
					{
						// now remove it from the parent
						ObjRef parent = xarch.getParent(intfRef);
						xarch.remove(parent, getElementType(intfRef), intfRef);
					}
				}
			}	
		}		
	}
	
	/**
	 * This function returns the type of the element
	 * @param e The element
	 * @return "Component" for components, etc
	 */
	protected String getElementType(ObjRef e)
	{
		String type = null;
		
		if(xarch.isInstanceOf(e, "edu.uci.isr.xarch.types.IComponent"))
		{
			type = COMPONENT;
		}
		else if(xarch.isInstanceOf(e, "edu.uci.isr.xarch.types.IConnector"))
		{
			type = CONNECTOR;
		}
		else if(xarch.isInstanceOf(e, "edu.uci.isr.xarch.types.ILink"))
		{
			type = LINK;
		}
		else if(xarch.isInstanceOf(e, "edu.uci.isr.xarch.annotations.IAnnotation"))
		{
			type = ANNOTATION;
		}
		else if(xarch.isInstanceOf(e, "edu.uci.isr.xarch.types.IInterface"))
		{
			type = INTERFACE;
		}
		
		//System.out.println(type);
		return type;
	}
}
