package archstudio.comp.cake;

/*
 * This code is based on an example provided by Richard Stanford, 
 * a tutorial reader.
 */

import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.*;
import java.awt.Color;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;

public class DynamicTree extends JPanel {
	protected CheckNode rootNode;
	protected DefaultTreeModel treeModel;
	protected JTree tree;
	TreeLogic treeLogic;
	VersionColorMap colorMap;
	private Toolkit toolkit = Toolkit.getDefaultToolkit();

	public DynamicTree(ICakeModelWrapper model, TreeLogic newTreeLogic, VersionColorMap newColorMap) {
		super(new GridLayout(1,0));
        
        //Initializing our special classes
        treeLogic = newTreeLogic;
        colorMap = newColorMap;
        
		rootNode = new CheckNode("Root Node", treeLogic);
		treeModel = new DefaultTreeModel(rootNode);
		treeModel.addTreeModelListener(new MyTreeModelListener(model));
		tree = new JTree(treeModel);

		tree.setCellRenderer(new CheckRenderer());
		tree.getSelectionModel().setSelectionMode(
		  TreeSelectionModel.SINGLE_TREE_SELECTION
		);
		tree.putClientProperty("JTree.lineStyle", "Angled");
		tree.addMouseListener(new NodeSelectionListener(tree, treeLogic));
    
		//various settings tweaks
		tree.setRootVisible(false);
		tree.setShowsRootHandles(true);
				tree.setToggleClickCount(8); 
		tree.setEditable(true);
		tree.getSelectionModel().setSelectionMode
				(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setShowsRootHandles(true);

		JScrollPane scrollPane = new JScrollPane(tree);
		add(scrollPane);
	}

	/** Remove all nodes except the root node. */
	public void clear() {
		rootNode.removeAllChildren();
		treeModel.reload();
	}

	/** Remove the currently selected node. */
	public void removeCurrentNode() {
		TreePath currentSelection = tree.getSelectionPath();
		if (currentSelection != null) {
			CheckNode currentNode = (CheckNode)
						 (currentSelection.getLastPathComponent());
			MutableTreeNode parent = (MutableTreeNode)(currentNode.getParent());
			if (parent != null) {
				treeModel.removeNodeFromParent(currentNode);
				return;
			}
		} 
	}

	/** Add child to the currently selected node. */
	public CheckNode addObject(Object child) {
		CheckNode parentNode = null;
		TreePath parentPath = tree.getSelectionPath();
		
		//We don't want people to be able to add to root
		if (parentPath == null )
			return null;
		
		//We don't want people to be able to add to a layer
		//Instead a layer will be added to this version
		if( parentPath.getPathCount() >= 3 ){
			parentNode = (CheckNode)(parentPath.getParentPath().getLastPathComponent());
		}
					
		//If we made it this far, we are a version
		else {
			parentNode = (CheckNode)(parentPath.getLastPathComponent());
		}
		
		return addObject(parentNode, child, true);
	}

	public CheckNode addObject(CheckNode parent, Object child) {
		return addObject(parent, child, false);
	}

	public CheckNode addObject(CheckNode parent, Object child, boolean shouldBeVisible) {
		CheckNode childNode = new CheckNode(child, treeLogic);
		if (parent == null) {
			parent = rootNode;
		}
		treeModel.insertNodeInto(childNode, parent, parent.getChildCount());

		if (shouldBeVisible) {
			tree.scrollPathToVisible(new TreePath(childNode.getPath()));
		}
		
		//tree.setSelectionPath(new TreePath(childNode.getPath()));
		return childNode;
	}
	
	
    // * * * * * * * * * * * * * * * * * * * *
    //Functions called by easelModel via checkNodeTree
    // * * * * * * * * * * * * * * * * * * * *
    
	//Adds a layer with the specified name, ID and type.
	//The ID of this layer's version is given.
	public void addLayer(String name, String nodeID, int layerType, String versionID){
		CheckNode parentNode = findNode(versionID);
		if( parentNode != null ){
			CheckNode newNode = new CheckNode((Object)name, nodeID, layerType, false, treeLogic, colorMap.getColor(versionID));
			treeModel.insertNodeInto(newNode, parentNode, parentNode.getChildCount());
		}
	}
	
	//Removes the layer with the specified layerID
	public void removeLayer(String layerID){
		CheckNode node = findNode(layerID);
		treeModel.removeNodeFromParent(node);
	}
	
	//Adds a version with the specified name and ID
	public void addVersion(String name, String nodeID){
		CheckNode newNode = new CheckNode((Object)name, nodeID, 4, true, treeLogic, colorMap.getColor(nodeID));
		treeModel.insertNodeInto(newNode, rootNode, rootNode.getChildCount());
		treeModel.reload();

		//if(rootNode.getChildCount() == 1)
			//tree.updateUI();
	}
	
	//Returns the name of the node with the given nodeID
	public String getName(String nodeID){
		String result;
		result = findNode(nodeID).getName();
		if(result == null)
			result = "";
		return result;
	}
	
	public boolean getIsEditable(String nodeID){
		CheckNode foundNode;
		foundNode = findNode(nodeID);
		//If we can't find the node with that ID, return false
		if(foundNode == null)
			return false;
		
		return foundNode.isSelected2();
	}
	
	//Sets the name of the node specified by the id to the name specified
	public void setName(String name, String nodeID){
		CheckNode result;
		result = findNode(nodeID);
		result.setName(name);
		//this.validate();
		/*
		System.out.println("THOSE LINES!");
		System.out.println(name);
		System.out.println(nodeID);
		System.out.println(result.getName());
		*/
	}
	
	// * * * * * * * * * * * * * 
	// Various Utility Functions
	// * * * * * * * * * * * * *
	
	//Returns the CheckNode with the NodeID input
	public CheckNode findNode(String nodeID){
		//Some temporary variables
		CheckNode version = new CheckNode(treeLogic);
		CheckNode layer = new CheckNode(treeLogic);
		int versionCount, layerCount;

		//For each version, check each layer to see if it is the one		
		versionCount = rootNode.getChildCount();
		for( int i = 0; i < versionCount; i++ )
		{
			
			version = (CheckNode)rootNode.getChildAt(i);
			//If the version itself is the one, return it
			if(version.getNodeID() == nodeID)
				return version;
				
			//Otherwise check its children
			layerCount = version.getChildCount();
			for( int j = 0; j < layerCount; j++ ){
				layer = (CheckNode)version.getChildAt(j);
				if(layer.getNodeID() == nodeID)
					return layer;
			}
		}
		System.out.println("Tried to find node, couldn't, returning null");
		return null;
	}
	
	//Returns the nodeID of the version to which the
	//layer with the specified ID belongs
	public String getVersionID(String nodeID){

		//Some temporary variables
		CheckNode version = new CheckNode(treeLogic);
		CheckNode layer = new CheckNode(treeLogic);
		int versionCount, layerCount;

		//For each version, check each layer to see if it is the one		
		versionCount = rootNode.getChildCount();
		for( int i = 0; i < versionCount; i++ )
		{
			
			version = (CheckNode)rootNode.getChildAt(i);
			//If a version matches the id, return its ID
			if(version.getNodeID() == nodeID){
				return version.getNodeID();
			}
				
			//Otherwise check its children and return the version's id if 
			//a layer with the matching ID is found
			layerCount = version.getChildCount();
			for( int j = 0; j < layerCount; j++ ){
				layer = (CheckNode)version.getChildAt(j);
				if(layer.getNodeID() == nodeID)
					return version.getNodeID();
			}
		}
		System.out.println("Tried to find node's version, couldn't, returning empty string");
		return "";
	
	}
	
	//Returns the nodeID of the selected node
	public String getSelectedLayerID(){
		//Get the path of our current selection
		TreePath selectedPath = tree.getSelectionPath();
		
		//If nothing is selected, return out of here
		if( selectedPath == null )
			return "";
			
		//Otherwise, lets get the selected node
		CheckNode selectedNode = (CheckNode)selectedPath.getLastPathComponent();
		
		//If we have selected the root, this has no ID, return
		if( selectedNode == rootNode )
			return "";
			
		///Baring all the special cases, we can return this node's ID			
		//System.out.println("NODE ID is: " + selectedNode.getNodeID());
		return selectedNode.getNodeID();
	}
	
	//Returns the ID of the version that the selected node is a part of (or is itself)
	public String getSelectedVersionID(){
		//Get the path of our current selection
		TreePath selectedPath = tree.getSelectionPath();
		
		//If nothing is selected, return out of here
		if( selectedPath == null )
			return "";
			
		//Otherwise lets proceed
		CheckNode selectedNode = (CheckNode)selectedPath.getLastPathComponent();
		CheckNode parentNode = (CheckNode)selectedPath.getParentPath().getLastPathComponent();
		
		//If we have selected the root, this has no ID, return
		if( selectedNode == rootNode )
			return "";
		
		//If this node's parent is the root then this is a version itself, return its ID
		if( parentNode == rootNode )
			return selectedNode.getNodeID();
			
		//Otherwise we need to return this node's parent's ID
		return parentNode.getNodeID();				
	}
	
	public String getSelectedName(){
		TreePath selectedPath = tree.getSelectionPath();
		if(selectedPath == null)
			return "";
		CheckNode selectedNode = (CheckNode)selectedPath.getLastPathComponent();
		return selectedNode.getName();
	}
	
	public boolean getSelectedIsVersion(){
		TreePath selectedPath = tree.getSelectionPath();
		if(selectedPath == null)
			return false;
		CheckNode selectedNode = (CheckNode)selectedPath.getLastPathComponent();
		return selectedNode.getIsVersion();
	}
	
	public boolean nullSelected(){
		TreePath selectedPath = tree.getSelectionPath();
		if(selectedPath == null)
			return true;
		return false;
	}
	
	public int getSelectedType(){
		TreePath selectedPath = tree.getSelectionPath();
		if(selectedPath == null)
			return 0;
		CheckNode selectedNode = (CheckNode)selectedPath.getLastPathComponent();
		return selectedNode.getType();
	}
	
	//Returns the color of the layer with the given ID
	public Color getColor(String nodeID){
	  return colorMap.getColor(getVersionID(nodeID));
	}

//	* * * * * * * * * * * * * * * * * * * * * * * * * * *
//	 * * * * * * * NodeSelectionListener * * * * * * * *
//	* * * * * * * * * * * * * * * * * * * * * * * * * * *

	class NodeSelectionListener extends MouseAdapter {
	   JTree tree;
	   int LABEL = 0;
	   int CHECK1 = 1;
	   int CHECK2 = 2;
	   int partClicked = LABEL;
	   TreeLogic treeLogic;
    
	   NodeSelectionListener(JTree tree, TreeLogic newTreeLogic) {
		 this.tree = tree;
		 treeLogic = newTreeLogic;
	   }
	   
	   public void mouseClicked(MouseEvent e) {
		 int x = e.getX();
		 int y = e.getY();
      
      	 //This is the amount the check columns are pushed over in the
		 //checkbox renderer
		 int columnOffset = CheckRenderer.getColumnX();
      
		 //These offsets make sure the clicks coorespond to the checkboxes
		 if(x < columnOffset+40)
		   partClicked = LABEL;
		 else if (x < columnOffset+60)
		   partClicked = CHECK1;
		 else if (x < columnOffset+80)
		   partClicked = CHECK2;
      
      	 //In order to edit the node, we need to click the label
		 if( partClicked == LABEL ){
		 	
		 	//And we need to be clicking a layer, not a version
			TreePath selectedPath = tree.getSelectionPath();
			if(selectedPath != null){
				CheckNode selectedNode = (CheckNode)selectedPath.getLastPathComponent();
		 		//if( !(selectedNode.getIsVersion()) )
				if( selectedNode.isLeaf() )
				{
					tree.setEditable(true);
					String id = selectedNode.getNodeID();
					// because a leaf node is a layer,
					// if we selected it, we want to edit/modify it,
					// so make sure its both editable and visible
					if(!selectedNode.isSelected())
					{
						selectedNode.setSelected(true);
						//If this is a layer, check its version too
						if (selectedNode.isLeaf())
						{
							TreePath parentPath = selectedPath.getParentPath();
							CheckNode parentNode = (CheckNode)parentPath.getLastPathComponent();
							parentNode.setJustMySelected(true);
						}
					}
					if(!selectedNode.isSelected2())
					{
						selectedNode.setSelected2(true);
						//If this is a layer, check its version too
						if (selectedNode.isLeaf())
						{
							TreePath parentPath = selectedPath.getParentPath();
							CheckNode parentNode = (CheckNode)parentPath.getLastPathComponent();
							parentNode.setJustMySelected2(true);
						}
					}
					((DefaultTreeModel)tree.getModel()).nodeChanged(selectedNode);
				}
				else
					tree.setEditable(false);
			}
		 }
		 	
		 else{
			 //If we are not clicking on the label, it should not be editable
			 tree.setEditable(false);
			 
			 int row = tree.getRowForLocation(x, y);
			 TreePath  path = tree.getPathForRow(row);
			 if (path != null) {
			   CheckNode node = (CheckNode)path.getLastPathComponent();
	           String id = node.getNodeID();	        
			   
			   //***If the first checkbox is clicked***
			   if( partClicked == CHECK1 ){
				   boolean isSelected = ! (node.isSelected());
				   node.setSelected(isSelected);
				   //treeLogic.setVisible(id, isSelected);
				   // if its not visible, it shouldn't be editable either
				   if(!isSelected && node.isSelected2()){
				     node.setSelected2(false);
				   }
				   
				   //If this is a layer being made visible, we need to make sure
				   //the version it is on is visible too
				   if (isSelected && node.isLeaf()){
				   	 TreePath parentPath = path.getParentPath();
				   	 CheckNode parentNode = (CheckNode)parentPath.getLastPathComponent();
				   	 parentNode.setJustMySelected(true);
				   }
	
					//If this is a layer being made invisible, we need to see
					//if this is making all of the layers in a version invisible,
					//in which case the version should be unchecked
					//Also, if this makes everyone uneditable, unpdate the version
					if (!isSelected && node.isLeaf()){
					  TreePath parentPath = path.getParentPath();
					  CheckNode parentNode = (CheckNode)parentPath.getLastPathComponent();
					  CheckNode child;
					  boolean allInvisible = true;
					  boolean allUneditable = true;
					  int childCount = parentNode.getChildCount();
					    
					  //Check all of the parent's children
					  //If any are selected, forget it   
					  for(int i = 0; i < childCount; i++){
					  	child = (CheckNode)parentNode.getChildAt(i);
					  	if(child.isSelected()){
					  		allInvisible = false;
					  	}
						if(child.isSelected()){
							allUneditable = false;
						}			  	
					  }
					  
					  //But if none are selected, unselect the version				
					  if(allInvisible){
						parentNode.setJustMySelected(false);	
					  }
					  if(allUneditable){
						parentNode.setJustMySelected2(false);	
					  }
					  
					}			   
			   }
			   
			   //***If the second checkbox is clicked***
			   else{
				   boolean isSelected2 = ! (node.isSelected2());
				   node.setSelected2(isSelected2);
				   //treeLogic.setEditable(id, isSelected2);
				   // if we checked for a layer to be editable,
				   // it should be visible too
				   if(isSelected2 && !node.isSelected())
				   {
				   	 node.setSelected(true);
					 //treeLogic.setVisible(id, true);
				   }
				   
					//If this is a layer being made editable, we need to make sure
					//the version it is on is editable too
					//Also, since we are making this layer visible, we may need
					//to make the version this is on visible
					if (isSelected2 && node.isLeaf()){
					  TreePath parentPath = path.getParentPath();
					  CheckNode parentNode = (CheckNode)parentPath.getLastPathComponent();
					  parentNode.setJustMySelected(true);
					  parentNode.setJustMySelected2(true);
					}
					
					//If this is a layer being made uneditable, we need to see
					//if this is making all of the layers in a version editable,
					//in which case the version should be unchecked
					if (!isSelected2 && node.isLeaf()){
					  TreePath parentPath = path.getParentPath();
					  CheckNode parentNode = (CheckNode)parentPath.getLastPathComponent();
					  CheckNode child;
					  boolean allUnselected = true;
					  int childCount = parentNode.getChildCount();
						    
					  //Check all of the parent's children
					  //If any are selected, forget it   
					  for(int i = 0; i < childCount; i++){
						child = (CheckNode)parentNode.getChildAt(i);
						if(child.isSelected2()){
							allUnselected = false;
						}					  	
					  }
						  
					  //But if none are selected, unselect the version				
					  if(allUnselected){
						parentNode.setJustMySelected2(false);	
					  }
						  
					}
			   }	        
	        
			   ((DefaultTreeModel)tree.getModel()).nodeChanged(node);
			   // I need revalidate if node is root.  but why?
			   if (row == 0) {
				 tree.revalidate();
				 tree.repaint();
			   }
			}
		 }
	   }
	 }
	 
	 

	class MyTreeModelListener implements TreeModelListener {
		ICakeModelWrapper cakeModelWrapper;
		
		public MyTreeModelListener(ICakeModelWrapper newCakeModelWrapper){
			cakeModelWrapper = newCakeModelWrapper;
		}
		
		public void treeNodesChanged(TreeModelEvent e) {
			CheckNode node;
			node = (CheckNode)
					 (e.getTreePath().getLastPathComponent());

			/*
			 * If the event lists children, then the changed
			 * node is the child of the node we've already
			 * gotten.  Otherwise, the changed node and the
			 * specified node are the same.
			 */
			try {
				int index = e.getChildIndices()[0];
				node = (CheckNode)
					   (node.getChildAt(index));
			} catch (NullPointerException exc) {}

			//System.out.println("Node renamed to " + node.getUserObject());
			
			//* * * * * * * * * * * * * * * * * * * *
			//Call model to inform it of rename here
			//* * * * * * * * * * * * * * * * * * * *
			if(node.getType() != CheckNode.VERSION)
			{
				// can only set name for layers, not versions
				cakeModelWrapper.setLayerName(node.getNodeID(), node.getName());
			}
			
			tree.repaint();
			
		}
		public void treeNodesInserted(TreeModelEvent e) {
			//Make it so that newly added children are selected
			Object child = e.getChildren()[0];
			final TreePath childPath = e.getTreePath().pathByAddingChild( child );
			tree.getSelectionModel().setSelectionPath(childPath);
			
			//Tell the world about your new bundle of joy
			//System.out.println("added!");
		}
		public void treeNodesRemoved(TreeModelEvent e) {
		}
		public void treeStructureChanged(TreeModelEvent e) {
		}
	}
}

