/*
 * ICakeTreeWrapper.java
 * Created on May 18, 2004
 *
 */
package archstudio.comp.cake;

/**
 * This interface defines the functions provided in order to
 * hide the actual xADL model from the JTree view of the 
 * version and layers.  It provides functionos to modify the layers
 * and versions in the xADL model.
 * 
 * @author Chris Van der Westhuizen and Ping Chen
 *
 */
public interface ICakeModelWrapper
{
	// These constants define the possible types of layers that we
	// are dealing with
	
	/**
	 * This is the constant representing a component layer
	 */
	static final int COMPONENT_LAYER = 0;
	/**
	 * This is the constant representing a connector layer
	 */
	static final int CONNECTOR_LAYER = 1;
	/**
	 * This is the constant representing a links layer
	 */
	static final int LINK_LAYER = 2;
	/**
	 * This is the constant representing a annotation layer
	 */
	static final int ANNOTATION_LAYER = 3;
	
	/**
	 * This function adds a particular layer to the underlying
	 * xADL model.
	 * 
	 * @param name specifies the name of the layer we are adding
	 * @param layerType specifies the type of the layer (should
	 * 			be one of the constants specified in this interface)
	 * @param versionID specifies version number of the layer set that 
	 * 			this layer should be added to.
	 */
	void addLayer(String name, int layerType, String versionID);

	/**
	 * This function creates a new version (layer set) based
	 * on an existing version.  The new version will have all 
	 * the layers (and its contents) of the existing version.
	 * 
	 * @param versionID specifies the layer that the new version
	 * 			should be based on.
	 */
	void copyVersion(String versionID);
	
	/**
	 * This function creates a brand new version.  Since
	 * it is not based on an existing version, it will be 
	 * be empty.
	 *
	 */
	void addVersion();

	/**
	 * This function removes the specified layer from the xADL model.
	 * @param layerID specifies the ID of the layer that should be removed.
	 */
	void removeLayer(String layerID);
	
	/**
	 * This function removes the specifies version (layer set) from the
	 * xADL model.
	 * @param versionID specifies the ID of the version that we want to remove
	 * 			from the model.
	 */
	void removeVersion(String versionID);
	
	/**
	 * 
	 * @param versionID
	 * @param name
	 */
	void setLayerName(String versionID, String name);
}
