/*
 * Created on May 29, 2004
 *
 */
package archstudio.comp.cake;

import java.util.Collections;
import java.util.HashMap;

import edu.uci.ics.bna.Box;

/**
 * LayerableBox is a Thing that can be layered; i.e., it has the ability to change its visibility and
 * change its editability.
 * 
 * @author Chris
 * 
 */
public class LayerableBox extends Box implements ILayerableThing
{
	/**
	 * 
	 */
	public LayerableBox()
	{
		super();
		setVisible(true);
		setEditable(true);
	}
	
	protected LayerableBox(ILayerableThing copyMe)
	{
		this.id = copyMe.getID();
		this.props = Collections.synchronizedMap(new HashMap(copyMe.getPropertyMap()));
		this.propertyLockObject = props;
	}
	
	public Object clone() throws CloneNotSupportedException
	{
		ILayerableThing m = new LayerableBox(this);
		return m;
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ILayerableThing#setVisible(boolean)
	 */
	public void setVisible(boolean visible)
	{
		setProperty(VISIBLE_PROPERTY_NAME, visible);
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ILayerableThing#isVisible()
	 */
	public boolean isVisible()
	{
		return getBooleanProperty(VISIBLE_PROPERTY_NAME);
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ILayerableThing#setEditable(boolean)
	 */
	public void setEditable(boolean editable)
	{
		setProperty(EDITABLE_PROPERTY_NAME, editable);
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ILayerableThing#isEditable()
	 */
	public boolean isEditable()
	{
		return getBooleanProperty(EDITABLE_PROPERTY_NAME);
	}
	
	public void setLayer(String layerName)
	{
		setProperty(LAYER_PROPERTY_NAME, layerName);
	}
	public String getLayer()
	{
		return (String)getProperty(LAYER_PROPERTY_NAME);
	}
	
	public void setVersion(String versionName)
	{
		setProperty(VERSION_PROPERTY_NAME, versionName);
	}
	public String getVersion()
	{
		return (String)getProperty(VERSION_PROPERTY_NAME);
	}
	
	public String getPeerClassname()
	{
		return "archstudio.comp.cake.LayerableBoxPeer";
	}
	
	public boolean hasOwnLayerType()
	{
		return true;
	}
}
