/*
 * Created on May 31, 2004
 *
 * LayerableEndpoint.java
 */
package archstudio.comp.cake;

import java.util.Collections;
import java.util.HashMap;

import edu.uci.ics.bna.Endpoint;

/**
 * @author Ping
 *
 * This class enhances the Endpoint class defined in BNA to be 
 * layerable (editable and visible)
 */
public class LayerableEndpoint extends Endpoint implements ILayerableThing
{

	public LayerableEndpoint()
	{
		super();
		setVisible(true);
		setEditable(true);
	}
	
	protected LayerableEndpoint(ILayerableThing copyMe)
	{
		this.id = copyMe.getID();
		this.props = Collections.synchronizedMap(new HashMap(copyMe.getPropertyMap()));
		this.propertyLockObject = props;
	}

	public Object clone() throws CloneNotSupportedException
	{
		ILayerableThing m = new LayerableEndpoint(this);
		return m;
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ILayerableThing#setVisible(boolean)
	 */
	public void setVisible(boolean visible)
	{
		setProperty(VISIBLE_PROPERTY_NAME, visible);
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ILayerableThing#isVisible()
	 */
	public boolean isVisible()
	{
		return getBooleanProperty(VISIBLE_PROPERTY_NAME);
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ILayerableThing#setEditable(boolean)
	 */
	public void setEditable(boolean editable)
	{
		setProperty(EDITABLE_PROPERTY_NAME, editable);
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.ILayerableThing#isEditable()
	 */
	public boolean isEditable()
	{
		return getBooleanProperty(EDITABLE_PROPERTY_NAME);
	}
	
	public void setLayer(String layerName)
	{
		setProperty(LAYER_PROPERTY_NAME, layerName);
	}
	public String getLayer()
	{
		return (String)getProperty(LAYER_PROPERTY_NAME);
	}
	
	public void setVersion(String versionName)
	{
		setProperty(VERSION_PROPERTY_NAME, versionName);
	}
	public String getVersion()
	{
		return (String)getProperty(VERSION_PROPERTY_NAME);
	}
	
	public String getPeerClassname()
	{
		return "archstudio.comp.cake.LayerableEndpointPeer";
	}
	
	public boolean hasOwnLayerType()
	{
		return false;
	}
}
