/*
 * Created on Jun 2, 2004
 *
 */
package archstudio.comp.cake;

import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Shape;

import edu.uci.ics.bna.BNAComponent;
import edu.uci.ics.bna.CoordinateMapper;
import edu.uci.ics.bna.FloatingLabelPeer;
import edu.uci.ics.bna.Thing;

/**
 * @author Chris
 *
 */
public class LayerableFloatingLabelPeer extends FloatingLabelPeer
{
	private LayerableFloatingLabel layerableFloatingLabel;
	
	public LayerableFloatingLabelPeer(BNAComponent c, Thing t)
	{
		super(c, t);
				
		if(!(t instanceof LayerableFloatingLabel))
		{
			throw new IllegalArgumentException("LayerableFloatingLabelPeer can only be a peer " +
				"for a LayerableFloatingLabel");
		}

		layerableFloatingLabel = (LayerableFloatingLabel)t;
	}
	
	public void draw(Graphics2D g, CoordinateMapper cm)
	{
		if(layerableFloatingLabel.isVisible())
		{
			super.draw(g, cm);				
			drawIndicatorShape(g, cm);
		}
	}

	protected void drawIndicatorShape(Graphics2D g, CoordinateMapper cm)
	{
 		// Draw indicator point with 50% glassmoded indicator shape
		Point ip = layerableFloatingLabel.getIndicatorPoint();
		if(ip != null)
		{
			Shape s = getIndicatorShape(cm);					
			g.setPaint(layerableFloatingLabel.getTrimColor());
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
			g.fill(s);
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.00f));
		}
	}
	
	protected Shape getIndicatorShape(CoordinateMapper cm)
	{		
		Point p = layerableFloatingLabel.getIndicatorPoint();
		if(p == null) return null;
		
		int ix = (int)p.getX();
		int iy = (int)p.getY();
		
		int x1 = layerableFloatingLabel.getX();
		int x2 = layerableFloatingLabel.getX2();
		
		if(x1 > x2){
			int temp = x1;
			x1 = x2;
			x2 = temp;
		}
		
		int y1 = layerableFloatingLabel.getY();
		int y2 = layerableFloatingLabel.getY2();
		
		if(y1 > y2){
			int temp = y1;
			y1 = y2;
			y2 = temp;
		}
		
		int cx = ((x2 - x1) / 2) + x1;
		int cy = ((y2 - y1) / 2) + y1;
		
		int ulX, ulY, urX, urY, llX, llY, lrX, lrY;
		
		ulX = cx - 5; if(ulX < x1) ulX = x1;
		ulY = cy - 5; if(ulY < y1) ulY = y1;
		
		urX = cx + 5; if(urX > x2) urX = x2;
		urY = cy - 5; if(urY < y1) urY = y1;
		
		llX = cx - 5; if(llX < x1) llX = x1;
		llY = cy + 5; if(llY > y2) llY = y2;
		
		lrX = cx + 5; if(lrX > x2) lrX = x2;
		lrY = cy + 5; if(lrY > y2) lrY = y2;
		
		//Now figure out which corner to move to the indicator point.
		if((ix < cx) && (iy < cy)){
			ulX = ix;
			ulY = iy;
		}
		else if((ix < cx) && (iy >= cy)){
			llX = ix;
			llY = iy;
		}
		else if((ix >= cx) && (iy < cy)){
			urX = ix;
			urY = iy;
		}
		else{
			lrX = ix;
			lrY = iy;
		}

		
		ulX = cm.worldXtoLocalX(ulX);
		urX = cm.worldXtoLocalX(urX);
		llX = cm.worldXtoLocalX(llX);
		lrX = cm.worldXtoLocalX(lrX);

		ulY = cm.worldYtoLocalY(ulY);
		urY = cm.worldYtoLocalY(urY);
		llY = cm.worldYtoLocalY(llY);
		lrY = cm.worldYtoLocalY(lrY);
		
		//Now make a shape.
		Polygon poly = new Polygon();
		poly.addPoint(ulX, ulY);
		poly.addPoint(urX, urY);
		poly.addPoint(lrX, lrY);
		poly.addPoint(llX, llY);
		
		return poly;
	}
}
