/*
 * Created on Jun 1, 2004
 *
 * LayeredNoThingLogic.java
 */
package archstudio.comp.cake;

import edu.uci.ics.bna.*;

import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.Iterator;

/**
 * @author Ping
 *
 * This class enhances the default NoThing logic such that the drag selection
 * won't accidentally select layered things that are not editable.
 */
public class LayeredNoThingLogic extends NoThingLogic
{

	protected ModeTrackingLogic modeTrackingLogic;
	
	public LayeredNoThingLogic(ModeTrackingLogic mtl)
	{
		this.modeTrackingLogic = mtl;
	}
	public void mousePressed(Thing t, MouseEvent evt, int worldX, int worldY)
	{
		if(modeTrackingLogic.getMode() == Palette.ADD_MODE || 
		   modeTrackingLogic.getMode() == Palette.SELECT_MODE)
		{
			super.mousePressed(t, evt, worldX, worldY);
		}
	}
	
	public void mouseReleased(Thing t, MouseEvent evt, int worldX, int worldY)
	{

		BNAComponent c = bnaComponent;
		if(evt.getButton() == MouseEvent.BUTTON1)
		{
			if(marqueeSelection != null){
				//System.out.println("Handle selection");
				int wx1 = marqueeSelection.getX1();
				int wx2 = marqueeSelection.getX2();
				int wy1 = marqueeSelection.getY1();
				int wy2 = marqueeSelection.getY2();
			
				if(wx1 > wx2)
				{
					int tmp = wx2;
					wx2 = wx1;
					wx1 = tmp;
				}
				if(wy1 > wy2)
				{
					int tmp = wy2;
					wy2 = wy1;
					wy1 = tmp;
				}
				c.getModel().removeThing(marqueeSelection);
			
				Rectangle selectRect = new Rectangle();
				selectRect.x = wx1;
				selectRect.y = wy1;
				selectRect.width = wx2 - wx1;
				selectRect.height = wy2 - wy1;
			
				if(!Utils.wasControlPressed(evt))
				{
					SelectionUtils.removeAllSelections(c);
				}
			
				for(Iterator it = c.getModel().getThingIterator(); it.hasNext(); )
				{
					Thing t2 = (Thing)it.next();
					if(t2 instanceof ILayerableThing)
					{
						ILayerableThing layerableThing = (ILayerableThing)t2;
						if(!layerableThing.isEditable())
						{
							// its not editable, just ignore it
							// so continue with the loop (next iteration)
							continue;
						}
					}
					if(t2 instanceof MarqueeSelectableThing)
					{
						Rectangle boundingBox = ((BoxBoundedThing)t2).getBoundingBox();
						if(Utils.isWithin(selectRect, boundingBox))
						{
							if(!Utils.wasControlPressed(evt))
							{
								((SelectableThing)t2).setSelected(true);
							}
							else
							{
								((SelectableThing)t2).setSelected(!((SelectableThing)t2).isSelected());
							}								
						}
					}
				}
				marqueeSelection = null;
			}
		}
	}
}
