/*
 * Created on Jun 7, 2004
 *
 * LayersMappingLogic.java
 */
package archstudio.comp.cake;

import java.util.Collection;
import java.util.Iterator;

import archstudio.comp.archipelago.ThingIDMap;
import edu.uci.ics.bna.*;
import edu.uci.ics.xarchutils.*;


/**
 * @author Ping
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class LayersMappingLogic extends CakeAbstractMappingLogic
{
	final protected String COMPONENT = "Component";
	final protected String CONNECTOR = "Connector";
	final protected String LINK = "Link";
	final protected String ANNOTATION = "Annotation";
	final protected String INTERFACE = "Interface";
	
	protected BNAModel bnaModel;
	protected ThingIDMap thingIDMap;
	protected InterfaceMap intfMap;
	protected CakeBrickMappingLogic brickMappingLogic;
	protected CakeLinkMappingLogic linkMappingLogic;
	protected CakeAnnotationMappingLogic annotationMappingLogic;
	
	protected static int offset = 0;
	
	public LayersMappingLogic(
	BNAComponent bnaComponent, XArchFlatInterface xarch, 
		ThingIDMap thingIDMap, InterfaceMap iMap, CakeBrickMappingLogic bml,
		CakeLinkMappingLogic lml, CakeAnnotationMappingLogic aml)
	{
		super(bnaComponent, xarch);
		//this.typesBNAComponent = bnaComponents[0];
	
		//this.typesBNAModel = bnaComponents[0].getModel();
		this.bnaModel = bnaComponent.getModel();
		this.thingIDMap = thingIDMap;
		this.intfMap = iMap;
		this.brickMappingLogic = bml;
		this.linkMappingLogic = lml;
		this.annotationMappingLogic = aml;
		//linkParent = new NoThing("$$LinkParent");
		//structureBNAModel.addThing(linkParent);
	}
	
	public void handleXArchFlatEvent(XArchFlatEvent evt)
	{
		
		//System.out.println("Handling events! Here");
		try
		{
		if(!evt.getIsAttached())
		{
			// no need to handle these yet (I think)
			return;
		}
		//System.out.println(evt);
		ObjRef src = evt.getSource();
		ObjRef docRef = xarch.getXArch(src);
		int evtType = evt.getEventType();
		XArchPath path = evt.getTargetPath();
		
		
		if(path == null || evtType == XArchFlatEvent.REMOVE_EVENT)
		{
			// Remove event the target path is just the element
			// so we want the source path instead.
			path = evt.getSourcePath();
		}
		if(path == null)
		{
			//System.out.println("Null again");
			return;
		}
		
		String pathString = path.toTagsOnlyString();
		
		//System.out.println(evt);
		//System.out.println("pathString = " + pathString);
		if(evtType == XArchFlatEvent.REMOVE_EVENT &&
		   pathString.startsWith("xArch/archLayers"))
		{
			String targetPath = evt.getTargetPath().toTagsOnlyString();
			ObjRef target;
			if(targetPath.equals("containingElement"))
			{
				target = (ObjRef)evt.getTarget();
				// we are removing the containing element
				//System.out.println("Removing a containing Element");
				removeContainingElement(target, docRef);
			}
			else if(pathString.startsWith("xArch/archLayers/layerSet"))
			{
				target = (ObjRef)evt.getTarget();
				//we are removing particular layers
				//System.out.println("Removing a layer");
				removeLayer(target, docRef);
			}
			else if(targetPath.equals("layerSet"))
			{
				target = (ObjRef)evt.getTarget();
				// we are removing layer sets
				//System.out.println("Removing a layer set");
				removeLayerSet(target, docRef);
			}
		}
		else if(pathString.endsWith("name"))
		{
			//System.out.println(evt.getSourcePath().toTagsOnlyString());
			// updating the name of the layers.
			ObjRef[] ancestors = xarch.getAllAncestors(src);
			// the last one is xArch, then ArchLayer, layerSet, then layer
			ObjRef layerRef = ancestors[ancestors.length - 4];
			updateLayerName(layerRef);
			
		}
		else if(pathString.endsWith("containingElement") &&
		        evtType == XArchFlatEvent.SET_EVENT)
		{
			//System.out.println("pathString = " + pathString);
			// we are only interested in them setting the href
			// if they are clearing it, we can't do much right now...
			// nothing to do with the BNA thing...
			if(evt.getTargetName().equals("href"))
			{
				if(pathString.lastIndexOf("componentLayer") != -1 ||
				   pathString.lastIndexOf("connectorLayer") != -1)
			    {
			    	// its a change in a component or connector layer's containing element
			   		brickMappingLogic.updateElementThing(src, docRef);
			    }
			    else if(pathString.lastIndexOf("linkLayer") != -1)
			    {
			    	//System.out.println("Here!!!");
			    	linkMappingLogic.updateLinkThing(src, docRef);
			    }
			    else if(pathString.lastIndexOf("annotationLayer") != -1)
			    {
			    	annotationMappingLogic.updateAnnotation(src);
			    }
			}
		}
		
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		/*
		
		if(pathString.startsWith("xArch/archStructure/link"))
		{
			//System.out.println("Link");
			//It's a component event
			if(src != null)
			{
				ObjRef[] ancestors = xarch.getAllAncestors(src);
				// 3 is where the actual link is...
				ObjRef linkRef = ancestors[ancestors.length - 3];
				//System.out.println("Updating Link");
				updateLink(linkRef, docRef);
			}
		}
		*/
	}
	/**
	 * This function updates the layer name of all the elements in this layer.
	 * @param layer
	 */
	protected void updateLayerName(ObjRef layer)
	{
		String name = "";
		ObjRef nameRef = (ObjRef)xarch.get(layer, "Name");
		if(nameRef != null)
		{
			String nameVal = (String)xarch.get(nameRef, "Value");
			if(nameVal != null)
				name = nameVal;
		}
		ObjRef[] elements = xarch.getAll(layer, "ContainingElement");
		if(elements != null)
		{
			for(int i = 0; i < elements.length; i++)
			{
				String thingID = thingIDMap.getThingID(elements[i]);
				if(thingID != null)
				{
					ILayerableThing layeredThing = (ILayerableThing)bnaModel.getThing(thingID);
					if(layeredThing != null)
					{
						layeredThing.setLayer(name);
					}
				}
			} 
		}
	}
	
	protected void removeLayerSet(ObjRef layerSet, ObjRef docRef)
	{
		// Now delete all the internal layers (if it has any)
		ObjRef layers[] = xarch.getAll(layerSet, "ComponentLayer");
		if(layers != null)
			removeLayers(layers, docRef);
	
		//System.out.println("Connector");
		layers = xarch.getAll(layerSet, "ConnectorLayer");
		if(layers != null)
			removeLayers(layers, docRef);
	
		//System.out.println("Link");
		layers = xarch.getAll(layerSet, "LinkLayer");
		if(layers != null)
			removeLayers(layers, docRef);
	
		//System.out.println("Ann");
		layers = xarch.getAll(layerSet, "AnnotationLayer");
		if(layers != null)
			removeLayers(layers, docRef);
	}
	
	protected void removeLayers(ObjRef[] layers, ObjRef docRef)
	{
		for(int i = 0; i < layers.length; i++)
		{
			removeLayer(layers[i], docRef);
		}
	}
	
	/**
	 * This function removes all the things within the layer passed in
	 * @param layer
	 * @param docRef
	 */
	protected void removeLayer(ObjRef layer, ObjRef docRef)
	{
		ObjRef[] elements = xarch.getAll(layer, "ContainingElement");
		if(elements != null)
		{	
			for(int i = 0; i < elements.length; i++)
			{
				removeContainingElement(elements[i], docRef);
			}
		}
	}
	/**
	 * This function removes the THing associated with the containingElement
	 * that was passed in.
	 * @param containingElement
	 * @param docRef
	 */
	protected void removeContainingElement(ObjRef containingElement, ObjRef docRef)
	{
		// this containing element was removed, lets update all the annotations
		// to make sure nothing is pointing at us.
		ObjRef[] references = xarch.getReferences(docRef, 
			(String) xarch.get(containingElement, "Id"));
		//System.out.println(references.length);
		for(int i = 0; i < references.length; i++)
		{
			// find all the references (annotations) and clear their href
			ObjRef parent = xarch.getParent(references[i]);
			if(xarch.isInstanceOf(parent, "edu.uci.isr.xarch.annotations.IAnnotation"))
			{
				xarch.clear(references[i], "Href");
			}
		}
		// now find out what the containing element was pointing at
		ObjRef element = CakeUtils.resolveXMLLink(xarch, docRef, containingElement);	
		//	clear the references since this one is being removed
		xarch.clear(containingElement, "Href");
		if(element != null)
		{
			// now if the thing that was being removed is not used else where
			// remove him too
			ObjRef[] refs = xarch.getReferences(docRef, 
				(String)xarch.get(element, "Id"));
			ObjRef parent;
			//System.out.println("Reference length " + refs.length);
			if(hasReferences(refs))
			{
				String type = getElementType(element);
				// that thing is not used else where, he is gone too
				parent = xarch.getParent(element);
				xarch.remove(parent, type, element);
			}
		}
		 
		// remove any interfaces that it may have
		Collection c = intfMap.getAllInterfaceThingIDs(containingElement);
		if(c != null)
		{
			Iterator it = c.iterator();
			String id;
			while(it.hasNext())
			{
				id = (String)it.next();
				bnaModel.removeThing(id);
				//CakeInterfaceThing intfThing = (CakeInterfaceThing)bnaModel.getThing(id);
				//bnaModel.removeThing(intfThing);
			}
			intfMap.removeInterfacePairs(containingElement);
		}

		// now remove the thing
		String thingID = thingIDMap.getThingID(containingElement);
		//System.out.println("Removing thing" + thingID);
		// remove it from the model
		bnaModel.removeThing(thingID);
	}
	
	
	/**
	 * This function determines that if an element should be deleted
	 * based on the list of references passed in thats pointed to it.
	 * @param refs
	 * @return
	 */
	private boolean hasReferences(ObjRef[] refs)
	{
		if(refs != null && refs.length != 0)
		{
			/*
			for(int i = 0; i < refs.length; i++)
			{
				ObjRef parent = xarch.getParent(refs[i]);
				if(!xarch.isInstanceOf(parent,
					"edu.uci.isr.xarch.annotations.IAnnotation"))
				{
					// I have found a link thats not from an annotation,
					// so don't delete this element
					return false;
				}
			}*/
			return false;
		}
		return true;
	}

	/**
	 * This function returns the type of the element
	 * @param e The element
	 * @return "Component" for components, etc
	 */
	protected String getElementType(ObjRef e)
	{
		String type = null;
		
		if(xarch.isInstanceOf(e, "edu.uci.isr.xarch.types.IComponent"))
		{
			type = COMPONENT;
		}
		else if(xarch.isInstanceOf(e, "edu.uci.isr.xarch.types.IConnector"))
		{
			type = CONNECTOR;
		}
		else if(xarch.isInstanceOf(e, "edu.uci.isr.xarch.types.ILink"))
		{
			type = LINK;
		}
		else if(xarch.isInstanceOf(e, "edu.uci.isr.xarch.annotations.IAnnotation"))
		{
			type = ANNOTATION;
		}
		else if(xarch.isInstanceOf(e, "edu.uci.isr.xarch.types.IInterface"))
		{
			type = INTERFACE;
		}
		
		//System.out.println(type);
		return type;
	}
}
