package archstudio.comp.cake;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import edu.uci.ics.bna.swingthing.*;

/**
 * 
 * @author Ping
 *
 * This class implements a basic palette to allow the user different modes
 * of interacting with elements within Cake.  The 3 modes are Add, Select, and Remove.
 * 
 */
public class Palette extends SwingPanelThing implements IOptionalScroll
{
	PalettePanel panel;
	public static final String MODE_PROPERTY_NAME = "mode";
	public static final int ADD_MODE = 0;
	public static final int SELECT_MODE = 1;
	public static final int REMOVE_MODE = 2;
	
	public Palette(){
		super();
		panel = new PalettePanel(this);
		setPanel(panel);
		setMode(ADD_MODE);
		setOptionalScroll(false);
		setWidth(84);
		setHeight(58);
	}
	
	//Sets the palette's mode to the number specified
	public void setMode(int mode)
	{
		setProperty(MODE_PROPERTY_NAME, mode);
	}
	
	public int getMode()
	{
		return getIntProperty(MODE_PROPERTY_NAME);
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.IOptionalScroll#setOptionalScroll(boolean)
	 */
	public void setOptionalScroll(boolean optionalScroll)
	{
		setProperty(OPTIONAL_SCROLL_PROPERTY_NAME, optionalScroll);
	}

	/* (non-Javadoc)
	 * @see archstudio.comp.cake.IOptionalScroll#getOptionalScroll()
	 */
	public boolean getOptionalScroll()
	{
		return getBooleanProperty(OPTIONAL_SCROLL_PROPERTY_NAME);
	}
	
}

class PalettePanel extends JPanel implements ActionListener
{
	protected Palette palette;
	
	//The buttons
	protected JButton addModeButton;
	protected JButton selectModeButton;
	protected JButton removeModeButton;

	private ImageIcon addIcon;
	private ImageIcon addIconSelected;
	private ImageIcon selectIcon;
	private ImageIcon selectIconSelected;
	private ImageIcon removeIcon;
	private ImageIcon removeIconSelected;
	
	public PalettePanel(Palette palette)
	{
		
		this.palette = palette;
		
		setLayout(new BorderLayout());
		
		//WindowHeaderPanel stuff
		PaletteWindowHeaderPanel header = new PaletteWindowHeaderPanel("Palette");
		header.setPreferredSize(new Dimension(84, 28));			
				
		//IconPanel stuff
		JPanel iconPanel = new JPanel();
		iconPanel.setLayout(new GridLayout(1,3));
		addIcon = createImageIcon("\\img\\palette_add.gif");
		addIconSelected = createImageIcon("\\img\\palette_add_selected.gif");
		selectIcon = createImageIcon("\\img\\palette_select.gif");
		selectIconSelected = createImageIcon("\\img\\palette_select_selected.gif");
		removeIcon = createImageIcon("\\img\\palette_remove.gif");
		removeIconSelected = createImageIcon("\\img\\palette_remove_selected.gif");
			
		addModeButton = new JButton(addIconSelected);
		addModeButton.addActionListener(this);
		addModeButton.setPreferredSize(new Dimension(26, 26));
		
		selectModeButton = new JButton(selectIcon);
		selectModeButton.addActionListener(this);
		selectModeButton.setPreferredSize(new Dimension(26, 26));
		
		removeModeButton = new JButton(removeIcon);
		removeModeButton.addActionListener(this);
		removeModeButton.setPreferredSize(new Dimension(26, 26));
		
		iconPanel.add(addModeButton);
		iconPanel.add(selectModeButton);
		iconPanel.add(removeModeButton);
		iconPanel.setPreferredSize(new Dimension(84, 30));
		
		this.add(header, BorderLayout.NORTH);
		this.add(iconPanel, BorderLayout.CENTER);
		
		palette.setPanel(this);

	}
	
	//Detects which button was hit, changes mode as appropriate
	public void actionPerformed(ActionEvent evt){
		if(evt.getSource() == addModeButton){
			palette.setMode(Palette.ADD_MODE);
			addModeButton.setIcon(addIconSelected);
			selectModeButton.setIcon(selectIcon);
			removeModeButton.setIcon(removeIcon);
			return;
		}
		if(evt.getSource() == selectModeButton){
			palette.setMode(Palette.SELECT_MODE);
			addModeButton.setIcon(addIcon);
			selectModeButton.setIcon(selectIconSelected);
			removeModeButton.setIcon(removeIcon);
			return;
		}
		if(evt.getSource() == removeModeButton){
			palette.setMode(Palette.REMOVE_MODE);
			addModeButton.setIcon(addIcon);
			selectModeButton.setIcon(selectIcon);
			removeModeButton.setIcon(removeIconSelected);
			return;
		}
	}
	
	protected ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = CheckNodeTreeExample.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}
}