/*
 * Created on May 29, 2004
 *
 * TreeLogicAdapter.java
 */
package archstudio.comp.cake;

import java.util.*;
//import java.util.Iterator;

import edu.uci.ics.bna.*;
import edu.uci.ics.xarchutils.*;
import archstudio.comp.archipelago.ThingIDMap;

/**
 * @author Ping
 *
 * This is the tree logic that will handle all the check box stuff
 * setting whether a layer is visible and selectable or not.
 */
public class TreeLogic extends CakeThingLogicAdapter 
{
	protected XArchFlatInterface xarch;
	protected ThingIDMap thingIDMap;
	protected InterfaceMap iMap;
	protected ObjRef docRef;
	
	public TreeLogic(XArchFlatInterface xarch, ThingIDMap idMap, 
		InterfaceMap intfMap, ObjRef docRef)
	{
		this.xarch = xarch;
		this.thingIDMap = idMap;
		this.iMap = intfMap;
		this.docRef = docRef;
	}
	
	/**
	 * This function sets all the things on the specified layer as editable 
	 * or not
	 * @param layerID The layer that must be changed
	 * @param v True if the layer is selectable
	 */
	public void setEditable(String layerID, boolean isEditable)
	{
		//System.out.println("SetEditable");
		ObjRef layer = xarch.getByID(docRef, layerID);
		if(layer != null && isLayer(layer))
		{
			final BNAComponent bnaComp = getBNAComponent();
			if(bnaComp != null)
			{
				final BNAModel bnaModel = bnaComp.getModel();
				
				ObjRef[] elements = xarch.getAll(layer, "ContainingElement");
				//System.out.println("Elements length " + elements.length);
				for(int i = 0; i < elements.length; i++)
				{
					//ObjRef element = Utils.resolveXMLLink(xarch, docRef, elements[i]);
					//String existingThingID;
					String existingThingID = thingIDMap.getThingID(elements[i]);
					if(existingThingID != null)
					{
						setIsEditable(bnaModel, existingThingID, isEditable);
					}
					
					Collection interfaces = iMap.getAllInterfaceThingIDs(elements[i]);
					//System.out.println(interfaces.size());
					// get its interfaces and set editability on those
					if(interfaces != null)
					{
						Iterator it = interfaces.iterator();
						String intfID;
						while(it.hasNext())
						{
							
							// set the editability of all its interfaces
							intfID = (String)it.next();
							//System.out.println(intfID + " still here..");
							setIsEditable(bnaModel, intfID, isEditable);
						}
					}
				}
			}
		}
	}	
	
	/**
	 * This function sets the editability of the Thing in the BNA Model 
	 * corresponding to the ObjRef.
	 * 
	 * @param bnaModel The BNA model that the Thing belongs to
	 * @param element The element we are changing the editability of
	 * @param isEditable True if it should be editable
	 */
	private void setIsEditable(final BNAModel bnaModel, String thingID,
		boolean isEditable)
	{
		if(thingID != null)
		{
			try
			{
				Thing t = bnaModel.getThing(thingID);
				if(t != null)
				{
				
					ILayerableThing layeredThing = (ILayerableThing)t;
					layeredThing.setEditable(isEditable);
					if(!isEditable)
					{
						// unselect it if its no longer editable
						unselectThing(t);
					}		
				}		
			}
			catch(ClassCastException cce)
			{
				System.err.println("Warning; ID/type mismatch.");
				cce.printStackTrace();
				//return;
			}
		}
	}

	
	
	/**
	 * This function sets all the things on the specified layer as visible 
	 * or not
	 * @param layerID The layer that must be changed
	 * @param isSelectable True if the layer is visible
	 */	
	public void setVisible(String layerID, boolean isVisible)
	{
		//System.out.println("Setting visibility");
		ObjRef layer = xarch.getByID(docRef, layerID);
		if(layer != null && isLayer(layer))
		{
			//System.out.println("Is a layer");
			final BNAComponent bnaComp = getBNAComponent();
			if(bnaComp != null)
			{
				final BNAModel bnaModel = bnaComp.getModel();
				
				ObjRef[] elements = xarch.getAll(layer, "ContainingElement");
				
				for(int i = 0; i < elements.length; i++)
				{
					//ObjRef element = Utils.resolveXMLLink(xarch, docRef, elements[i]);					
										
					String existingThingID = thingIDMap.getThingID(elements[i]);
					if(existingThingID != null)
					{
						setIsVisible(bnaModel, existingThingID, isVisible);
					}
					
					Collection interfaces = iMap.getAllInterfaceThingIDs(elements[i]);
					// get its interfaces and set editability on those
					if(interfaces != null)
					{
						Iterator it = interfaces.iterator();
						String intfID;
						while(it.hasNext())
						{
							// set the editability of all its interfaces
							intfID = (String)it.next();
							setIsVisible(bnaModel, intfID, isVisible);
						}
					}
				}
			}
		}
	}
	
	/**
	 * This function sets the visibility of the Thing in the BNA Model 
	 * corresponding to the ObjRef.
	 * 
	 * @param bnaModel The BNA model that the Thing belongs to
	 * @param element The element we are changing the visibility of
	 * @param isVisible True if it should be visible
	 */
	private void setIsVisible(final BNAModel bnaModel, String thingID,
		boolean isVisible)
	{
		if(thingID != null)
		{
			try
			{
				Thing t = bnaModel.getThing(thingID);
				if(t != null)
				{
					ILayerableThing layeredThing = (ILayerableThing)t;
					layeredThing.setVisible(isVisible);
				}
				
				/*
				if(!isVisible)
				{
					// if its no longer visible,
					unselectThing(t);
				}
				*/
				//System.out.println("Setting visible " + isVisible);
			}
			catch(ClassCastException cce)
			{
				System.err.println("Warning; ID/type mismatch.");
				cce.printStackTrace();
				//return;
			}
		}
	}
	
	/**
	 * This function determines if the obj ref passed in corresponds to some layer
	 * 
	 * @param layer The objref to check
	 * @return true if its a layer element
	 */
	private boolean isLayer(ObjRef layer)
	{
		return xarch.isInstanceOf(layer, "edu.uci.isr.xarch.layers.IComponentLayer") ||
			   xarch.isInstanceOf(layer, "edu.uci.isr.xarch.layers.IConnectorLayer") ||
			   xarch.isInstanceOf(layer, "edu.uci.isr.xarch.layers.ILinkLayer")      ||
			   xarch.isInstanceOf(layer, "edu.uci.isr.xarch.layers.IAnnotationLayer");
	}
	
	
	private boolean hasInterfaces(ObjRef element)
	{
		return xarch.isInstanceOf(element, "edu.uci.isr.xarch.types.IComponent") ||
			   xarch.isInstanceOf(element, "edu.uci.isr.xarch.types.IConnector");
	}
	/**
	 * This function checks to see if the thing passed in is a SelectableThing
	 * And if it is, it will unselect it.
	 * @param t
	 */
	private void unselectThing(Thing t)
	{
		// if the thing was selectable, unselect it
		if(t instanceof SelectableThing)
		{
			SelectableThing selectableThing = (SelectableThing)t;
			if(selectableThing.isSelected())
			{
				selectableThing.setSelected(false);
			}
		}
	}
}
