/*
 * Created on Jun 1, 2004
 *
 */
package archstudio.comp.cake;

import java.awt.Color;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

/**
 * An object that maps version numbers to color values.
 * 
 * @author Chris 
 */
public class VersionColorMap
{
	/** Maps version numbers to color values. */
	protected Map colorMap;
	protected Random rand;

	/**
	 * Constructs a new VersionColorMap. 
	 */
	public VersionColorMap()
	{
		colorMap = Collections.synchronizedMap(new HashMap());
		rand = new Random();
		initColorMap();
	}

	/**
	 * Returns the color associated with a particular version number. 
	 * @param versionNumber The version number whose associated color is to be returned.
	 * @return The <color>Color</code> to which the provided version number is mapped. Note: this method guarantees
	 * 		that a mapping is provided so a valid <color>Color</code> will always be returned.
	 */
	public Color getColor(String versionNumber)
	{				
		Color color = (Color)colorMap.get(versionNumber);
		
		if(color == null)
		{
			// Create new color value to associate with this version
			color = generateColor();
			colorMap.put(versionNumber, color);
		}
		
		return color;
	}

	/**
	 * Generates an "acceptable" color; i.e., is not too dark.
	 * @return
	 */
	private Color generateColor()
	{
		Color color;		
		int red, green, blue;
		red = green = blue = 0;
		
		// Generates R,G,B values that range between 100 and 255
		red = 100 + Math.abs(rand.nextInt() % 156);
		green = 100 + Math.abs(rand.nextInt() % 156);
		blue = 100 + Math.abs(rand.nextInt() % 156);					
		
		color = new Color(red, green, blue);
		return color;
	}
	
	private void initColorMap()
	{
		// Initialize the color map with a few initial "safe" colors.
		// TODO: This is a hack and should be fixed.  We should never assume that the keys will look
		// exactly like "1", "2", etc.
		colorMap.put("1", Color.GREEN);
		colorMap.put("2", Color.CYAN);
		colorMap.put("3", Color.RED);
		colorMap.put("4", Color.BLUE);
		colorMap.put("5", Color.LIGHT_GRAY);
		colorMap.put("6", Color.ORANGE);
		colorMap.put("7", Color.MAGENTA);
		colorMap.put("8", Color.YELLOW);
	}
}
