/*
 * Created on May 23, 2004
 *
 * CakeModelListener.java
 */
package archstudio.comp.cake;

import edu.uci.ics.xarchutils.*;
import edu.uci.ics.xadlutils.*;
import edu.uci.ics.bna.*;
/**
 * @author Ping
 *
 * This class provides a mapping between the xADL layer and layerSets
 * to the BNA world including the version tree and BNA Things.
 * It mainly handles the addition and removal of layerSets and layers
 * 
 * @see edu.uci.ics.xarchutils.XArchFlatListener
 */
public class VersionTreeMappingLogic extends CakeAbstractMappingLogic
{
	//private XArchFlatInterface xarch;
	protected CheckNodeTreeWrapper treeModel;
	
	VersionTreeMappingLogic(BNAComponent bnaComponent, 
		XArchFlatInterface xArch, CheckNodeTreeWrapper cakeTreeModel)
	{
		super(bnaComponent, xArch);
		treeModel = cakeTreeModel;
	}
	
	/**
	 * This function is called initially to populate the tree model according
	 * to the layers and layerSets in the document passed in
	 * @param docRef The ObjRef to the document
	 */
	public void populateTreeModel(ObjRef docRef)
	{
		ObjRef layersContext = xarch.createContext(docRef, "Layers");
		ObjRef archLayerRef = xarch.getElement(layersContext, "ArchLayers", docRef);
		
		if(archLayerRef != null)
		{
			ObjRef[] layerSets = xarch.getAll(archLayerRef, "LayerSet");
			
			if(layerSets != null && layerSets.length > 0)
			{
				for(int i = 0; i < layerSets.length; i++)
				{
					addLayerSet(layerSets[i]);
				}
			}
		}
	}
	
	/**
	 * This file determines the type of event we received and makes
	 * the appropriate changes to the tree model such as
	 * add a version, add a layer, removing a version, or removing a layer.
	 * 
	 * @param evt The xArch event
	 */
	public void handleXArchFlatEvent(XArchFlatEvent evt)
	{
		//System.out.println(evt);
		/*
		try
		{
		*/
		// First figure out of its an arch layer event
		XArchPath path = evt.getSourcePath();
		XArchPath targetPath = evt.getTargetPath();
		if( path != null && evt.getIsAttached())
		{
			String pathString;
			if(targetPath != null)
			{
				pathString = targetPath.toTagsOnlyString();
				if(pathString.startsWith(
					"xArch/archLayers/layerSet/versionGraphNode"))
				{
					// we don't care about versionGraphNode events
					return;
				}
			}
			if(evt.getSourceType() == XArchFlatEvent.ATTRIBUTE_CHANGED)
			{
				// we don't care about attribute changed events here
				return;
			}
					
			pathString = path.toTagsOnlyString();
			// Make sure that the source is within layer set -
			// this means that the changes are made to the internal layers
			if(pathString.startsWith("xArch/archLayers/layerSet"))
		   	{
		   		handleLayerEvents(evt);
		   	}
		   	// this is a change within the archLayers directly,
		   	// so should be addition and removal of layer sets (versions)
		   	else if(pathString.startsWith("xArch/archLayers"))
		   	{
		   		handleLayerSetEvents(evt);
		   	}
		}
		/*
		}
		catch(Exception e)
		{
			System.out.println("*****" + e);
			e.printStackTrace();
		}
		*/
	}

	/**
	 * This function handles the addition and removal of layerSets
	 * @param evt The xArch event
	 */
	private void handleLayerSetEvents(XArchFlatEvent evt)
	{
		int evtType = evt.getEventType();
		
		switch(evtType)
		{
			case XArchFlatEvent.ADD_EVENT:
			{
				ObjRef layerSet = (ObjRef)evt.getTarget();
				if(layerSet != null)
				{
					addLayerSet(layerSet);
				}
			}
			break;
			
			case XArchFlatEvent.REMOVE_EVENT:
			{
				ObjRef layerSet = (ObjRef)evt.getTarget();
				if(layerSet != null)
				{
					String id = XadlUtils.getID(xarch, layerSet);
					if(id != null)
					{
						treeModel.removeVersion(id);
					}
				}
			}
			break;
		}
	}

	
	/**
	 * This function handles the changes to a particular layer
	 * within a layerSet.  This includes addition & removal of layers
	 * and changes to the name of a layer ONLY.
	 * @param evt The xArch event
	 */
	private void handleLayerEvents(XArchFlatEvent evt)
	{
		XArchPath path = evt.getTargetPath();
		
		if(path == null)
		{
			path = evt.getSourcePath();
		}	
		// we assume IDs and simple type values (attributes) won't change
		// so all the events we care about will have a target path
		if(path != null)
		{
			String pathString = path.toTagsOnlyString();
			// we ignore events dealing with the complex element 
			// - containingElement
			if(pathString.lastIndexOf("containingElement") != -1)
			{
				return;
			}
			int evtType = evt.getEventType();
			switch(evtType)
			{
				case XArchFlatEvent.ADD_EVENT:
				{
					// we are adding a layer
					// get the layer that was added
					ObjRef layer = (ObjRef)evt.getTarget();
					ObjRef layerSet = evt.getSource();
					addLayer(layer, layerSet);
				}
				break;
				
				case XArchFlatEvent.REMOVE_EVENT:
				{
					//System.out.println("Got to remove!");
					// we are removing a layer
					ObjRef layer = (ObjRef)evt.getTarget();
					
					String id = XadlUtils.getID(xarch, layer);
					if(id != null)
						treeModel.removeLayer(id);
				}
				break;
					
				case XArchFlatEvent.CLEAR_EVENT:
				{
					//System.out.println("Got clear event");
					ObjRef src = evt.getSource();
					if(src != null)
					{
						ObjRef[] ancestors = xarch.getAllAncestors(src);
						// the layer will be at length - 4 (xArch/ArchLayer/LayerSet/layer
						ObjRef layer = ancestors[ancestors.length - 4];
						
						if(layer != null)
						{
						
							String id = XadlUtils.getID(xarch, layer);
							if(id == null)
							{
								// shoulnd't happen
								id = "";
							}
							String currentName = treeModel.getName(id);
							// if the current name is not empty, we clear it
							if(!currentName.equals(""))
							{
								treeModel.setName("", id);
							}
						}
					}					
				}
				break;
				case XArchFlatEvent.SET_EVENT:
				{
					int sourceType = evt.getSourceType();
					
					if(sourceType == XArchFlatEvent.SIMPLE_TYPE_VALUE_CHANGED)
					{
						ObjRef src = evt.getSource();
						if(src != null)
						{
							ObjRef[] ancestors = xarch.getAllAncestors(src);
							// the layer will be at length - 4 (xArch/ArchLayer/LayerSet/layer
							ObjRef layer = ancestors[ancestors.length - 4];							
							if(layer != null)
							{
								String id = XadlUtils.getID(xarch, layer);
								if(id == null)
								{
									// shoulnd't happen
									id = "";
								}
								String currentName = treeModel.getName(id);
								String newName = (String)evt.getTarget();
								// if the current name is not the same as the new name
								if(!currentName.equals(newName))
								{
									treeModel.setName(newName, id);
								}
							}
						}
					}		
				}
				break;
					
			}
		}
		
	}
	/**
	 * This function adds the layer set and all its internal layers.
	 * @param layerSet The layerSet to add
	 */
	private void addLayerSet(ObjRef layerSet)
	{
		String id = XadlUtils.getID(xarch, layerSet);
	
		if(id == null)
		{
			id = "";
		}
		treeModel.addVersion("Version " + id, id);
	
		// Now add all the internal layers (if it has any)
		ObjRef layers[] = xarch.getAll(layerSet, "ComponentLayer");

		addLayers(layers, layerSet);
	
		//System.out.println("Connector");
		layers = xarch.getAll(layerSet, "ConnectorLayer");
		addLayers(layers, layerSet);
	
		//System.out.println("Link");
		layers = xarch.getAll(layerSet, "LinkLayer");
		addLayers(layers, layerSet);
	
		//System.out.println("Ann");
		layers = xarch.getAll(layerSet, "AnnotationLayer");
		addLayers(layers, layerSet);
	}
	
	/**
	 * This function adds a set of layers
	 * 
	 * @param layers The array of layers we are adding
	 * @param layerSet The layerSet that contains all the layers
	 */
	private void addLayers(ObjRef[] layers, ObjRef layerSet)
	{
		if(layers != null)
		{
		
			//System.out.println("Adding set of layers!");
			for(int i = 0; i < layers.length; i++)
			{
				//System.out.println(i);
				addLayer(layers[i], layerSet);
			}
		}
	}
	
	/**
	 * This function extracts all the information and adds the layer to the
	 * tree model.
	 * @param layer This is the layer that should be added
	 * @param layerSet This is the layerSet (version) that the layer belongs to
	 */
	private void addLayer(ObjRef layer, ObjRef layerSet)
	{
		//System.out.println("Adding layer");
		
		// first extract the ID
		//String id = XadlUtils.getID(xarch, layer);
		//System.out.println("layer type is: " + xarch.getType(layer));
		String id = (String)xarch.get(layer, "Id");
		//System.out.println("In Ping's AddLayer after getID, id is: " + id);
		//System.out.println("layer ref is: " + layer);
		if(id == null)
			id = "";
		//System.out.println("In Ping's AddLayer after assignment, id is: " + id);
		// now extract the name
		String name = null;
		ObjRef nameRef = (ObjRef)xarch.get(layer, "Name");
		if(nameRef != null)
		{
			name = (String)xarch.get(nameRef, "Value");
		}		
		if(name == null)
			name = "";
		
		// now extract the version ID which is just the
		// ID of the layerSet that this belongs to.
		String versionID = XadlUtils.getID(xarch, layerSet);
		if(versionID == null)
			versionID = "";
		// get the type of layer it is
		int layerType = getLayerType(layer);
		
		treeModel.addLayer(name, id, layerType, versionID);
	}
	
	/**
	 * This function determines the type of the layer that is passed in
	 * 
	 * @param layer The layer in question
	 * @return a constant specified in ICakeModelWrapper
	 * @see archstudio.comp.cake.ICakeModelWrapper
	 */
	private int getLayerType(ObjRef layer)
	{
		int layerType = 0;
		if(xarch.isInstanceOf(layer, "edu.uci.isr.xarch.layers.IComponentLayer"))
		{
			layerType = ICakeModelWrapper.COMPONENT_LAYER;
		}
		else if(xarch.isInstanceOf(layer, "edu.uci.isr.xarch.layers.IConnectorLayer"))
		{
			layerType = ICakeModelWrapper.CONNECTOR_LAYER;
		}
		else if(xarch.isInstanceOf(layer, "edu.uci.isr.xarch.layers.ILinkLayer"))
		{
			layerType = ICakeModelWrapper.LINK_LAYER;
		}
		else if(xarch.isInstanceOf(layer, "edu.uci.isr.xarch.layers.IAnnotationLayer"))
		{
			layerType = ICakeModelWrapper.ANNOTATION_LAYER;
		}
		return layerType;	
	}

}
